# %W% %G% %U% - %Q% #
#ident "@(#)VCSOR:%M% %I%"
#
# $Copyright: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

package netlsnrlib;
use strict;
use lib qw(.);
use netconfig () ;
use vars qw(%CONF);
*CONF =\%netconfig::CONF;
my $CAT = $CONF{CAT};
my $SU = $CONF{SU};
my $PS = $CONF{PS};
my $LIBRARY_PATH = $CONF{LIBRARY_PATH};

################# FUNCTION DEFN START ##########################
# Function : check_oracle_home
# Desc     : Checks for inexistence of ORACLE_HOME
# Input    : $Home
# Output   : None
# Returns  : (1) -  home does not exist
#

sub check_oracle_home  {
# Bail if the Oracle home directory doesn't exist 
#
my ($Home) =@_;
	unless ( -d $Home ) {
		return  1;
	}
	return 0;
}
# Function : check_oracle_client
# Desc     : Checks if svrmgrl/sqlplus does not exist .
# Desc     : If they exist , this function sets DBAMGR and CONNECTSTR
# Input    : $Home
# Output   : $LSNRMGR - changes values of these variables
# Returns  : (1) - no suitable client exists
#
sub check_oracle_client {
my ($Home,$LSNRMGR)=@_;

	if ( -x "$Home/bin/lsnrctl" ) {
		$LSNRMGR = "$Home/bin/lsnrctl";
		$_[1] = $LSNRMGR;
	} else {
		return 1 ;
	}
	return 0;
}

# Function : set_environment
# Desc     : Checks the EnvFile, if not valide sets $ValidFile to 1
# Desc     : Else sets the Oracle related env vars
# Input    : $UseCSH - shell of the owner ( 1 if csh)
# Input    : $EnvFile - source file for env vars
# Input    : $Owner
# Input    : $Home
# Output   : $ValidFile -
# Returns  :  $LD_ENV - Env vars in format to be set
# Note     :  This function sets LD_LIBRARY_PATH ( and equivalents)
#          :  TWO_TASK, extra env vars
#
sub set_environment {
my ($UseCSH,$EnvFile, $Owner,$Home,$ValidFile) =@_;
	my $EnvFileExists = 0;
	my $caterr = "";
	
	if ( $EnvFile ) {
		
		$_[4] = 0 ;     #is a valid file
		#
		# It is the user's responsibility to ensure that EnvFile can
		# be sourced successfully using the login shell of $User. 
		#
		# First ensure that the file is a Plain file AND a Text file.
		# Other file types (e.g. Named pipe) etc. cause $EnvFile to be
		# NOT sourced, with a message in the engine log saying so.
		#
		if ( -f $EnvFile && -T $EnvFile ) {
			#
			# Doing a 'cat' on the file as user '$Owner' ensures that the 
			# permission are correct for user '$Owner'.
			# Execute permissions are NOT needed for $EnvFile to source it.
			#
			# Notes:
			#   * 2>&1 is for the su and NOT for cat.
			#   * `` always invokes sh.
			#   * > /dev/null is for the cat command.
			#   * The cat command syntax used is shell-neutral.
			#
			$caterr = `$SU $Owner -c "$CAT $EnvFile > /dev/null" 2>&1`;
			if ( $caterr ) {
				$EnvFileExists = 0;	# error encountered by cat
				$_[4]=1;                # Not a valid file
			} else {
				$EnvFileExists = 1;	# cat successful
			}
		} else {
			$EnvFileExists = 0;		# not a text file
			$_[4]=1;                # Not a valid file
		}
	}
        # Important points from the man page of 'su':
        #   * Regarding LD_LIBRARY_PATH,
        #       Variables with LD_ prefix are removed for security  reasons.
        #       Thus,  su  bin will not retain previously exported variables
        #       with LD_ prefix while becoming user bin.
        #   * The environment  of  user root is passed along to user oracle,
        #       with the exception of $PATH, which is controlled by PATH and
        #       SUPATH in /etc/default/su.
        #   * $LANG environment variable: The su should work even if LANG=ja:
        #       If any  of  the  LC_*  variables  (  LC_CTYPE,  LC_MESSAGES,
        #       LC_TIME,   LC_COLLATE,  LC_NUMERIC,  and  LC_MONETARY)  (see
        #       environ(5)) are not set in the environment, the  operational
        #       behavior  of  su  for  each corresponding locale category is
        #       determined by the value of the LANG environment variable. If
        #       LC_ALL  is  set,  its contents are used to override both the
        #       LANG and the other LC_* variables.  If  none  of  the  above
        #       variables  are  set in the environment, the "C" (U.S. style)
        #       locale determines how su behaves.

	# NOTE: that the LD_LIBRARY_PATH should be set AFTER the su to $Owner, and
	#	NOT for environment of the user (root) running this script.
	#
	# Be careful with the backslashes in this script.
	#
	# $ORACLE_HOME/lib is NOT set in LD_LIBRARY_PATH, because if set, a connection
	# to the 8.1.6 database server using the 8.1.7 Listener fails. This happens
	# because the 8.1.7 Listener passes the LD_LIBRARY_PATH set for itself, to the
	# 8.1.6 background process, causing the connection to fail.
	# If LD_LIBRARY_PATH needs to be set, for some reason, then set it in the
	# resource attribute "EnvFile".
	
	my $ldlibpath="/usr/lib:\$$LIBRARY_PATH";

	my ($LD_ENV);
	if ($UseCSH) {
		$LD_ENV = "setenv $LIBRARY_PATH \\\"\\\"; ";
		#
		# Note the .= concatenation operator below
		#
		if ($EnvFileExists) {
			$LD_ENV .= "source $EnvFile; ";
		}
		$LD_ENV .= "setenv $LIBRARY_PATH $ldlibpath; unsetenv TWO_TASK; ";
	} else {
		$LD_ENV = "$LIBRARY_PATH=\"\"; export $LIBRARY_PATH; ";
		#
		# Note the .= concatenation operator below
		#
		if ($EnvFileExists) {
			$LD_ENV .= ". $EnvFile; ";
		}
		$LD_ENV .= "$LIBRARY_PATH=$ldlibpath; export $LIBRARY_PATH; TWO_TASK=; export TWO_TASK";
	}
	return $LD_ENV;
}

# Function : get_pid_list
# Desc     : get list of pids specific to an listener
# Input    : Home, listener
# Output   : None
# Returns  : @all_pids
#
sub get_pid_list {

	my ($Home,$Listener) = @_;
        my ($processname);
        my ($pid_one);
        my (@all_pids);
        unless (open(PS, "$PS |")) {
        	push(@all_pids,"-1");
                return @all_pids;
	}
        while (<PS>){
                ($pid_one, $processname) = split " ",$_,2;

		if ($processname =~ /^$Home\/bin\/tnslsnr/ and $processname =~ /\btnslsnr\s*$Listener\b/i) {
                        push(@all_pids,$pid_one);
                }
        }
        close PS;
        return @all_pids;
}
1;
