# $Copyright: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Header: /project/isv-agent/VProAgents/Apache/Apache.pm,v 1.6.2.9 2009/01/06 06:41:25 lpatil Exp $
#
# $Author: lpatil $
# $Date: 2009/01/06 06:41:25 $
# $Revision: 1.6.2.9 $
#
#	Veritas Software Corporation
#
#
# Provides:                  Apache
# Author:                    011342
# Creation Date:             20021205 (Thursday December  5, 22:40:21 MST 2002)
# Modification History:
# Known bugs/caveats:
# Future Enhancements Under Consideration:
#
#
# Provides: package Apache
#
# Functionality: provides services for the Apache application-set.
#-------------------------------------------------------------------------------

use strict;
use IO::Socket;
use POSIX qw ( uname );

#-------------------------
# Declare the package
#-------------------------
package Apache;

#-------------------------------------
# include required framework modules
#-------------------------------------
use VCSagentFW;     # Primary Agent Interface to the ACC Library
use VCSDefines;     # This contains constants that VCS uses
use Sys;            # support generalized UNIX commands
use Version;        # version display generator
use Proc;           # For calls to the process routines


#---------------------------------------------------------------------------
# Add our version number to the version hash and check if versions are
# requested.
#---------------------------------------------------------------------------
Version::TellVersion("\$Id: Apache.pm,v 1.6.2.9 2009/01/06 06:41:25 lpatil Exp $::ETVHeader");


#----------------------------------------------------
# Hard code if you still didn't get it from the env..
#----------------------------------------------------
my $CLUS_HOME;
if ( $ENV { CLUSTER_HOME } ) {
  $CLUS_HOME ||= $ENV{CLUSTER_HOME};
}
else { # VCS
   $CLUS_HOME ||= $ENV{VCS_HOME} ||= "/opt/VRTSvcs";
}


#--------------------------------
# Define Application constants.
#--------------------------------

my $sCmdHaType = "${CLUS_HOME}/bin/hatype";
use constant HTTPD_PROCESS => "httpd";
use constant HTTPD_START   => "bin/apachectl";
use constant HTTPD_STOP    => "bin/apachectl";

=pod

 -------------------------------------------------------------------------------
  Function : bool ArgsValid (IN string sEntryPointName,
                             IN string localhttpdDir,
                             IN string localEnvFile,
                             IN string localPidFile,
                             IN string localSharedObjDir,
                             IN int localMonitorGetTimeOut,
                             IN string localHost,
                             IN int localPort,
                             IN string localUser,
                             IN int localSecondLevelMonitor
                             IN int localSecondLevelTimeout,
                             IN string localConfigFile,
                             IN string localDirectiveAfter,
                             IN string localDirectiveBefore );

 Description : Validates the ArgList passed to the entry point.

 Parameters :

 $sEntryPointName               Entry pt name
 $localhttpdDir                 Installation base directory of instance
 $localEnvFile                  Absolute path to enviroment file if any
 $localPidFile                  Absolute path to Pid file if any
 $localSharedObjDir             Location of shared objects for instance
 $localMonitorGetTimeout        MonitorTimeout for resource type
 $localHost                     Virtual hostname hosting the server instance
 $localPort                     HTTP port for web server
 $localUser                     Dedicated UNIX user for web server
 $localSecondLevelMonitor       Option to enable in-depth monitoring
 $localSecondLevelTimeout       Timeout for in-depth monitoring
 $localConfigFile               Configuration file for the Apache HTTP instance
 $localDirectiveAfter           DirectiveAfter
 $localDirectiveBefore          DirectiveBefore

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------------

=cut

sub ArgsValid 
{

  #----------------------------------------
  # set up local vars from passed in ones
  #----------------------------------------
  my($sEntryPointName,
     $localhttpdDir,
     $localEnvFile,
     $localPidFile,
     $localSharedObjDir,
     $localMonitorGetTimeout,
     $localHost,
     $localPort,
     $localUser,
     $localSecondLevelMonitor,
     $localSecondLevelTimeout,
     $localConfigFile,
     $localDirectiveAfter,
     $localDirectiveBefore) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20001, "Subroutine <%s> called with:
  EntryPointName          [%s]
  httpdDir                [%s]
  EnvFile                 [%s]
  PidFile                 [%s]
  SharedObjDir            [%s]
  MonitorGetTimeout       [%s]
  Host                    [%s]
  Port                    [%s]
  User                    [%s]
  SecondLevelMonitor      [%s]
  SecondLevelTimeout      [%s]
  ConfigFile              [%s]
  DirectiveAfter          [%s]
  DirectiveBefore         [%s]",

  $sMyName, $sEntryPointName, $localhttpdDir, $localEnvFile,
  $localPidFile, $localSharedObjDir, $localMonitorGetTimeout,
  $localHost, $localPort, $localUser, $localSecondLevelMonitor,
  $localSecondLevelTimeout, $localConfigFile, $localDirectiveAfter,
  $localDirectiveBefore );

 
  #--------------------------------------------------
  # Create bitmask values - constants are capitals
  #--------------------------------------------------
  use constant BM_ROOTDIR                     => 1;
  use constant BM_HOST                        => 2;
  use constant BM_ENVFILE                     => 4;
  use constant BM_MONITORGETTIMEOUT           => 8;
  use constant BM_PORT                        => 16;
  use constant BM_CONFIG                      => 32;

  #---------------------------------------------
  # Set up a bitmask value to field name hash
  #---------------------------------------------
  my %hHTTPtoFieldName                     = ();
  $hHTTPtoFieldName{Apache::BM_ROOTDIR}    = "httpdDir";
  $hHTTPtoFieldName{Apache::BM_CONFIG}     = "ConfigFile";

  #--------------------------------------------------
  # Use passed in vars to setup the bitmask to check
  #--------------------------------------------------
  my $bmArgsSet = 0;
  foreach my $iBmKey ( keys(%hHTTPtoFieldName) ) 
  {
    if ( eval("defined \$local$hHTTPtoFieldName{$iBmKey}") &&
         eval("\$local$hHTTPtoFieldName{$iBmKey}") ne "" )
    { 
      $bmArgsSet |= $iBmKey ;
    }
  }

  #--------------------------------------------------
  # Setup mandatory fields per the Entry Point.
  #--------------------------------------------------
  my @bmMandatoryFieldList = (Apache::BM_ROOTDIR,
                              Apache::BM_CONFIG);

  my $sMandatoryFieldsNotSetList = undef;

  foreach my $tmpMandatoryField (@bmMandatoryFieldList) 
  {
    if($bmArgsSet & $tmpMandatoryField) 
    {
      #-------------------------
      # field found - continue
      #-------------------------
    }
    else 
    {
      #---------------------------------------------
      # Mandatory field missing - add name to list.
      #---------------------------------------------
      $sMandatoryFieldsNotSetList .= " [" . $hHTTPtoFieldName{$tmpMandatoryField} . "]";
    }
  }

  #------------------------------
  # Report on the missing fields
  #------------------------------
  if($sMandatoryFieldsNotSetList) 
  {
    &logError( 20002,
               "<%s> The following mandatory fields were not set [%s]",
               $sMyName, $sMandatoryFieldsNotSetList);
    return 0;
  }
  else
  {
    &logTrace( 20003, "<%s> All mandatory fields are set", $sMyName );
  }

  #-----------------------------------------------
  # Check mandatory fields for SecondLevelMonitor 
  #-----------------------------------------------

  if ( $localSecondLevelMonitor )
  {
    if ( eval("defined \$localHost") &&
         eval("\$localHost") ne "" )
    {
      &logTrace( 20004, "<%s> Mandatory fields for second level monitoring are set", $sMyName );
    }
    else
    {
      &logError( 20005,
                 "<%s> [HostName] field is not set. It is mandatory field when SecondLevelMonitor is True.",
                 $sMyName);
      return 0;
    }
  }

  my $sCmdToTest        = '';
  my $localStartProgram = Apache::HTTPD_PROCESS;
  my $localStopProgram  = Apache::HTTPD_PROCESS;

  if ( $sEntryPointName eq "online" || $sEntryPointName eq "offline")
  {
    #--------------------------------------------------
    # Check existence of ConfigFile in online/offline
    #--------------------------------------------------
    if ( $localConfigFile ne "")
    {
      if(! &CheckFile("$localConfigFile", "r") ) 
      {
        return 0;
      }
    }
    else
    {
      #---------------------------------------------
      # NULL ConfigFile is not allowed.
      #---------------------------------------------
      return 0;
    }
    
    #------------------------------------
    # Check existence of SharedObjDir
    #------------------------------------
    if ( $localSharedObjDir ne "")
    {
      if(! -d $localSharedObjDir ) 
      {
        &logError(20006,
        "<%s> SharedObjDir [%s] is not a valid directory.",
        $sMyName, $localSharedObjDir ) ;
        return 0;
      }
    }
  }

  #--------------------------------------------------
  # Validate ConfigFile is not NULL in monitor/clean
  #--------------------------------------------------
  if ( $sEntryPointName eq "monitor" || $sEntryPointName eq "clean")
  {
    if ( $localConfigFile eq "")
    {
      return 0;
    }
  }

  #--------------------------------------------------
  # Check Validity of EnvFile only in online/offline
  #--------------------------------------------------
  
  if ($sEntryPointName eq "online" || $sEntryPointName eq "offline")
  {
    if( $localEnvFile ne "") 
    {
      if(! &CheckFile("$localEnvFile", "r") ) 
      {
        return 0;
      }
    }
  }

  #----------------------------------------------------
  # Check Validity of httpdDir for online,offline
  #----------------------------------------------------
  if ( $sEntryPointName eq "online" || $sEntryPointName eq "offline" )
  {
    #--------------------------------
    # Check the validity of httpdDir
    #--------------------------------
    if(! -d $localhttpdDir ) 
    {
      &logError(20007,
      "<%s> httpdDir [%s] is not a valid directory.", $sMyName, $localhttpdDir ) ;
      return 0;
    }
  }
  else 
  {
    if ( !$localhttpdDir )
    {
      &logError(20008, "<%s> httpdDir is NULL.", $sMyName);
      return 0;
    }
  }

  if($sEntryPointName eq "online"  || $sEntryPointName eq "offline")
  {
    if ( $sEntryPointName eq "online" )
    {
      if ( $localSecondLevelMonitor )
      { 
        #-----------------------------------
        # Check validity of the HostName
        #-----------------------------------
        my $sHostLookup = gethostbyname($localHost);
        if ( !defined($sHostLookup) )
        {
          &logError(20009,
          "<%s> Host [%s] is not valid. gethostbyname failed.",
          $sMyName, $localHost);
          return 0;
        }
      }
    }

    #-----------------------------------
    # Check validity of the username
    #-----------------------------------
    if ( $localUser ne "" )
    {
      my $spwnam = getpwnam($localUser);
      if ( !defined($spwnam) )
      {
        &logError(20010, "<%s> User name [%s] not valid.",
        $sMyName, $localUser);
        return 0;
      }
    }
    else
    {
      $localUser = "root";
    }

    if ( $localSecondLevelTimeout >= $localMonitorGetTimeout )
    {
      &logError(20011,
      "<%s> SecondLevelMonitorTimeOut must be less than MonitorTimeOut.",
      $sMyName);
      return 0;
    }

    #--------------------------------------------------
    # Check validity of StartProgram only on Online
    #--------------------------------------------------
    if( $localStartProgram =~ /^\// ) 
    {
      #--------------------------------------------------
      # Test command given as it begins with a slash
      #--------------------------------------------------
      ($sCmdToTest, undef) = split(/ /, $localStartProgram, 2);
    }
    else 
    {
      #--------------------------------------------------
      # Test Command given is relative to httpdDir
      #--------------------------------------------------
      $sCmdToTest = $localhttpdDir . "/" . $localStartProgram;
      ($sCmdToTest, undef) = split(/ /, $sCmdToTest, 2);
    }
    if(! &CheckFile($sCmdToTest, "x") ) 
    {
      return 0;
    }
  }

  #--------------------------------------------------
  # Check validity of StopProgram only on Offline
  #--------------------------------------------------

  if($sEntryPointName eq "offline" ) 
  {
    if( $localStopProgram =~ /^\// ) 
    {
      #--------------------------------------------------
      # Test command given as it begins with a slash.
      #--------------------------------------------------
      ($sCmdToTest, undef) = split(/ /, $localStopProgram, 2); 
    }
    else 
    {
      #-------------------------------------------------- 
      # Test Command given is relative to httpdDir
      #--------------------------------------------------
      $sCmdToTest = $localhttpdDir . "/" . $localStopProgram;
      ($sCmdToTest, undef) = split(/ /, $sCmdToTest, 2);
    }
  }

  #--------------------------------------------------
  # Check validity of httpdDir, HostName, SecondLevelTimeout
  #--------------------------------------------------
  if($sEntryPointName eq "monitor" )
  {
    if ( ! $localhttpdDir )
    {
      &logError(20012, "<%s> httpdDir cannot be Null.", $sMyName);
      return 0;
    }
    if ( $localSecondLevelMonitor )
    {
      my $sHostLookup = gethostbyname($localHost);
      if ( !defined($sHostLookup) )
      {
        &logError(20013, "<%s> Host [%s] is not valid. gethostbyname failed.",
                  $sMyName, $localHost);
        return 0;
      }

      if ( $localSecondLevelTimeout >= $localMonitorGetTimeout )
      {
        &logError(20014,
        "<%s> SecondLevel MonitorTimeOut must be less than MonitorTimeOut.",
        $sMyName);
        my $localNewSecondLevelTimeout = $localMonitorGetTimeout - 10;
        &logTrace(20015, "<%s> Setting SecondLevelTimeout to [%s].",
        $sMyName, $localNewSecondLevelTimeout);
      }
    }
  }

  return 1;
}


=pod

 -------------------------------------------------------------------------------
  Function : Boolean Start(IN string shttpdDir,
                           IN string sUser,
                           IN string sEnvFile,
                           IN string sSharedObjDir,
                           IN string sDirectiveAfter,
                           IN string sDirectiveBefore,
                           IN string sConfigFile,
                           IN string bEnableSSL );

 Description : Start logic needed for online entry pt

 $shttpdDir                 Installation base directory of instance
 $sUser                     Dedicated UNIX user for web server
 $sEnvFile                  Absolute path to enviroment file if any
 $sSharedObjDir             Location of shared objects for instance
 $sDirectiveAfter           DirectiveAfter
 $sDirectiveBefore          DirectiveBefore
 $sConfigFile               Configuration file for the Apache HTTP instance
 $bEnableSSL                Flag to indicate if SSL is enabled

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------------

=cut

sub Start 
{
  #---------------
  # Get Arguments
  #---------------
  my ( $shttpdDir, 
       $sResourceName,
       $sUser, 
       $sEnvFile,
       $sSharedObjDir,
       $sDirectiveAfter,
       $sDirectiveBefore,
       $sConfigFile,
       $bEnableSSL) = @_ ;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace( 20016, "Subroutine <%s> called with:,
  httpdDir                 [%s]
  ResourceName             [%s]
  User                     [%s]
  EnvFile                  [%s]
  SharedObjDir             [%s]
  DirectiveAfter           [%s]
  DirectiveBefore          [%s]
  ConfigFile               [%s]
  EnableSSL                [%s]",

  $sMyName,   $shttpdDir, $sResourceName, $sUser,   $sEnvFile,  
  $sSharedObjDir, $sDirectiveAfter, $sDirectiveBefore, $sConfigFile,
  $bEnableSSL );

  #--------------------------------------
  # Set Return Code Optimistically to 1
  #--------------------------------------
  my $iReturnCode   = 1;
  my $iCmdExitCode  = 1;
  my $snewArgsToPass   = " -f $sConfigFile ";
  if ( $sDirectiveBefore ne "" )
  {
    $snewArgsToPass = $snewArgsToPass." ".$sDirectiveBefore;
  }
  if ( $sDirectiveAfter ne "")
  {
    $snewArgsToPass = $snewArgsToPass." ".$sDirectiveAfter;
  }
  if ( $sSharedObjDir )
  {
    $snewArgsToPass = $snewArgsToPass." -R ".$sSharedObjDir;
  }
  if ( $bEnableSSL )
  {
    $snewArgsToPass = $snewArgsToPass." -DSSL";
  }

  my $sApacheVersion;
  if ( $sEnvFile ) {
    ( $sApacheVersion ) = grep { s#.*version:.*Apache/(...).*#$1#s } readpipe ( ". $sEnvFile;$shttpdDir/httpd -v");
  }
  else {
    ( $sApacheVersion ) = grep { s#.*version:.*Apache/(...).*#$1#s } readpipe ( "$shttpdDir/httpd -v");
  }

  &logTrace(20017,"<%s>:Apache Version is [%s]",$sMyName, $sApacheVersion);

  #------------------------------
  # Apache Version is 2.0 or 2.2 
  #------------------------------
  if ( ($sApacheVersion eq "2.0" || $sApacheVersion eq "2.2") )
  {
    $snewArgsToPass = $snewArgsToPass." "." -k start";
  }

  my $sStartProgram = Apache::HTTPD_PROCESS;

  #------------------------------
  # Initialize Command Strings
  #------------------------------
  my $sCmdToRun;
  if( $sStartProgram =~ /^\// ) 
  {
    $sCmdToRun = $sStartProgram;
  }
  else 
  {
    #-------------------------------------
    # Start Command relative to httpdDir
    #-------------------------------------
    $sCmdToRun = $shttpdDir . "/" . $sStartProgram; 
  }

  my $bSourceEnv = 0;
  my @lsCmdOut;
  my $bTimedOut;
  my $iTimedOutSeconds;
  if ( $sUser eq "" )
  {
    $sUser = "root";
  }
  if ( $sEnvFile )
  {
    $bSourceEnv = 1;
  }

  #---------------------------------------------
  # Kill leftover processes if any for apache.
  #---------------------------------------------
  KillApacheHTTPProcesses($shttpdDir,$sResourceName,$sConfigFile);

  if ( Sys::RunWithEnvCmdWithOutputWithTimeOut( $sEnvFile,
                                                $bSourceEnv,
                                                $sCmdToRun,
                                                $snewArgsToPass,
                                                \$iCmdExitCode,
                                                $sUser,
                                                \@lsCmdOut,
                                                \$bTimedOut,
                                                \$iTimedOutSeconds) )
  {
    if ( 1 == $bTimedOut )
    {
      &logError( 20018,
      "<%s> Command [%s] with args [%s] timed out after [%s] seconds",
      $sMyName, $sCmdToRun, $snewArgsToPass, $iTimedOutSeconds );
    }
    elsif ( $iCmdExitCode > 0 )
    {
      &logError( 20019,
      "<%s> Command [%s] with args [%s] returned exit code [%s]",
      $sMyName, $sCmdToRun, $snewArgsToPass, $iCmdExitCode );
    }
  }

  if ( $iCmdExitCode > 0 ) {
    #--------
    # Error
    #--------
    &logError(20020, "<%s> Start Failed. Return code [%s]",
    $sMyName, $iCmdExitCode);

    $iReturnCode=0;
  }
  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Stop ( IN String ConfigFile,
                        IN String EnvFile,
                        IN String SharedObjDir,
                        IN String httpdDir,
                        IN String DirectiveBefore,
                        IN String DirectiveAfter );

 Description : Stop logic needed for offline entry pt

 Parameters :

 $sConfigFile               Configuration file for the Apache HTTP instance
 $sEnvFile                  Absolute path to enviroment file if any
 $sSharedObjDir             Location of shared objects for instance
 $shttpdDir                 Installation base directory of instance
 $sDirectiveBefore          DirectiveBefore
 $sDirectiveAfter           DirectiveAfter

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Stop 
{
  #---------------
  # Get Arguments
  #---------------
  my ( $sConfigFile,
       $sResourceName,
       $sEnvFile,
       $sSharedObjDir,
       $shttpdDir,
       $sDirectiveBefore,
       $sDirectiveAfter) = @_ ;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20021, "Subroutine <%s> called with:
  ConfigFile               [%s]
  ResourceName             [%s]
  EnvFile                  [%s]
  SharedObjDir             [%s]
  httpdDir                 [%s]
  DirectiveBefore          [%s]
  DirectiveAfter           [%s]",

  $sMyName, $sConfigFile, $sResourceName, $sEnvFile,  
  $sSharedObjDir, $shttpdDir, $sDirectiveBefore,
  $sDirectiveAfter );

  #--------------------------------------
  # Set Return Code Optimistically to 1
  #--------------------------------------
  my $iAppReturnCode;
  my $iReturnCode  = 1;
  my $snewArgsToPass   = " -f $sConfigFile ";
  if ( $sDirectiveBefore ne "" )
  {
    $snewArgsToPass = $snewArgsToPass." ".$sDirectiveBefore;
  }
  if ( $sDirectiveAfter ne "")
  {
    $snewArgsToPass = $snewArgsToPass." ".$sDirectiveAfter;
  }

  my $sApacheVersion;

  if ( $sEnvFile ) {
    ( $sApacheVersion ) = grep { s#.*version:.*Apache/(...).*#$1#s } readpipe ( ". $sEnvFile;$shttpdDir/httpd -v");
  }
  else {
    ( $sApacheVersion ) = grep { s#.*version:.*Apache/(...).*#$1#s } readpipe ( "$shttpdDir/httpd -v");
  }

  &logTrace(20022,"<%s>:Apache Version is [%s]",$sMyName, $sApacheVersion);

  #------------------------------
  # Apache Version is 2.0 or 2.2
  #------------------------------
  if ( ($sApacheVersion eq "2.0" || $sApacheVersion eq "2.2") )
  {
    $snewArgsToPass = $snewArgsToPass." "." -k stop";
  }
  
  my $sStopProgram = Apache::HTTPD_PROCESS;

  #----------------------------
  # Initialize Command Strings
  #----------------------------
  my $sCmdToRun;
  if( $sStopProgram =~ /^\// ) 
  {
    $sCmdToRun = $sStopProgram;
  }
  else 
  {
    #--------------------------------------------------
    # Stop Command relative to httpdDir
    #--------------------------------------------------
    $sCmdToRun = $shttpdDir . "/" . $sStopProgram; 
  }

  if ( $sApacheVersion eq "1.3" )
  {
    #-------------------------------------------------
    # Kill parent process with TERM (1.3 behaviour)
    #-------------------------------------------------
    Kill($sConfigFile);
  }
  else
  {
    #------------------------------------------------------------
    # This is a 2.x instance hence should be called with -k stop
    #------------------------------------------------------------
    my $bSourceEnv = 0;
    my $iCmdExitCode;
    my @lsCmdOut;
    my $bTimedOut;
    my $iTimedOutSeconds;
    my $sUser = "root";

    if ( $sEnvFile )
    {
      $bSourceEnv = 1;
    }
    else 
    {
      $sEnvFile = "";
    }
    if ( Sys::RunWithEnvCmdWithOutputWithTimeOut( $sEnvFile,
                                                  $bSourceEnv,
                                                  $sCmdToRun,
                                                  $snewArgsToPass,
                                                  \$iCmdExitCode,
                                                  $sUser,
                                                  \@lsCmdOut,
                                                  \$bTimedOut,
                                                  \$iTimedOutSeconds) )
    {
      if ( 1 == $bTimedOut )
      {
        &logError( 20023,
                   "<%s> Command [%s] with args [%s] timed out after [%s] seconds",
                   $sMyName, $sCmdToRun, $snewArgsToPass, $iTimedOutSeconds );
      }
      elsif ( $iCmdExitCode > 0 )
      {
        &logError( 20024,
        "<%s> Command [%s] with args [%s] returned exit code [%s]",
        $sMyName, $sCmdToRun, $snewArgsToPass, $iCmdExitCode );
      }
      else
      {
        &logTrace( 20025,
        "<%s> Command [%s] with args [%s] returned exit code [%s]",
        $sMyName, $sCmdToRun, $snewArgsToPass, $iCmdExitCode );
      }
    }
  
    if ( $iCmdExitCode > 0 ) 
    {
      #--------
      # Error
      #--------
      &logError(20026,
      "<%s> Stop Failed. Return code [%s]", $sMyName, $iCmdExitCode);
      $iReturnCode=0;
    }

  }
  sleep (10);

  #---------------------------------------------
  # Kill leftover processes if any for apache.
  #---------------------------------------------
  KillApacheHTTPProcesses($shttpdDir,$sResourceName,$sConfigFile);

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int GetResourceState ( IN String httpdDir,
                                   IN String ConfigFile,
                                   IN String ResourceName );

 Description : First level monitor to determine state of resource.

 Parameters :

 $shttpdDir
 $sConfigFile               Configuration file for the Apache HTTP instance
 $sResourceName             Resource Name

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub GetResourceState 
{
  #-----------------
  # Get Arguments
  #-----------------
  my ( $shttpdDir, $sConfigFile ,$sResourceName) = @_ ;


  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20027, "Subroutine <%s> called:
  httpdDir                [%s]
  ConfigFile              [%s]
  ResourceName            [%s]",
  $sMyName, $shttpdDir, $sConfigFile, $sResourceName );

  my $iReturnCode = VCSDef_MonitorExit::UNKNOWN;

  #-----------------------
  # Initialize Variables
  #-----------------------
  my @liPIDsWithPPIDOfOne    = ();
  my @liPIDSWithoutPPIDOfOne = ();

  $iReturnCode = Apache::GetProcState( $sConfigFile, 
                                       $sResourceName,
                                       \@liPIDsWithPPIDOfOne, 
                                       \@liPIDSWithoutPPIDOfOne) ;

  if ( VCSDef_MonitorExit::ONLINE == $iReturnCode )
  {
    &logTrace(20028,
    "<%s> GetResourceState found all required processes", $sMyName);
  }
  elsif ( VCSDef_MonitorExit::OFFLINE == $iReturnCode ) 
  {
    &logTrace(20029,
    "<%s> GetResourceState found that resource was offline", $sMyName);
  } 
  elsif ( VCSDef_MonitorExit::PARTIAL_ONLINE == $iReturnCode )
  {
    &logTrace(20030,
    "<%s> GetResourceState found the resource partially online", $sMyName);
  } 
  else 
  {
    &logTrace(20031,
    "<%s> GetResourceState could not determine state", $sMyName);
  }
  return $iReturnCode ;
}

=pod

 -------------------------------------------------------------------------
 Function : int GetProcState ( IN String ConfigFile,
                               IN String ResourceName
                               OUT ReferenceToListOfIntegers  rliPIDsWithPPIDOfOne,
                               OUT ReferenceToListOfIntegers  rliPIDsWithoutPPIDOfOne) 

 Description : First level monitor to determine state of resource.

 Parameters :
 ConfigFile                  Configuration file for the Apache HTTP instance
 ResourceName                ResourceName
 PIDsWithPPIDOfOne           List of PID whose PPIDs are '1'
 PIDsWithoutPPIDOfOne        List of PID whose PPIDs are NOT '1'

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub GetProcState 
{
  #------------------
  # Get Arguments
  #------------------
  my ( $sConfigFile,
       $sResourceName,
       $rliPIDsWithPPIDOfOne, 
       $rliPIDsWithoutPPIDOfOne ) = @_ ;

  #------------------
  # Announce Entry
  #------------------
  my $sMyName = ( caller 0 )[3];

  &logTrace( 20032, "Subroutine <%s> called:
  ConfigFile               [%s]
  ResourceName             [%s]
  rliPIDsWithPPIDOfOne     [%s]
  rliPIDsWithoutPPIDOfOne  [%s]",
  $sMyName,
  $sConfigFile, $sResourceName, $rliPIDsWithPPIDOfOne, $rliPIDsWithoutPPIDOfOne );
  
  #---------------------------------------------
  # Set Return Code Optimistically to ONLINE
  #---------------------------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;

  #--------------------
  # Set up the counts
  #--------------------
  my $iOnlineCount  = 0;
  my $iOfflineCount = 0;

  my $iZsched1;
  my $iZsched2;
  my $bZoneFlag = 0;
  my $sOSType = Arch::GetOSType();

  #-----------------------------------
  # Call processes and store in hash
  #-----------------------------------
  my @hProcListHash = (); #set this empty;
  my $iProcessCount = Proc::GetLongProcessListHash(\@hProcListHash);

  #----------------------------------------
  # Confirm that processes were found.
  #----------------------------------------
  if(0 == $iProcessCount) 
  {
    #--------------------------------------------------------------------------
    # Log error and return offline - if we cant get a process list, we dont
    # know the state.
    #--------------------------------------------------------------------------
    &logError(20033,
    "<%s> Call to get process list did not return any processes", $sMyName);
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  else 
  {
    &logTrace( 20034,
    "<%s> Call to get process list returned [%s] processes",
    $sMyName, $iProcessCount);

# Solaris Zones
    my @lsOsDetails = POSIX::uname();
    my $iRel = $lsOsDetails[2];
    &logTrace(12345,"Operating System revision is [%s].",$iRel);

    if ( ( Arch::SUN_OS == $sOSType ) && ( ( $iRel ne "5.8" ) && ( $iRel ne "5.9" ) ) )
    {
      &logTrace(12345,"Platform is solaris");

      my $CLUS_HOME;
      if ( $ENV { CLUSTER_HOME } ) {
         $CLUS_HOME ||= $ENV{CLUSTER_HOME};
      }
      else { # VCS
         $CLUS_HOME ||= $ENV{VCS_HOME} ||= "/opt/VRTSvcs";
      }

      my $sCommand = "${CLUS_HOME}/bin/hares";
      my $bHaCmd_Zone = 0;

      # Find out the Zone Scheduler for this zone
      foreach my $tmpproc (@hProcListHash)
      {
        if ( defined $tmpproc->{"PPID"} && defined $tmpproc->{"PID"})
        {
          $iZsched1 = $tmpproc->{'PID'};
          $iZsched2 = $tmpproc->{"PPID"};
          if( $iZsched1 == $iZsched2 )
          {
            if( 0 != $iZsched1 )
            {
              &logTrace(12345,"Zone Scheduler PID is [%s]",$iZsched1);
              $bZoneFlag = 1;
              last;
            }
          }
        }#if
      }#for
    }
# Solaris Zones

    #----------------------------------------
    # Test for the StopProgram process.
    #----------------------------------------
    my $sProc = Apache::HTTPD_PROCESS;
    my @hProcListHashMatched=();
    my $sFilter="-f $sConfigFile"; #build filter based on ConfigFile
    my $iNumberOfMatches=Proc::FilterProcs($sFilter,
                                           \@hProcListHash,
                                           \@hProcListHashMatched);

    &logTrace( 20035,
    "<%s> Matches for process [%s] with filter [%s] got [%s] matches",
    $sMyName, $sProc, $sFilter, $iNumberOfMatches);

    my @snewProcListHasMatched=();
    my $localCount = 0;
    foreach my $hEachProcHash (@hProcListHashMatched) 
    {
      my $iTmpPPID = $hEachProcHash->{"PPID"};
      my $iTmpPID  = $hEachProcHash->{"PID"};
      my $iTmpCMD  = $hEachProcHash->{"COMMAND"};

      
      if ( $iTmpCMD =~ /$0/ )
      {
        #-----------------------------------------------------------------
        # Filter out entry point processes from the matched process list.
        #-----------------------------------------------------------------
      }
      else
      {
        #--------------------------------------------------
        # Exact match for config file is found.
        #--------------------------------------------------
        if ($iTmpCMD =~ /$sConfigFile\b/ )
        {
#          my @lsOsDetails = POSIX::uname();
#          my $iRel = $lsOsDetails[2];
#          &logTrace(12345,"Operating System revision is [%s].",$iRel);

          if ( ( Arch::SUN_OS == $sOSType ) && ( ( $iRel ne "5.8" ) && ( $iRel ne "5.9" ) ) )
          {
            # Following checks are to be done when you are on OS with zone support
            if ( 1 == $bZoneFlag ) # local zone
            {
              &logTrace(12345,"Matched local zone process [%s].",$iTmpCMD);
              push(@snewProcListHasMatched, $hEachProcHash); # Return local copy
              $localCount++;
            }
            else
            {
              # we are now in global zone check for following file
              if ( stat "/proc/$iTmpPID/root/etc/zones/index" ) # global zone
              {
                &logTrace(12345,"Matched global zone process [%s].",$iTmpCMD);
                push(@snewProcListHasMatched, $hEachProcHash); # Return local copy
                $localCount++;
              }
              else
              {
                # In global zone but the file does not exists, this is local zone process, skip it
                &logTrace(12345,"Skipping local zone process [%s].",$iTmpCMD);
              }
            }
          } 
          else
          {
            # For Operating systems which does not support zones
            push(@snewProcListHasMatched, $hEachProcHash); # Return local copy
            $localCount++;
          }
        }
      }
    }
    #--------------------------------------------------------------------
    # $localCount is the new count with filtered entry point processes.
    #--------------------------------------------------------------------
    $iNumberOfMatches = $localCount;

    @hProcListHashMatched = @snewProcListHasMatched;


    if ( 0 == $iNumberOfMatches  )
    {
      #---------------------------------------------
      # Offline at this point.  No procs to look at.
      #---------------------------------------------
      $iReturnCode = VCSDef_MonitorExit::OFFLINE;
      return $iReturnCode; 
    }
    else 
    {
      #----------------------------------------------------------------------
      # for httpd (part of WebServer) to be online there must be at least 2
      # httpd processes AND one must have PPID of 1 (the master httpd)
      #----------------------------------------------------------------------
      my @lhPIDsWithPPIDOfOne    = ();
      my @lhPIDsWithoutPPIDOfOne = ();

      foreach my $hEachProcHash (@hProcListHashMatched) 
      {
        my $iTmpPPID = $hEachProcHash->{"PPID"};
        my $iTmpPID  = $hEachProcHash->{"PID"};
        my $iTmpCMD  = $hEachProcHash->{"COMMAND"};

        if ( $iTmpCMD =~ /$0/ )
        {
          #-----------------------------------------------------------------
          # Filter out entry point processes from the matched process list.
          #-----------------------------------------------------------------
        }
        else
        {
          if ($iTmpCMD =~ /$sConfigFile\b/ )
          {
            if(1 == $iTmpPPID)
            {
              push(@lhPIDsWithPPIDOfOne, $hEachProcHash); # Return local copy
              push(@$rliPIDsWithPPIDOfOne, $iTmpPID); # Return global copy
            } 
            else 
            {
              push(@lhPIDsWithoutPPIDOfOne, $hEachProcHash); # Return local copy
              push(@$rliPIDsWithoutPPIDOfOne, $iTmpPID); # Return global copy
            }
          }
        }
      }

      #-------------------------------------------------------------------------
      # Need to make sure that at least one of the proc's owned by init has
      # some (one or more!) children  #build a hash slice of the PIDs in the 
      # lhPIDsWithPPIDOfOne list.
      #-------------------------------------------------------------------------

      my @liPIDsWithPPIDOfOne = ();
      Proc::GetValuesForKeyFromProcDataHash("PID",
                                            \@lhPIDsWithPPIDOfOne,
                                            \@liPIDsWithPPIDOfOne); 
      my %hCheckPPIDHash = ();
      @hCheckPPIDHash { @liPIDsWithPPIDOfOne } = (1) x @liPIDsWithPPIDOfOne;
      @hCheckPPIDHash { @liPIDsWithPPIDOfOne } = (25080) x @liPIDsWithPPIDOfOne;

      #------------------------------------------------------------------------
      # Now spin the children, we only need to find one PPID in the hash 
      # slice to be online
      #------------------------------------------------------------------------
      my $fFoundParentOwnedByInit=0;

      foreach my $hrPIDsWithoutPPIDOfOne (@lhPIDsWithoutPPIDOfOne) 
      {
        my $iTmpPPID = $hrPIDsWithoutPPIDOfOne->{"PPID"};
        my $iTmpCMD = $hrPIDsWithoutPPIDOfOne->{"COMMAND"};

        if ( 1 == $bZoneFlag )
        {
            # Running inside a Solaris Zones
            if( ($iTmpPPID == $iZsched1) && !($iTmpCMD =~ /$0/) )
            {
              $fFoundParentOwnedByInit ++;
            }
        }
        else
        {

          #-----------------------------------------------
          # Filter entry point script from process hash.
          #-----------------------------------------------
          if( (defined $hCheckPPIDHash{$iTmpPPID})  && !($iTmpCMD =~ /$0/) )
          {
            #------------------------------------------------------------
            # Found a child with a parent that is in turn owned by init.
            #------------------------------------------------------------
            $fFoundParentOwnedByInit = 1;
            last;
          }
        }
      }
      if($fFoundParentOwnedByInit > 0 ) 
      {
        &logTrace( 20036,
        "<%s> Online. At least one httpd process is owned by init and has one or more children.", $sMyName ) ; 
        $iReturnCode = VCSDef_MonitorExit::ONLINE;

        #--------------------------------
        # We are online at this point
        #--------------------------------

      } 
      else 
      {

        &logError( 20037, "<%s> Apache instance is not healthy.  At least one httpd process is owned by init, but it has NO child processes.", $sMyName ) ; 
        #$iReturnCode = VCSDef_MonitorExit::PARTIAL_ONLINE;
        $iReturnCode = VCSDef_MonitorExit::OFFLINE;;
        return $iReturnCode;
        
	#----------------------------------------------------------------------
        # We might say we are partial here if we have some online pids,
        # that way when we get to a clean (clean will occur after partial is
        # reported) we will have some pids to kill -9 ;)
	#----------------------------------------------------------------------
      }

      if ( $bZoneFlag == 1 && $iNumberOfMatches == $fFoundParentOwnedByInit )
      {
        &logError( 20036,"Setting Offline. All httpd instances owned by zsched.");
        $iReturnCode = VCSDef_MonitorExit::OFFLINE;
      }
      elsif ( $#lhPIDsWithPPIDOfOne < 0 && $bZoneFlag == 0)
      {
        &logTrace( 20038,
        "<%s> Setting Offline.  Number of processes with a PPID of 1 is less than 1 [%s].",
        $sMyName, $#lhPIDsWithPPIDOfOne );
         $iReturnCode = VCSDef_MonitorExit::OFFLINE;
      }
      else 
      {
        $iReturnCode = VCSDef_MonitorExit::ONLINE;
        &logTrace( 20039, 
          "<%s> There are [%s] total httpd processes, including [%s] httpd processes with a PPID of 1", 
          $sMyName, $#hProcListHashMatched + 1, 
          $#lhPIDsWithPPIDOfOne + 1);
      }
    }
  }
  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int IsOnlineLvl2 ( IN String sHost
                               IN Int iPort,
                               IN Int iMonitorGetTimeout );

 Description : Second level monitoring

 Parameters :

 $sHost                     Virtual hostname hosting the server instance
 $sPort                     HTTP port for web server
 $sMonitorGetTimeout        MonitorTimeout for resource type

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub IsOnlineLvl2 
{
  #----------------
  # Get Arguments
  #----------------
  my ( $sHost,
       $sPort,
       $iMonitorGetTimeout ) = @_ ;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20040, "Subroutine <%s> called with:
  Host               [%s]
  Port               [%s]
  MonitorGetTimeout  [%s]",
  $sMyName, $sHost, $sPort, $iMonitorGetTimeout );

  my $iAppStatus = &CheckHTTP( $sHost, $sPort, $iMonitorGetTimeout );
  if ( $iAppStatus != VCSDef_MonitorExit::ONLINE ) 
  {
    return $iAppStatus ;
  }

  #------------------------------------------------------------
  #If pass 1st and 2nd level monitor then HTTP server is up
  #------------------------------------------------------------
  return $iAppStatus ;
}

#-------------------------------------------------------------------------------
# Function CheckHTTP :
# Int CheckHTTP ( IN String Shost,
#                 IN Int sPort,
#                 IN Int iMonitorGetTimeout)
#-------------------------------------------------------------------------------

=pod

 -------------------------------------------------------------------------
 Function : Int CheckHTTP ( IN String sHost,
                            IN Int iPort,
                            IN Int iMonitorGetTimeout )

 Description : Checks if HTTP is up and listening

 Parameters :

 $sHost                     Virtual hostname hosting the server instance
 $iPort                     HTTP port for web server
 $iMonitorGetTimeout        MonitorTimeout for resource type

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub CheckHTTP 
{
  #--------------------
  # Get Arguments
  #--------------------
  my ( $sHost,
       $sPort,
       $iMonitorGetTimeout ) = @_ ;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20041, "Subroutine <%s> called with:
  Host                  [%s]
  Port                  [%s]
  MonitorGetTimeout     [%s]",
  $sMyName, $sHost, $sPort, $iMonitorGetTimeout );

  #--------------------
  # Define Variables
  #--------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;
  my $iHttpTimeOut = undef;

  if ($iMonitorGetTimeout < 1 ) 
  {
    &logInfo(20042,
    "<%s> MonitorGetTimeout is not above 0 seconds [%s].
    Setting MonitorGetTimeout to 30 seconds.", $sMyName, $iMonitorGetTimeout);
    $iHttpTimeOut = 30;
  }
  else 
  {
    $iHttpTimeOut = $iMonitorGetTimeout;
  }

  #--------------------------------------------------
  # Do a socket connect and parse an http get
  #--------------------------------------------------
  my ($ret, $eol, $space, $sock, $ver, $sGetFile) = ();
  $sGetFile =  '/';
  $ret      =  0;
  $eol      =  "\015\012";
  $space    =  $eol x 2;
  &logTrace(20043, "<%s> HTTP test starting\n", $sMyName );
  $sock = IO::Socket::INET->new( Proto    => "tcp",
                                 PeerAddr => "$sHost",
                                 PeerPort => "$sPort",
                                 Timeout  => "$iHttpTimeOut");
  unless ($sock) 
  {
    &logError(20044,
    "<%s> HTTPD Server not responding to connect attempt on port [%s] for host [%s]",
    $sMyName, $sPort, $sHost);
    return VCSDef_MonitorExit::OFFLINE;
  }

  $sock->autoflush(1);
  print $sock "HEAD $sGetFile HTTP/1.0" . $space;
  while ( <$sock> ) 
  {
    if (/^HTTP/) 
    {
      ($ver, $ret) = split(/\s+/o);
    }
    last if ($ret gt 0);
  }
  close $sock;
  &logTrace(20045, "<%s> HTTP test completed within [%s] seconds",
  $sMyName, $iHttpTimeOut);

  if ($ret > 0 && $ret < 408) 
  {
    &logTrace(20046, "<%s>  HTTP socket test returned [%s]",
    $sMyName, $ret);
  }
  else 
  {
    &logError(20047, "<%s>  HTTP GET test failed for host [%s] port [%s]",
    $sMyName, $sHost, $sPort);
    return VCSDef_MonitorExit::OFFLINE;
  }
  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : bool CheckFile ( IN String FileName,
                             IN Char FileType );

 Description : Check for specified filetype for given file

 Parameters :

 sFileName             File to be tested
 cFileType             Mode for file to be tested for

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub CheckFile 
{
  #--------------------
  # Get Arguments
  #--------------------
  my ( $sFileName,
       $cFileType ) = @_ ;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];
 
  &logTrace( 20048, "Subroutine <%s> called with:
  FileName      [%s]
  FileType      [%s]",
  $sMyName, $sFileName, $cFileType );

  #-----------------------------------
  # Set Return Optimistically to 1
  #-----------------------------------
  my $iReturnCode = 1 ;

  if ( ! Sys::FileExists( $sFileName ) ) 
  {
    &logError( 20049, "<%s> File [%s] does not exist",
    $sMyName, $sFileName );
    $iReturnCode = 0 ;
  }
  elsif ( $cFileType eq "x" && ! -x $sFileName ) {
    &logError( 20050, "<%s> File [%s] is not executable",
    $sMyName, $sFileName );
    $iReturnCode = 0 ;
  }
  return $iReturnCode ;
}

#-------------------------------------------------------------------------------
# Function Kill :
# Int sub Kill ( IN String shttpdDir)
# Return Value : Int iReturnCode 
#-------------------------------------------------------------------------------
=pod

 -------------------------------------------------------------------------
 Function : bool Kill ( IN String sConfigFile);

 Description : Forces shutdown of resource needed for clean entry point

 Parameters :

 ConfigFile              Configuration file for the Apache HTTP instance

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Kill
{
  #---------------
  # Get Arguments
  #---------------
  my ( $sConfigFile , $sResourceName) = @_ ;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20051, "Subroutine <%s> called with:
  ConfigFile        [%s]
  ResourceName      [%s]",
  $sMyName, $sConfigFile, $sResourceName );

  #----------------------
  # Initialize Variables
  #----------------------
  my $iReturnCode            = 1 ;   # Set Return Code Optimistically to 1
  my @liPIDsWithPPIDOfOne    = () ;
  my @liPIDSWithoutPPIDOfOne = () ;

  #-----------------------------------
  # see if any processes are online
  #-----------------------------------
  if(VCSDef_MonitorExit::OFFLINE == Apache::GetProcState( $sConfigFile,
                                                          $sResourceName,
                                                          \@liPIDsWithPPIDOfOne,
                                                          \@liPIDSWithoutPPIDOfOne)) 
  {
    &logTrace(20052, "<%s> Kill found all processes offline", $sMyName );
  }
  else 
  {
    &logTrace(20053, "<%s> Kill found some processes online", $sMyName );

    #----------------------
    # Send kills to pids
    #----------------------
    if(Proc::KillPidsFromList(\@liPIDsWithPPIDOfOne)) {
      #----------------------
      # Success - continue
      #----------------------
    } else {
      &logError(20054, "<%s> Failed to kill all parent processes", $sMyName );
      $iReturnCode = 0;
    }
    if(Proc::KillPidsFromList(\@liPIDSWithoutPPIDOfOne)) {
      #---------------------
      # Success - continue
      #---------------------
    }
    else {
      &logError(20055, "<%s> Failed to kill all child processes", $sMyName );
      $iReturnCode = 0;
    }
  }
  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : bool KillApacheHTTPProcesses ( IN String shttpdDir,
                                           IN String ResourceName,
                                           IN String ConfigFile );

 Description : Kills process belonging to HTTP resource

 Parameters :

 httpdDir                  Installation base directory of instance
 ResourceName              ResourceName
 ConfigFile                Configuration file for the Apache HTTP instance

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub KillApacheHTTPProcesses
{
  my ( $slocalhttpdDir,
       $sResourceName,
       $sConfigFile ) = @_;

  
  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20056, "Subroutine <%s> called with:
  httpdDir        [%s]
  ResourceName    [%s]
  ConfigFile      [%s]",
  $sMyName, $slocalhttpdDir, $sResourceName, $sConfigFile );
  
  my @liPIDsWithPPIDOfOne    = ();
  my @liPIDsWithoutPPIDOfOne = ();
  my $iReturnCode            = 1;

  Apache::GetProcState ( $sConfigFile,
                         $sResourceName,
                         \@liPIDsWithPPIDOfOne,
                         \@liPIDsWithoutPPIDOfOne);
  {
    #-----------------------------------
    # First kill parent httpd processes.
    #-----------------------------------
    if (Proc::KillPidsFromList(\@liPIDsWithPPIDOfOne))
    {
      #----------------------
      # Success - continue
      #----------------------
    }
    else
    {
      &logError(20057, "<%s> Failed to kill all parent processes",
      $sMyName );
      $iReturnCode = 0;
    }
    if (Proc::KillPidsFromList(\@liPIDsWithoutPPIDOfOne))
    {
      #--------------------
      # Success - continue
      #--------------------
    }
    else
    {
      &logError(20058, "<%s> Failed to kill all child processes", $sMyName);
      $iReturnCode = 0;
    }
  }
  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : Int GetMonitorTimeout ( Void );

 Description : Get monitor timeout for Apache resource type

 Parameters :

 None

 Returns : Integer depicting value in seconds
 -------------------------------------------------------------------------

=cut

sub GetMonitorTimeout
{
  my $sEnvFile = "";
  my $bSourceEnv = 0;
  my $sCommand = $sCmdHaType; # Use global..
  my $sCommandArgs = "-value Apache MonitorTimeout";
  my $iCommandExitCode ; 
  my $sUser = "root";
  my @lsCommandOutput;
  my $bTimedOut;
  my $iTimeout = 10;

  #-------------------
  # Announce Entry 
  #-------------------
  my $sMyName = ( caller 0 )[3];

  &logTrace(20059, "Subroutine <%s> called", $sMyName );

  if( Sys::RunWithEnvCmdWithOutputWithTimeOut( $sEnvFile,
                                               $bSourceEnv,
                                               $sCommand,
                                               $sCommandArgs,
                                               \$iCommandExitCode,
                                               $sUser,
                                               \@lsCommandOutput,
                                               \$bTimedOut,
                                               $iTimeout ) )
  {
    &logTrace(20060, "<%s> GetMonitorTimeout is [%s]", $sMyName, $lsCommandOutput[0]);
    return $lsCommandOutput[0];
  }
  else
  {
    if ( $bTimedOut )
    {
      &logError( 20061,
      "<%s> The command line [%s] did not complete within the allotted
       amount of time ( [%s] seconds )",
       $sMyName, $sCommand . $sCommandArgs, $iTimeout );
      return 0;
    }

    &logError(20062,
    "<%s> Errors encountered during call to RunWithEnvCmdWithOutputWithTimeOut",
    $sMyName);
    return 0;
  }
}

1; #so perl knows its included ok
