#
# $Copyrights: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
# $Revision: 1.1.6.12 $
# $Source: /project/acvcs_agunix/src/common/DiskGroupSnap/DiskGroupSnap.pm,v $
#


package DiskGroupSnap;

use Exporter;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default);
@ISA = qw(Exporter);

@EXPORT = qw(&run_cmd &log_resname_and_exit);

$VERSION = '1.00';

require 5.000;
use ag_i18n_inc;

# General return codes
use constant SUCCESS => 0;
use constant ERROR => 1;

my $VXPRINT  = "/usr/sbin/vxprint";
my $VXDISK = "/usr/sbin/vxdisk";
my $VXDG = "/usr/sbin/vxdg";
my $VXVOL = "/usr/sbin/vxvol";

%default = (
	resname       => '',
	targetresname => '',
	targetonline  => '',
	sotype        => '',
	fdsitename    => '',
	thissys       => '',
	diskgroup     => ''
);

my $lockfile_prefix = "$VCSLOG" . "/lock/.VCS_DiskGroupSnap_";
my $lockfile_suffix = "_Online_Lockfile.DO-NOT-REMOVE";

sub new {
	my $proto = shift;
	my ($resName, $targetResName, $fdsitename) = @_;
	my ($grp, $basedir, $sys);
	my $class = ref($proto) || $proto;
	my @tmparr;

	my $self = { %default };
	$self->{'resname'} = $resName;
	$self->{'targetresname'} = $targetResName;
	$self->{'fdsitename'} = $fdsitename;

	#
	# This is just a dummy value, we will recompute the right one during 
	# pitc and resynch. Setting true value here will need a call to the 
	# action entry	
	#
	$self->{'sotype'} = "dummy"; 

	if (run_cmd("$HASYS -nodeid", \@tmparr)) {
		$tmparr[0] =~ /.*Node name:[\s]+(.*)[\s]+Node.*/;
		$self->{'thissys'} = $1;
	} else {
		return undef ;
	}

	bless($self, $class);
	return $self;
}



sub compute_fdtype {
	my ($self) = @_;
	my @tmparr;

	#
	# Firedrill type is bronze if 'fdsitename' is the same as the site name
	# for hosts at the secondary site. Else type is bronze
	#

	if (run_cmd( "$HARES -action $self->{'targetresname'} getvxvminfo -actionargs getsitename -sys $self->{'thissys'}",
		\@tmparr)) 
	{
		my $tmplocalsite = $tmparr[1];
		if ($tmplocalsite eq $self->{'fdsitename'}){
			$self->{'sotype'} = "bronze" ;
		}else{
			$self->{'sotype'} = "gold" ;
		}
	}
}

sub DESTROY { }

sub get_dg_online_node {
	my ($self) = @_;
	my ($grp, $sys, $res);
	my @tmparr = ();
	
	if (run_cmd("$HARES -value $self->{'targetresname'} Group", \@tmparr)) {
		$grp = $tmparr[0];
		if (run_cmd("$HARES -list Group=$grp Type=DiskGroup State=ONLINE"
			, \@tmparr))
		{
			if (scalar @tmparr > 0) {
				($res, $sys) = split(/\s+/, $tmparr[0]);
				$self->{'targetonline'} = $sys;
				run_cmd("$HARES -value $res DiskGroup", \@tmparr);
				$self->{'diskgroup'} = $tmparr[0];
				return 1;
			}
		}
	}

	return undef;
}

sub create_lockfile {
	my ($self) = @_;
	my $LockFile = $lockfile_prefix . $self->{'resname'} . $lockfile_suffix;
	open(lfh, ">$LockFile")
	  || VCSAG_LOG_MSG("E", "Could not create lock $LockFile : $!",
		15502, $LockFile, $!);
	close(lfh);
}

sub remove_lockfile {
	my ($self) = @_;
	my $LockFile = $lockfile_prefix . $self->{'resname'} . $lockfile_suffix;
	unlink("$LockFile");
}

sub lockfile_exists {
	my ($self) = @_;
	my $LockFile = $lockfile_prefix . $self->{'resname'} . $lockfile_suffix;
	if (-f "$LockFile") {
		return 1;
	} else {
		return 0;
	}
}

#
# Creates a point-in-time-copy. If this fails return 0. Else return a non-zero
# value.
#
sub pitc {
	my ($self) = @_;
	my $ret = 0;

	my ($thissys, $resname, $targetres, $fdsitename, $dgonlinesys );

	$thissys = $self->{'thissys'};
	$resname = $self->{'resname'};
	$targetres = $self->{'targetresname'};
	$fdsitename = $self->{'fdsitename'};

	$self->compute_fdtype();
	
	#
	# Proceed only if the base diskgroup is online at some node
	#
	if (defined($self->get_dg_online_node())) {
		$dgonlinesys = $self->{'targetonline'};
		if ( validate_dr_config( $thissys, $resname, $targetres, 
			$fdsitename, $dgonlinesys) ){
			if ($self->{'sotype'} =~ /gold/) {
				$ret = $self->pitc_gold();
			} elsif ($self->{'sotype'} =~ /silver/) {
				$ret = $self->pitc_silver();
			} else {
				my $split = 0;
				$ret = $self->pitc_bronze($split);
			}
		}
	}
	return $ret;
}

sub pitc_bronze {
	my ($self)  = @_;
	my $ret     = 0;
	my @tmparr;

	my $fdsitename       = $self->{'fdsitename'};
	my $targetres = $self->{'targetresname'};
	my $targetsys = $self->{'targetonline'};
	my $diskgroup = $self->{'diskgroup'};

	my $r1 =  run_cmd("$HARES -action  $targetres splitdg -actionargs $fdsitename -sys $targetsys", \@tmparr); 
	my $r2 = run_cmd("$VXDISK scandisks", \@tmparr );
	my $r3 = run_cmd("$VXDG -C -t -n ${diskgroup}_fd -o site=$fdsitename import ${diskgroup}", \@tmparr );

	#
	# Note: since we do not have a DiskGroup attribute anymore, we ensure
	# that the volumes get started as part of DiskGroupSnap online
	# Start all volumes -- Command returns 0 even if volumes are already started
	#
	my $r4 = run_cmd("$VXVOL -g ${diskgroup}_fd startall", \@tmparr);

	if ($r1 && $r2 && $r3 && $r4) {
		$ret = 1;
	}

	return $ret;
}

sub pitc_silver {
	my ($self)  = @_;

	VCSAG_LOG_MSG("E", "This mode is not currently supported", 15503);
	return 0;
}

sub pitc_gold {
	my ($self)  = @_;

	return $self->pitc_bronze(); 
}

#
# Deports the diskgroup if it is imported. Else no action is performed
# returns 1 if the diskgroup is deported, 0 otherwise
#
sub deport_fddg {
	my ($self)  = @_;
	my $ret = 0;
	my @tmparr;
	my $dg = $self->{'diskgroup'} . "_fd";        

	if (run_cmd( "$VXDG list $dg", \@tmparr)) {
		if (run_cmd( "$VXDG deport $dg", \@tmparr)) {
			if (!run_cmd( "$VXDG list $dg", 
				\@tmparr)) 
			{
				$ret = 1;
			}
		}
	}else{
		$ret = 1;
	}

	return $ret;

}


sub resynch {
	my ($self) = @_;
	my $ret = 0;

	$self->compute_fdtype();



	#
	# Proceed only if the base diskgroup is online at some node
	#
	if (defined($self->get_dg_online_node())) {

		#
		# If we do not have a DiskGroup resource, the FD dg would 
		# continue to be online at this point.
		#
		$ret = $self->deport_fddg();
		if ($ret){        
			if ($self->{'sotype'} =~ /gold/) {
				$ret = $self->resynch_gold();
			} elsif ($self->{'sotype'} =~ /silver/) {
				$ret = $self->resynch_silver();
			} else {
				$ret = $self->resynch_bronze();
			}
        }
	}
	return $ret;
}

sub resynch_bronze {
	my ($self)  = @_;
	my $ret     = 0;
	my $snapret = 0;
	my @tmparr;

	my $fdsitename       = $self->{'fdsitename'};
	my $targetres = $self->{'targetresname'};
	my $targetsys = $self->{'targetonline'};

	if (run_cmd(
		"$HARES -action  $targetres joindg -actionargs $fdsitename -sys $targetsys",
		\@tmparr))
	{
		$ret = 1;
	} 

	return $ret;
}

sub resynch_silver {
	my ($self)  = @_;

	VCSAG_LOG_MSG("E", "This mode is not currently supported", 15503);
	return 0;
}

sub resynch_gold {
	my ($self)  = @_;
	return $self->resynch_bronze();
}

#
# Validates the Setup;
# Returns 1 on success, 0 on failure

sub validate_dr_config {

	my ($p_sys, $p_resname, $p_targetres, $p_fdsitename, $p_dgonlinesys) = @_;
	my $retval           = 0;     # Return value from the function
	my @systems          = ();    # All nodes in the cluster
	my @allsites         = ();    # Site ids set on the nodes + fdsitename
	my %zoneToSite       = ();    # Maps the 2 systemzones to site names
	my %sysToZone        = ();    # Maps the nodes to their zones
	my %sysToSite        = ();    # Maps the nodes to their 'real' sites
	my %sysToFDSite      = ();    # Maps the nodes to their 'dummy' site
	my %siteToEnclosures = ();    # Enclosures local to each site
	my %sysToState       = ();    # VCS state for each node
	my $targetgrp;
	my $diskgrp;
	my $drSite;    # The site id of the node running the firedrill
	my $priSite;
	my @syslist          = ();

	# Scratchpad variables
	my ($tmpsys, $tmpstate, $tmpline, $idx, $sgname);
	my @tmparr;
	my @tmparr2;
	my @tmpdgsites;



	#
	# Get list of all nodes in this service group
	#
	if (!run_cmd("$HARES -value $p_resname Group", \@tmparr)) {
		return $retval;
	}
	$sgname = $tmparr[0];

	if (!run_cmd("$HAGRP -value $sgname SystemList", \@tmparr)) {
		return $retval;
	}

	@tmparr2 = split (/\s+/, $tmparr[0]);
	for ($idx=0; $idx<scalar(@tmparr2); $idx+=2) {
		push (@syslist, $tmparr2[$idx]);
	}


	#
	# Get list of all nodes in the cluster
	#
	if (!run_cmd("$HASYS -state", \@tmparr)) {
		return $retval;
	}

	#
	# if state of system == RUNNING, add to list; else error out
	#
	foreach $tmpline (@tmparr) {
		next if ($tmpline =~ /^#.*/);
		($tmpsys, undef, $tmpstate) = split(/\s+/, $tmpline);
		next if (!grep(/$tmpsys/, @syslist));
		$sysToState{$tmpsys} = $tmpstate ; 
	}

	#
	# Check that this node and the target node are still in RUNNING state
	# We need this for action entry points
	#
	if($sysToState{$p_sys} ne "RUNNING" || 
		$sysToState{$p_dgonlinesys} ne "RUNNING" )
	{
		VCSAG_LOG_MSG("E", "Systems $p_sys and $p_dgonlinesys should be in \'RUNNING\' state. Exiting ...", 15505, $p_sys, $p_dgonlinesys);
		return $retval;
	}

	@systems = (keys %sysToState );


	#
	# Get site name for each system; Error out if a system does not have the
	# site id configured
	#
	foreach $tmpsys (@systems) {
		if (!run_cmd( "$HARES -action $p_targetres getvxvminfo -actionargs getsitename -sys $tmpsys",
				\@tmparr)) {
			return $retval;
		} else {
			if (!defined($tmparr[1])) {
				VCSAG_LOG_MSG("E", "No site definition found on $tmpsys. Exiting ...",
				15506, $tmpsys);
				return $retval;

			} else {
				$sysToSite{$tmpsys} = $tmparr[1];
			}
		}
	}

	#
	# drSite = site where the firedrill is to be run
	#
	$drSite = $sysToSite{$p_sys};
	$priSite = $sysToSite{$p_dgonlinesys};

	#
	# Get the system Zone info from the target resource's service group
	# Check System Zone to Site mapping
	#

	if (!run_cmd("$HARES -value $p_targetres DiskGroup", \@tmparr)) {
		return 0;
	} else {
		$diskgrp = $tmparr[0];
	}

	if (!run_cmd("$HARES -value $p_targetres Group", \@tmparr)) {
		return 0;
	} else {
		$targetgrp = $tmparr[0];
	}

	if (!run_cmd("$HAGRP -value $targetgrp SystemZones", \@tmparr)) {
		return 0;
	} else {
		%sysToZone = split(/\s+/, $tmparr[0]);
		if ( scalar (keys  %sysToZone) != scalar (@systems)){
			VCSAG_LOG_MSG( "E",
"SystemZones for  $targetgrp should cover all the systems in the cluster",
				15507, $targetgrp);

			return $retval;

		}
	}

	foreach $tmpsys (@systems) {
		if (!exists($zoneToSite{ $sysToZone{$tmpsys} })) {
			$zoneToSite{ $sysToZone{$tmpsys} } = $sysToSite{$tmpsys};
		} elsif ($zoneToSite{ $sysToZone{$tmpsys} } eq $sysToSite{$tmpsys}) {
			next;
		} else {
			my $tmp_msg_var1 =  $sysToSite{$tmpsys};
			my $tmp_msg_var2 =  $zoneToSite{$sysToZone{$tmpsys}};
			VCSAG_LOG_MSG( "E", "System $tmpsys is in site $tmp_msg_var1, should have been in site $tmp_msg_var2",
				15508, $tmpsys, $tmp_msg_var1, $tmp_msg_var2);

			return $retval;
		}
	}

	#
	# Check for distinct site ids
	#

	if (   (scalar keys %zoneToSite != 2)
		|| ((values %zoneToSite)[0] eq (values %zoneToSite)[1]))
	{
		VCSAG_LOG_MSG("E", "Setup should have 2 SystemZones with distinct site ids ",
			15509);
		return $retval;
	}


	#
	# The diskgroup should have all site ids in its configuration
	#

	@allsites = (values %zoneToSite);
	push(@allsites, $p_fdsitename) if (!grep (/^$p_fdsitename$/, @allsites));

	@tmpdgsites = get_sites_dg($diskgrp, $p_dgonlinesys, $p_targetres);
	if (!is_superset(\@tmpdgsites, \@allsites)) {
		my $tmp_msg_var1 = join(" ", @allsites);
		VCSAG_LOG_MSG("E",
"DiskGroup $diskgrp should have all of the following sites in ACTIVE state in its configuration: $tmp_msg_var1",
		15510, $diskgrp, $tmp_msg_var1);
		return $retval;
	}

	#
	# FDSiteName should be the same across all nodes in the system zone.
	# The system zone from the base service group is considered here
	# This check is needed only if FDSiteName is a dummy site. For a valid site,
	# this check has already been done.
	#

	if (!grep(/^$p_fdsitename$/, (values %zoneToSite))) {
		foreach my $tmpsys (@systems) {
			if ( !run_cmd( "$HARES -value $p_resname FDSiteName $tmpsys",
				 \@tmparr)){
				return $retval;
			} else {
				$sysToFDSite{$tmpsys} = $tmparr[0];
			}
		}

		#
		# Check that the FDSite is the same on all nodes in this System Zone
		#
		my $tmpthiszone = $sysToZone{$p_sys};
		foreach $tmpsys (@systems) {

			#
			# All nodes in this zone should have FDSiteName set to
			# $p_fdsitename
			#
			if ($sysToZone{$tmpsys} eq $tmpthiszone) {
				if ($sysToFDSite{$tmpsys} ne $p_fdsitename) {
					VCSAG_LOG_MSG( "E", "System $tmpsys in Zone $tmpthiszone should have FDSiteName set to $p_fdsitename",
						15511, $tmpsys, $tmpthiszone,
						$p_fdsitename);
					return $retval;
				}
			} else {

				#
				# No node in this zone should have FDSiteName set to
				# $p_fdsitename
				#
				if ($sysToFDSite{$tmpsys} eq $p_fdsitename) {
					VCSAG_LOG_MSG( "E", "System $tmpsys is not in Zone $tmpthiszone but has FDSiteName set to $p_fdsitename",
						15512, $tmpsys, $tmpthiszone,
						$p_fdsitename);
					return $retval;
				}

			}
		}

	}

	#
	# Get the local enclosure names for the RealSites and the FDSiteName
	# Verify that RealSites do not share any enclosure
	# Verify that FDSite does not share any enclosure with the primary site

	if(!get_site_enclosures(\@allsites, \%siteToEnclosures, $p_targetres, 
		$diskgrp, $p_dgonlinesys)) {

		return $retval;
	}

	my @tmppri_enclrs = (@{$siteToEnclosures{$priSite}});
	my @tmpdr_enclrs  = (@{$siteToEnclosures{$drSite}});

	if (!is_disjoint(\@tmppri_enclrs, \@tmpdr_enclrs)) {
		VCSAG_LOG_MSG("E", "Sites: $priSite(primary site) and $drSite(secondary site) have common enclosures", 
				15513, $priSite, $drSite);
		return $retval;
	}
	if ($p_fdsitename ne $drSite) {
		my @tmpfd_enclrs = (@{$siteToEnclosures{$p_fdsitename}});
		if (!is_disjoint(\@tmppri_enclrs, \@tmpfd_enclrs)) {
			VCSAG_LOG_MSG("E", "Sites: $priSite(primary site) and $p_fdsitename(firedrill site) have common enclosures",
				15514, $priSite, $p_fdsitename);
			return $retval;
		}
	}

	#
	# Check that all volumes on the diskgroup have a plex on each site
	#

	if ( !check_vols_sites_sanity (  $diskgrp, $p_dgonlinesys, $p_targetres, \@allsites) ){
		VCSAG_LOG_MSG("E", "Volumes do not have plexes at all sites",
			15515);
		return $retval;
	}

	# All checks passed
	$retval = 1;

	return $retval;
}

sub run_cmd {
	my ($cmd, $output_ref) = @_;
	my $retval = 1;

	@{$output_ref} = `$cmd`;

	if ($? != 0) {
		$retval = 0;
		VCSAG_LOG_MSG("W", "Command: \'$cmd\' failed !", 15516, "$cmd");
	} else {
		chomp @{$output_ref};
	}

	return $retval;
}

#
# Return the list of sitenames added to this diskgroup
#

sub get_sites_dg {
	my ($diskgrp, $dgonlinesys, $targetres) = @_;
	my @sites = ();
	my @output;
	my @tmparr;
	my $retval = undef;

	if ( !run_cmd(
"$HARES -action $targetres getvxvminfo -actionargs getdgsites $diskgrp -sys $dgonlinesys",
		\@tmparr)) {
		return $retval;
	} else {
		if (!defined($tmparr[1])) {
			VCSAG_LOG_MSG("E", "No sites found in ${diskgrp}. Exiting ...", 
			15517, ${diskgrp});
			return $retval;

		} else {
			@sites = split(/\s+/, $tmparr[1]);
		}
	}

	return @sites;
}


#
# Check that all volumes in the disk group have a plex on all sites
#


sub check_vols_sites_sanity {
	my ($diskgrp, $dgonlinesys, $targetres, $allsitesref) = @_;
	my @allsites =  @{$allsitesref} ;
	my @tmparr;
	my $retval = 0;
	my @arr;
	my @sites;
	my @sitestrs;
	my $vol;


	if ( !run_cmd(
	"$HARES -action $targetres getvxvminfo -actionargs getplexsites $diskgrp -sys $dgonlinesys",
		\@tmparr)) {
		return $retval;
	} else {
		if (!defined($tmparr[1])) {
			VCSAG_LOG_MSG("E", "No plex info found on volumes in ${diskgrp}. Exiting ...", 
			15518, ${diskgrp});
			return $retval;
		} else {
			@sitestrs = split(/;/, $tmparr[1]);

			#
			# Each line looks like this: <vol1>:site1:site2:..;
			# pos 0 is the volume name; everything after that is
			# an enclosure
			#
			foreach (@sitestrs) {
				@arr = split(/:/, $_);
				$vol = $arr[0];
				@sites = splice(@arr, 1);
				#
				# ${$vTShashref}{ $vol } = [@sites];
				#
				if (!is_superset(\@sites, \@allsites)) {
					if (!is_layeredvol($vol, $diskgrp, $dgonlinesys,
						$targetres) || !is_allsites_set($vol, $diskgrp, 
						$dgonlinesys, $targetres)) {

						return $retval;
					}		
				}
			} 
		}
	}

	$retval = 1;
	
	return $retval;
}


#
# Compares 2 arrays passed by reference;
# If the 1st array is a super-set of the 2nd, return 1
# Return 0 otherwise
#
sub is_superset {
	my ($arr_ref1, $arr_ref2) = @_;
	my $elem ;
	my @arr1 = @{$arr_ref1};
	my @arr2 = @{$arr_ref2};

	foreach $elem (@arr2) {
		if (!grep(/^$elem$/, @arr1)) {
			return 0;
		}
	}

	return 1;
}


sub get_site_enclosures {
	my ($sitesref, $sTEhashref, $targetres, $diskgrp, $dgonlinesys) = @_;
	my @arr      = ();
	my @enclrs   = ();
	my @sitestrs = ();
	my @tmparr   = ();
	my $retval   = 0;

	if (!run_cmd(
"$HARES -action $targetres getvxvminfo -actionargs getsiteenclrs $diskgrp -sys $dgonlinesys",
			\@tmparr)) {
		return $retval;
	} else {
		if (!defined($tmparr[1])) {
			VCSAG_LOG_MSG("E", "No enclosures found in ${diskgrp}. Exiting", 
			15519, ${diskgrp});
			return $retval;

		} else {

			@sitestrs = split(/;/, $tmparr[1]);

			#
			# Each line looks like this: <site1>:enc1:enc2:..;
			# pos 0 is the site name; everything after that is
			# an enclosure
			#
			foreach (@sitestrs) {
				@arr = split(/:/, $_);
				@enclrs = splice(@arr, 1);
				${$sTEhashref}{ $arr[0] } = [@enclrs];
			}
			$retval = 1;
		}
	}
	return $retval;
}

#
# Checks if 2 arrays have any common elements.
# Returns 1 if yes, 0 if there are common elements (not disjoint );
#

sub is_disjoint {
	my ($arr_ref1, $arr_ref2) = @_;

	foreach my $member (@{$arr_ref1}) {
		return 0 if (grep(/^$member$/, @{$arr_ref2}));
	}

	return 1;
}


sub log_resname_and_exit {
	my $ep     = shift;
	my $attr   = shift;

	VCSAG_LOG_MSG("E",
		"$ep terminated abnormally as the $attr attribute was missing",
	15520, "$ep", "$attr");

	exit 0;
}

#
# returns true/1 if the volume has sub-volumes underneath it
# false/0 otherwise
#
sub is_layeredvol {
	my ($volname, $diskgrp, $dgonlinesys, $targetres) = @_;
	my @tmparr = ();

	if (!run_cmd(
		"$HARES -action $targetres getvxvminfo -actionargs islayeredvol $volname $diskgrp -sys $dgonlinesys",
		\@tmparr)) {
		return 0;
	}else{
		return $tmparr[0];
	}
}


#
# returns true/1 if the volume has the allsites flag set
# false/0 otherwise
#
sub is_allsites_set {
	my ($volname, $diskgrp, $dgonlinesys, $targetres) = @_;
	my @tmparr = ();

	if (!run_cmd(
		"$HARES -action $targetres getvxvminfo -actionargs isallsitesset $volname $diskgrp -sys $dgonlinesys",
		\@tmparr)) {
		return 0;
	}else{
		return $tmparr[0];
	}
}


sub validate_args {
	my $href = shift;
	my $ret = 0;
	my $attrib;

	my $targetresname;
	($ret, $targetresname) = VCSAG_GET_ATTR_VALUE("TargetResName", -1, 1,@ARGV);
	if ($ret != $VCSAG_SUCCESS || !length $targetresname || 
		($targetresname =~ /[\s]+/)) {

		$attrib = "TargetResName";
		VCSAG_LOG_MSG("E", "'$attrib' attribute is either absent or incorrect", 
			15528, "$attrib");
		return ERROR;
	}
	$href->{"TargetResName"} = $targetresname ;

	my $fdsitename;
	($ret, $fdsitename) = VCSAG_GET_ATTR_VALUE("FDSiteName", -1, 1, @ARGV);
	if ($ret != $VCSAG_SUCCESS || !length $fdsitename || 
		($fdsitename =~ /[\s]+/)) {
		$attrib = "FDSiteName";
		VCSAG_LOG_MSG("E", "'$attrib' attribute is either absent or incorrect", 
			15528, "$attrib");
		return ERROR;
	}
	$href->{"FDSiteName"} = $fdsitename ;

	return SUCCESS;
}


1;


