package NFSFunctions;
use Exporter;
@ISA = ('Exporter');
@EXPORT = ('getBasename', 'getProcPid', 'getProcOptions', 'AcquireLock', 'ReleaseLock', 'StartNFS', 'StopNFS', 'AddGroup', 'DelGroup', 'StartLockDaemons', 'AddGroup1', 'NFSisRunning');

use strict;
use warnings;
use Fcntl qw(:flock);
use POSIX;

use ag_i18n_inc;
#This file contains helper functions for NFS Agent
my ($lockd, $nfsd, $mountd, $statd) = ("rpc.lockd", "rpc.nfsd", "rpc.mountd", "rpc.statd");
my ($idmapd, $gssd, $rpcinfo) =  ("rpc.idmapd", "rpc.svcgssd", "/usr/sbin/rpcinfo");
my ($sm_notify, $kill, $path_statd) = ("/sbin/sm-notify", "kill", "/sbin/rpc.statd");
my ($sysctl, $modprobe, $ps) = ("sysctl", "modprobe", "ps");
my ($counter, $suse_version, $no_lockd, $no_statd, %bmap) = ();
my ($rpcbind, $portmap) = ("/sbin/rpcbind", "/sbin/portmap");

my ($rmmod) = ("rmmod");
my ($exportfs, $umount) = ("exportfs", "umount");
my ($bnfsd, $blockd) = ("nfsd", "lockd");
my ($bmountd, $bstatd) = ("rpc.mountd", "rpc.statd");
my ($bidmapd, $bgssd) = ("rpc.idmapd", "rpc.svcgssd");
# order in which they will be unloaded
my (@nfs_mods) = ("nfs", "nfsd","lockd");

$bmap{$nfsd} = $bnfsd;
$bmap{$lockd} = $blockd;
$bmap{$mountd} = $bmountd;
$bmap{$statd} = $bstatd;

#
# Didn't want to call it basename() since there already is a
# library module with that name.  Since the 'strict' module wasn't
# available with the VxCS Perl I'm assuming that the File::Basename
# module isn't either.


sub start_daemon {

	my ($daemon, @portarr) = @_;

	if (scalar @portarr == 0) {
		my $retout = `$daemon`;
		my $retval = $? >> 8;
		VCSAG_LOGDBG_MSG(4, "Output for starting daemon [$daemon]: [$retout]");
		if ($retval != 0) {
			VCSAG_LOG_MSG("W", "Could not start the daemon [$daemon]", 7032, "$daemon");
		}
		return;
	}
	my ($ports, $counter, $procpid) = (0, 0, 0);
	$ports = @portarr;
	while ($counter < $ports) {
		my $retout = `$daemon -p $portarr[$counter]`;
		my $retval = $? >> 8;
		$procpid = 0;
		VCSAG_LOGDBG_MSG(4, "Output for starting daemon [$daemon]: [$retout]");
		if ($retval == 0) {
			# Looks like the daemon was started successfully. Lets check it...
			$daemon =~ s/^\s*(\S+)\s+.*/$1/;
			$daemon =~ s/.*\/(\S+)/$1/;
			$procpid = getProcPid ($bmap{$daemon});
			last if ($procpid);
		}
		if (!$procpid) {
			VCSAG_LOG_MSG("W", "Could not start the daemon [$daemon] on port [$portarr[$counter]]", 7033, "$daemon", "$portarr[$counter]");
		}
		$counter++;
	}
}


#This function deletes group name 
#from the file ,updates its time stamp and removes it in case
#this was the last group name in the file
#NOTE: isgroup is a flag which is 1 when name contains a group name
# 0 o.w


sub DelGroup
{
    my ($name, $nfs_sync_file, $isgroup)=(@_);
    my ($syncfile_lock) = "$VCSLOG/lock/.nfs_syncfile_lock"; 
    my ($grouplist, $group);
    if(! $isgroup)
    {
        chomp(my $op = `$HASYS -nodeid`);
        $op =~ /: (\S+)/;
        my $sys = $1;
        $group=`$HARES -value $name Group $sys`;
        chomp($group);
    }
    else
    {
        $group = $name;
    }

    my ($SYNCLOCK) = AcquireLock($syncfile_lock);
    if(-e $nfs_sync_file)
    {
        $grouplist=`cat $nfs_sync_file`;
        chomp($grouplist);
        if($grouplist =~ s/:$group://)
        {
            VCSAG_LOGDBG_MSG(4, "GroupList changed to \"$grouplist\" after removing $group", $grouplist, $group);
        }
        #check if last group
        if( $grouplist =~ m/:/)
        {
            `echo $grouplist > $nfs_sync_file`;
        }
        else
        {
            `unlink $nfs_sync_file`;
        }
    }
    ReleaseLock($SYNCLOCK);
}

#This function adds group name of the parameter resource
#to the file, updates its time stamp
sub AddGroup
{
    my ($resname, $nfs_sync_file)=(@_);
    my ($group, $grouplist);
    my ($syncfile_lock) = "$VCSLOG/lock/.nfs_syncfile_lock";

    my ($SYNCLOCK) = AcquireLock($syncfile_lock);
    `touch $nfs_sync_file`;
    $grouplist=`cat $nfs_sync_file`;
    chomp($grouplist);
    chomp(my $op = `$HASYS -nodeid`);
    $op =~ /: (\S+)/;
    my $sys = $1;
    $group=`$HARES -value $resname Group $sys`;
    chomp($group);

    if($grouplist=~ m/:$group:/)
    {
        VCSAG_LOGDBG_MSG (1, "Group $group is already present in the grouplist=\"$grouplist\"", $grouplist, $group);
    }
    else
    {
        $grouplist="$grouplist" . " ". ":$group:" ;
        VCSAG_LOGDBG_MSG (1, "GroupList changed to $grouplist after adding $group", $grouplist, $group);
        `echo $grouplist > $nfs_sync_file`;
    }
    ReleaseLock($SYNCLOCK);
}


#
# Didn't want to call it basename() since there already is a
# library module with that name.  Since the 'strict' module wasn't
# available with the VxCS Perl I'm assuming that the File::Basename
# module isn't either.
sub getBasename
{
    my ($i, $cmd) = (0, $_[0]);

    $i = rindex ($cmd, "/");
    $cmd = substr $cmd, $i+1 if ($i != -1);

#    $i = rindex ($cmd, ".");
#    $cmd = substr $cmd, $i+1 if ( $i != -1);

    chomp ($cmd);
    return ($cmd);
}


# getProcPid ()
#
# Didn't want to call it getpid() because perl might have a
# getpid() function one day (it has just about everything
# else).
sub getProcPid
{
    my ($target) = @_;
    my ($pid, $cmd, @tmp)=("","");

    open (PIDS, "$ps -ea -o comm,pid | grep $target 2> /dev/null |");
    while (<PIDS>)
    {
        @tmp = split;
        ($cmd) = getBasename ($tmp[0]);
        if ($cmd eq $target)
        {
            if (length $pid)
            {
                #to handle situations where output of ps has entry of the form
                #monitor <defunct> 1113
                $pid = sprintf ("%s %s", $pid, $tmp[$#tmp]);
            }
            else
            {
                $pid = $tmp[$#tmp];
            }
        }
    }
    close (PIDS);
    return ($pid);
}

# getProcOptions ()
# Get options associated with the command
sub getProcOptions
{
    my ($target) = $_[0];
    my ($pid, $proc, @proc_options, $junk, @tmp, $cmd);

    open (PIDS, "$ps -ea -o comm,args | grep $target 2> /dev/null |");
    while (<PIDS>)
    {
        @tmp = split;
        ($cmd) = getBasename ($tmp[0]);
        if ($cmd eq $target)
        {
            ($junk, $junk, @proc_options) = @tmp;
            $proc = join (" ", @proc_options);
            last;
        }
    }
    close (PIDS);
    return ($proc);
}

sub AcquireLock
{
    my ($lock_file) = (@_);
    local *LHANDLE;
    my ($arg, $ret_val) = ("", 0);

    open (LHANDLE, ">$lock_file");
    $arg = pack ("SSLLLLLLLL", F_WRLCK (), 0, 0, 0, 0, 0, 0, 0, 0, 0);
    $ret_val = fcntl(LHANDLE, F_SETLKW (), $arg);

    if ($ret_val < 0)
    {
        VCSAG_LOG_MSG ("E", "Could not get lock on file $lock_file.", 7001, "$lock_file");
        exit 0;
    }

    return *LHANDLE;
}

sub ReleaseLock
{
    close ($_[0]);
}

sub StartLockDaemons
{
	my ($address, $portref) = @_;
	my ($procpid);

	$no_statd = 0;

    if( -e "/etc/SuSE-release")
    {
    	$suse_version = `cat "/etc/SuSE-release" | head -1`;
	if ($suse_version =~ /SUSE Linux Enterprise Server 11/) {
		$no_lockd = 1;
	}
	elsif ($suse_version =~ /SUSE Linux Enterprise Server 10/) {
		$no_statd = 1;
	}

    }

	my (%port, @statd_portarr) = ();
	%port = %$portref if (%$portref);
	if (defined($port{"STATD"})) {
		@statd_portarr = split (/,/, $port{"STATD"});
	}

    if (!$no_statd) {
	    if ( ! -x "$path_statd" ) {
		    $path_statd = "/usr/sbin/rpc.statd";
	    }
	    if ( -x "$path_statd" ) {
		    $procpid = getProcPid ("statd");
		    system ("$kill -9 $procpid") if ($procpid);
		    VCSAG_LOGDBG_MSG (4, "Starting $path_statd...", $path_statd);
			my $statd_args = $path_statd;
		    if ($address) {
			    $statd_args = "$path_statd -n $address";
		    }
		    start_daemon($statd_args, @statd_portarr);
	    }
	    else {
		    VCSAG_LOG_MSG ("W", "Could not find executable for statd/sm-notify", 7019);
	    }
    }

    if ($suse_version) {
	    if ( ! -x "$sm_notify" ) {
		    $sm_notify = "/usr/sbin/sm-notify";
	    }
	    elsif ( -x "$sm_notify" ) {
		    if ($address) {
			    $procpid = getProcPid ("sm-notify");
			    system ("$kill -9 $procpid") if ($procpid);
			    VCSAG_LOGDBG_MSG (4, "Starting $sm_notify...", $sm_notify);
			    system ("$sm_notify -v $address");
		    }
	    }
    }
    if (!$no_lockd) {
	    start_daemon("$lockd");
    }


}

sub StopNFS
{
    my ($num_nfsd, $address, $grace_period, $nfs_security, $nfsv4_support) = (@_);

    my (@nd, @d, @o, $n, $proc, $pids, $killed_all, $ret_val, $mod, $i);
    @nd = ("$nfsd", "$mountd", "$lockd", "$statd", $gssd, $idmapd);
    @d = ("$bnfsd", "$bmountd", "$blockd", "$bstatd", "$bgssd", "$bidmapd");

    foreach $n (0..$#nd)
    {
        $pids = getProcPid ($d[$n]);
        $o[$n] = getProcOptions ($d[$n]);
        system ("$kill $pids") if ($pids);
    }

    foreach $n (0..$#nd)
    {
        $proc = $d[$n];
        $pids = getProcPid ($proc);
        if ($pids)
        {
            VCSAG_LOG_MSG ("W", "Sending SIGKILL signal to $proc ($pids).", 7002, $proc, $pids);
            system ("$kill -9 $pids");
        }
    }

    # check if all nfs daemons are killed. if yes, unload nfs modules.
    $killed_all = 1;
    foreach $n (0..$#nd)
    {
        if($bstatd eq $d[$n])
        {
                next if(-e "/etc/SuSE-release");
        }
        if($bidmapd eq $d[$n])
        {
                next if($nfsv4_support <= 0);
        }
        if($bgssd eq $d[$n])
        {
                next if($nfs_security <= 0);
        }
        $proc = $d[$n];
        $pids = getProcPid ($proc);
        if ($pids)
        {
            $killed_all = 0;
            VCSAG_LOG_MSG ("W", "NFS daemon ($proc) is still running. NFS modules will not be unloaded.", 7003, $proc);
            last;
        }
    }
    #Remove modules if daemons are killed
    if ($killed_all)
    {
        for ($i = 0; $i <= $#nfs_mods; $i++)
        {
            $mod = $nfs_mods[$i];

            #no lockd,nfs module on SuSE
            next if(("$mod" eq "lockd") && (-e "/etc/SuSE-release"));
            next if(("$mod" eq "nfs") && (-e "/etc/SuSE-release"));

            system ("$rmmod $mod 2>/dev/null");
        }
    }
}

sub StartNFS
{
    my ($num_nfsd, $address, $grace_period, $nfs_security, $nfsv4_support, $portref) = (@_);
    my ($svc, $cmd, $options, $nfs_help, $proc_pid) = ("", "", "", "", "", "");

	# Need to start rpcbind before starting all RPC daemons
	# Check for rpcbind and start it if not running
    if ( -x "$rpcbind" ) {
	    $proc_pid = getProcPid ("rpcbind");
		if (!$proc_pid) {
		    VCSAG_LOGDBG_MSG (4, "Starting $rpcbind...", $rpcbind);
		    system ("$rpcbind");
		}
     }
	elsif (-x "$portmap") {
		$proc_pid = getProcPid ("portmap");
		if (!$proc_pid) {
			VCSAG_LOGDBG_MSG (4, "Starting $portmap...", $portmap);
			system ("$portmap");
		}
	}

    # Start nfsd
    system("mount -t nfsd nfsd /proc/fs/nfsd");
    $svc = sprintf "$nfsd  %d", $num_nfsd;
    if (! $nfsv4_support)
    {
        $options  = " --no-nfs-version 4";
		`echo +2 +3 -4 > /proc/fs/nfsd/versions 2> /dev/null`;
    }
	else {
		`echo +2 +3 +4 > /proc/fs/nfsd/versions 2> /dev/null`;
	}

    # Set grace period. Before setting this nfs module
    # should be loaded o.w proc entries not avialable.
    #This should be done only if address is non-null 
    if($address)
    {
        system ("$modprobe nfsd");
        system ("$sysctl -w fs.nfs.nlm_grace_period=$grace_period");
    }


	# Check for ports
	my (%port, @nfs_portarr, @mount_portarr, @lockd_portarr) = ();
	%port = %$portref if (%$portref);
	if (defined($port{"NFSD"})) {
		@nfs_portarr = split (/,/, $port{"NFSD"});
	}
	if (defined($port{"MOUNTD"})) {
		@mount_portarr = split (/,/, $port{"MOUNTD"});
	}
	if (defined($port{"LOCKD"})) {
		@lockd_portarr = split (/,/, $port{"LOCKD"});
		my ($tcpport, $udpport) = (0, 0);
		$tcpport = $lockd_portarr[0] if ($lockd_portarr[0]);
		$udpport = $lockd_portarr[1] if ($lockd_portarr[1]);
		if ($udpport == 0) {
			$udpport = $tcpport;
		}
		system ("$sysctl -w fs.nfs.nlm_tcpport=$tcpport");
		system ("$sysctl -w fs.nfs.nlm_udpport=$udpport");
	}

    # Check if nfs has a support for --no-nfs-version
    # Earlier versions of rhel4 and sles systems dont have this option by default
    $nfs_help = `$nfsd -h 2>&1`;
    if ($nfs_help =~ /--no-nfs-version/)
    {
		$svc = $svc . $options;
    }

	start_daemon($svc, @nfs_portarr);

    # Start mountd
	my $mountd_args = "$mountd $options";
	start_daemon($mountd_args, @mount_portarr);
    my ($procpid) = getProcPid ("mountd");
    if ($procpid =~ /([0-9]*)\s+(.*)/) {
	    system ("$kill -9 $2") if ($2);
    }


    StartLockDaemons($address, \%port);

    if($nfsv4_support)
    {
		# Check for pipefs directory
		my $pipefs_dir = `grep Pipefs-Directory /etc/idmapd.conf 2>/dev/null`;
		if ($pipefs_dir) {
			$pipefs_dir =~ s/.*=\s*(\S+)/$1/;
			chomp($pipefs_dir);
		}
		else {
			$pipefs_dir = "/var/lib/nfs/rpc_pipefs";
		} 
		if ( -d "$pipefs_dir/nfs") {
			# Nothing to be done
		}
		else {
			VCSAG_LOGDBG_MSG (3, "Mounting the pipefs filesystem [$pipefs_dir]...", 0);
    		system("mount -t rpc_pipefs rpc_pipefs $pipefs_dir");
		}
	
        system("$idmapd");
    }
    if($nfs_security)
    {
        system("$modprobe rpcsec_gss_krb5");
        system("$gssd");
    }
}

#This function adds group name (passed as a argument)
#to the file, updates its time stamp.

sub AddGroup1 {
		my ($group, $nfs_sync_file)=(@_);
		my ($syncfile_lock) = "$VCSLOG/lock/.nfs_syncfile_lock";

		my ($SYNCLOCK) = AcquireLock($syncfile_lock);

		open(DAT, $nfs_sync_file);
		my @raw_data=<DAT>;
		close(DAT);

		$raw_data[0]=time() . "\n";

		open(DAT,">$nfs_sync_file");
		print DAT @raw_data;
		close(DAT);

		my $i=1;
		my $found=0;

		while($raw_data[$i]) {
			my $temp=$raw_data[$i];
			if($temp eq ("$group" . "\n")) {
				$found = 1;
				last;
			}
			$i = $i+1;
		}

		if(!$found) {
			$raw_data[$i]="$group" . "\n";
			open(DAT,">$nfs_sync_file");
			print DAT @raw_data;
			close(DAT);
		}

		ReleaseLock($SYNCLOCK);
}

sub NFSisRunning {
	
	my ($nfsv4support, $nfssecurity, $portref) = @_;

    my @nd = ("$nfsd", "$mountd", "$lockd", "$statd");
    my @d = ("$bnfsd", "$bmountd", "$blockd", "$bstatd");
	my ($pids, $n, $iStart, $iStop, $cnt, $ret, %port, %daemonmap) = ();
	my ( $sKey, @lsTemp, %hsHash, %hsVers, %hsPort, @aVersProto, $sles10 ) = ();

	# For SLES 10
	$sles10 = 0;
    if( -e "/etc/SuSE-release")
    {
    	my $suse_version = `cat "/etc/SuSE-release" | head -1`;
		if (($suse_version) && ($suse_version =~ /SUSE Linux Enterprise Server 10/)) {
			@nd = ("$nfsd", "$mountd", "$lockd");
			$sles10 = 1;
		}
	}

	# Test for RPC calls
	
	if (!open (RPCINFO, "$rpcinfo -p 2>&1 |")) {
		$ret = $!;
		VCSAG_LOG_MSG("E", "Unable to execute command [$rpcinfo]. Error: $ret",
			7024, "$rpcinfo", "$ret");
		return 0;
	}

	while ( <RPCINFO> ) {
		s/\n//; # Chomp..
		s/^\s+//;
		next if /^program/;
		VCSAG_LOGDBG_MSG (4, "[$rpcinfo] returned: [$_]");
		@lsTemp = $_ =~ m/[^\s]+/g;
		$sKey = $lsTemp [ -1 ];
		$hsHash { $sKey } = 1;
		if ($hsVers { $sKey }) {
			$hsVers { $sKey } =  $lsTemp [ 1 ] . ":" . 
					$lsTemp [ 2 ] . "," . $hsVers { $sKey };
			$hsPort { $sKey } =  $lsTemp [ 3 ] . ":" . 
					$lsTemp [ 2 ] . "," . $hsPort { $sKey };
		}
		else {
			$hsVers { $sKey } =  $lsTemp [ 1 ] . ":" .  $lsTemp [ 2 ];
			$hsPort { $sKey } =  $lsTemp [ 3 ] . ":" .  $lsTemp [ 2 ];
		}
	}
	close (RPCINFO);

	$ret = $? >> 8;
	if ($ret) {
		VCSAG_LOG_MSG("E", "Unable to contact portmapper. Error: [$ret]",
			7025, "$ret");
		return 0;
	}

	%port = %$portref if (%$portref);
	foreach $sKey ( keys %hsHash ) {
		VCSAG_LOGDBG_MSG (4, "RPC checking for daemon [$sKey]...");
		if (!$hsVers { $sKey }) {
			VCSAG_LOG_MSG ("E", "RPC call for daemon [$sKey] failed.",
				7026, "$sKey");
			return 0;
		}
		@aVersProto = split (/,/, $hsVers { $sKey });
		if ($sKey eq "nfs") {
			$iStart = 2;
			$iStop = 3;
			if ($nfsv4support) {
				$iStop = 4;
			}
			$daemonmap { $sKey } = "NFSD";
		}
		elsif ($sKey eq "mountd") {
			$iStart = 1;
			$iStop = 3;
			$daemonmap { $sKey } = "MOUNTD";
		}
		elsif ($sKey eq "status") {
			$iStart = 1;
			$iStop = 1;
			$daemonmap { $sKey } = "STATD";
		}
		elsif ($sKey eq "nlockmgr") {
			$iStart = 1;
			$iStop = 4;
			$daemonmap { $sKey } = "LOCKD";
		}
		else {
			next;
		}

		foreach $cnt ( $iStart..$iStop) {
			next if (($sKey eq "nlockmgr") && ($cnt == 2));
			if ($hsVers { $sKey } !~ /$cnt:tcp/) {
				VCSAG_LOG_MSG ("E", "Daemon [$sKey] not running version [$cnt] on TCP port",
					7027, "$sKey", "$cnt");
				return 0;
			}
			elsif ($hsVers { $sKey } !~ /$cnt:udp/) {
				VCSAG_LOG_MSG ("E", "Daemon [$sKey] not running version [$cnt] on UDP port",
					7028, "$sKey", "$cnt");
				return 0;
			}
		}

		my ($sDaemon, @portarr, $tcpport, $udpport) = ();
		$sDaemon = $daemonmap { $sKey };
		$tcpport = 0;
		$udpport = 0;
		if (defined($port{$sDaemon})) {
			next if (($sDaemon eq "STATD") && ($sles10 == 1));
			@portarr = split (/,/, $port{$sDaemon});
			foreach my $portnum (@portarr) {
				if ($hsPort { $sKey } =~ /$portnum:tcp/) {
					$tcpport = 1;
				}
				if ($hsPort { $sKey } =~ /$portnum:udp/) {
					$udpport = 1;
				}
			}
			if ($tcpport == 0) {
				 VCSAG_LOG_MSG ("W", "Daemon [$sDaemon] not running on configured TCP ports [$port{$sDaemon}]",
					7034, "$sKey", "$port{$sDaemon}");
			}
			if ($udpport == 0) {
				 VCSAG_LOG_MSG ("W", "Daemon [$sDaemon] not running on configured UDP ports [$port{$sDaemon}]",
					7035, "$sKey", "$port{$sDaemon}");
			}
		}
	}

	# Test for NFS, Mount, Status, and Lock daemons in process table
    foreach $n (0..$#nd)
    {
        $pids = getProcPid ($d[$n]);
        if (!$pids) {
			VCSAG_LOG_MSG ("E", "Daemon [$nd[$n]] is not running.",
				7029, "$nd[$n]");
			return 0;
		}
		VCSAG_LOGDBG_MSG (4, "PID returned: [$pids] for daemon [$nd[$n]]");
    }

	# Test for NFSv4
	if ($nfsv4support) {
		$pids = getProcPid($idmapd);
		if (!$pids) {
			VCSAG_LOG_MSG ("E", "Daemon [$idmapd] is not running.",
				7030, "$idmapd");
			return 0;
		}
		VCSAG_LOGDBG_MSG (4, "PID returned: [$pids] for daemon [$idmapd]");
	}

	# Test for NFSSecurity
	if ($nfssecurity) {
		$pids = getProcPid($gssd);
		if (!$pids) {
			VCSAG_LOG_MSG ("E", "Daemon [$gssd] is not running.",
				7031, "$gssd");
			return 0;
		}
		VCSAG_LOGDBG_MSG (4, "PID returned: [$pids] for daemon [$gssd]");
	}

	return 1;
}

1;

