/* 
 * $Copyright: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $ 
 */

#ifndef	_VCSAgApi_h_
#define	_VCSAgApi_h_

static const char vcsagapi_cvsid[] = "$RCSfile: VCSAgApi.h,v $ $Revision: 1.12.2.2 $";

//
// Contains the agent framework API definitions for use by the agent
// developer.
//
#ifdef	WINNT
#ifdef	__ENABLE_L10N_INPUT__
#include <wchar.h>
#endif	/* WINNT && __ENABLE_L10N_INPUT__ */
#endif	/* WINNT */
#include "VCSAgApiDefs.h"

/* 
 * Macros for exporting, importing & calling convention. 
 * Meaningful in NT only.
 */
#if (defined(WINNT) || defined(WIN32))	/* NT definitions */
#define VCSAGEXPORT __declspec(dllexport)
#define VCSAGIMPORT __declspec(dllimport)
#define VCSAGCCON __cdecl
#else	/* Unix Definitions */
#define VCSAGEXPORT 
#define VCSAGIMPORT 
#define VCSAGCCON 
#endif	/* Unix Definitions */

/*
 * Specify the linkage for the API functions as export or import depending
 * on whether it is included from the framework or from the agent developer's
 * code.
 */
#ifdef	VCSAGDLL
#define VCSAGLINKAGE VCSAGEXPORT
#else	/* !VCSAGDLL */
#define VCSAGLINKAGE VCSAGIMPORT
#endif	/* !VCSAGDLL */

#ifdef  __cplusplus
extern "C" {
#endif	/* __cplusplus */

	VCSAGLINKAGE void VCSAGCCON VCSAgCloseFile(void *vp);
	VCSAGLINKAGE void VCSAGCCON VCSAgDelString(void *vp); 
	VCSAGLINKAGE int VCSAGCCON VCSAgExec(const char *path, 
		char *const argv[], char *buf, long buf_size, 
		unsigned long *exit_code);
#ifndef	WINNT
	/* Not supported on windows yet */
	VCSAGLINKAGE int VCSAGCCON VCSAgExecWithTimeout(const char *path,
		char *const argv[], unsigned int timeout, char *buf, 
		long buf_size, unsigned long *exit_codep);
#endif	/* !WINNT */

	VCSAGLINKAGE void VCSAGCCON VCSAgSendTrap(const char *msg);
	VCSAGLINKAGE void VCSAGCCON VCSAgGenSnmpTrap(int trap_num, 
		const char *msg, VCSAgBool is_global);
	VCSAGLINKAGE void* VCSAGCCON VCSAgGetCookie(const char *name);
	VCSAGLINKAGE int VCSAGCCON VCSAgLockFile(const char *fname, 
		VCSAgLockType ltype, VCSAgBlockingType btype, 
		VCSAgErrnoType *errp);
	VCSAGLINKAGE void VCSAGCCON VCSAgLogConsoleMsg(int tag, 
		const char *msg, int flags);
	VCSAGLINKAGE void VCSAGCCON VCSAgLogI18NConsoleMsg(int tag, 
		const char *msg, int msg_id, const char *arg1_string,
		const char *arg2_string, const char *arg3_string, 
		const char *arg4_string, int flags);
	VCSAGLINKAGE void VCSAGCCON VCSAgLogI18NConsoleMsgEx(int tag, 
		const char *msg, int msg_id, const char *arg1_string,
		const char *arg2_string, const char *arg3_string, 
		const char *arg4_string, const char *arg5_string,
		const char *arg6_string, int flags);
	VCSAGLINKAGE void VCSAGCCON VCSAgLogMsg(int tag, const char *msg, 
		int flags);
	VCSAGLINKAGE void VCSAGCCON VCSAgLogI18NMsg(int tag, const char *msg, 
		int msg_id, const char *arg1_string, const char *arg2_string, 
		const char *arg3_string, const char *arg4_string, int flags);
	VCSAGLINKAGE void VCSAGCCON VCSAgLogI18NMsgEx(int tag, const char *msg,
		int msg_id, const char *arg1_string, const char *arg2_string,
		const char *arg3_string, const char *arg4_string,
		const char *arg5_string, const char *arg6_string, int flags);
	VCSAGLINKAGE void VCSAGCCON VCSAgMain(int argc, char **argv);
	VCSAGLINKAGE void VCSAGCCON VCSAgRegister(const char *notify_res_name,
		const char *res_name, const char *attr_name);
	VCSAGLINKAGE void VCSAGCCON VCSAgSetCookie(const char *name,
		void *cookie);
	VCSAGLINKAGE void *VCSAGCCON VCSAgSetCookie2(const char *name,
		void *cookie);
	VCSAGLINKAGE void VCSAGCCON VCSAgUnregister(
		const char *notify_res_name, const char *res_name, 
		const char *attr_name);
	VCSAGLINKAGE int VCSAGCCON VCSAgUnlockFile(const char *fname,
		VCSAgErrnoType *errp);
	VCSAGLINKAGE void VCSAGCCON VCSAgSetStackSize(int i);

	/* Methods for registering entry points */
	VCSAGLINKAGE void VCSAGCCON VCSAgSetEntryPoints(
		VCSAgEntryPointStruct& ep);
	VCSAGLINKAGE void VCSAGCCON VCSAgRegisterEPStruct(
		VCSAgAgentVersion ag_ver, void *);
	VCSAGLINKAGE void VCSAGCCON VCSAgInitEntryPointStruct(
		VCSAgAgentVersion agent_version);
	VCSAGLINKAGE void VCSAGCCON VCSAgSetEntryPoint(
		VCSAgEntryPoint ep, void *f_ptr);

	VCSAGLINKAGE VCSAgFwVersion VCSAGCCON VCSAgGetFwVersion();

	/* Methods for string manipulation */
	VCSAGLINKAGE void VCSAGCCON VCSAgStrlcpy(char *dst, const char *src, 
		int size);
	VCSAGLINKAGE void VCSAGCCON VCSAgStrlcat(char *dst, const char *src, 
		int size);
	VCSAGLINKAGE int VCSAGCCON VCSAgSnprintf(char *buffer, int size,
		const char *format, ...);
	VCSAGLINKAGE int VCSAGCCON VCSAgEncodeString(VCSAgEncodingType f_enc, 
		void *i_str, int i_str_len, VCSAgEncodingType t_enc, 
		void **o_str, int *o_str_len);
	VCSAGLINKAGE int VCSAGCCON VCSAgGetEncodedArgList(
		VCSAgEncodingType f_enc, void **i_str, 
		VCSAgEncodingType t_enc, void ***o_str);

	VCSAGLINKAGE void VCSAGCCON VCSAgLogConsoleMsg1(VCSAgAppSev sev,
		const char *who, const char *file, int line,
		int msg_id, int flags, const char *fmtp, ...);
	VCSAGLINKAGE void VCSAGCCON VCSAgLogMsg1(VCSAgAppSev sev, 
		const char *who, const char *file, int line, 
		int msg_id, int flags, const char *fmtp, ...);
	VCSAGLINKAGE void VCSAGCCON VCSAgLogDbgMsg1(VCSAgDbgSev sev, 
		const char *who, const char *file, int line, 
		int flags, const char *fmtp, ...);
	VCSAGLINKAGE void VCSAGCCON VCSAgResLogMsg1(VCSAgDbgSev sev, 
		const char *func, const char *file, int line, 
		int flags, const char *fmtp, ...);
	VCSAGLINKAGE void VCSAGCCON VCSAgSetLogCategory(int cat_id);
	VCSAGLINKAGE const char *VCSAgGetProductName();
	VCSAGLINKAGE int VCSAGCCON VCSAgDecryptKey(char *key,
		char *outbuf, int buflen);

#ifndef WINNT
	// Methods for procfs scanning in the agent framework

	// NOTE: These interfaces are for agent framework internal use only
        // and are subject to change at any time without prior notice. Backward
        // compatibility for these interfaces is not guaranteed and hence
	// agents using them may fail to run on future releases.
	
	VCSAGLINKAGE int VCSAGCCON VCSAgRegisterSingleProc(
		const char *res_name, VCSAgProcInfoStruct *pinfo,
		VCSAgProcMatchCriteriaStruct *proc_mc);
	VCSAGLINKAGE int VCSAGCCON VCSAgRegisterProcs(const char *res_name,
		int num_procs, VCSAgProcInfoStruct **procs,
		VCSAgProcMatchCriteriaStruct *proc_mc);
	VCSAGLINKAGE int VCSAGCCON VCSAgSetResUserId(const char *res_name,
		uid_t uid);
#ifdef	SOLARIS
	VCSAGLINKAGE int VCSAGCCON VCSAgSetResContainerId(const char *res_name,
		int container_id);
#endif	/* !WINNT && SOLARIS */
	VCSAGLINKAGE int VCSAGCCON VCSAgLookupSingleProc(const char *res_name,
		VCSAgProcInfoStruct *pinfo, void *psinfo,
		int *lookup_timestamp);
	VCSAGLINKAGE int VCSAGCCON VCSAgLookupProcs(const char *res_name,
		int num_procs, VCSAgProcInfoStruct **procs,
		int ***proc_ret, void *psinfo,
		int *lookup_timestamp);
	VCSAGLINKAGE int VCSAGCCON VCSAgGetResOnlineProcs(const char *res_name,
		int *num_online_procs, void *psinfo,
		int *lookup_timestamp);
	VCSAGLINKAGE int VCSAGCCON VCSAgResGetRegisteredProcs(
		const char *res_name, VCSAgProcInfoStruct ***registered_procs);
	VCSAGLINKAGE int VCSAGCCON VCSAgUnregisterAllProcs(
		const char *res_name);

	// End methods for procfs scanning in the agent framework
#endif	/* !WINNT */

#ifdef	SOLARIS

	VCSAGLINKAGE char *VCSAGCCON VCSAgGetContainerName(
		const char *resource_name);
	VCSAGLINKAGE int VCSAGCCON VCSAgGetContainerID(
		const char *resource_name);
	VCSAGLINKAGE int VCSAGCCON VCSAgExecInContainer(char *container_name, 
		const char *path, char *const argv[], char *buf, 
		long buf_size, unsigned long *exit_codep);
	VCSAGLINKAGE VCSAgBool VCSAGCCON VCSAgISZoneCapable();

#endif	/* SOLARIS */
	
#ifdef	WINNT
#ifdef	__ENABLE_L10N_INPUT__

	VCSAGLINKAGE void VCSAGCCON VCSAgDelEncodedArgList(void **enc_arg_list);
	VCSAGLINKAGE void VCSAGCCON VCSAgStrlcpyW(wchar_t *dst, 
		const wchar_t *src, int size);
	VCSAGLINKAGE void VCSAGCCON VCSAgStrlcatW(wchar_t *dst, 
		const wchar_t *src, int size);
	VCSAGLINKAGE int VCSAGCCON VCSAgSnprintfW(wchar_t *buffer, 
		int size, const wchar_t *format, ...);
	VCSAGLINKAGE void * VCSAGCCON VCSAgAllocMem(size_t size);
	VCSAGLINKAGE void VCSAGCCON VCSAgDeleteMem(void *mp);
	VCSAGLINKAGE void VCSAGCCON VCSAgLogConsoleMsg2(VCSAgAppSev sev,
		const char *who, const char *file, int line,
		int msg_id, int flags, const wchar_t *fmtp, ...);
	VCSAGLINKAGE void VCSAGCCON VCSAgLogMsg2(VCSAgAppSev sev,
		const char *who, const char *file, int line, 
		int msg_id, int flags, const wchar_t *fmtp, ...);
	VCSAGLINKAGE void VCSAGCCON VCSAgLogDbgMsg2(VCSAgDbgSev sev,
		const char *who, const char *file, int line, 
		int flags, const wchar_t *fmtp, ...);
	VCSAGLINKAGE void VCSAGCCON VCSAgResLogMsg2(VCSAgDbgSev sev,
		const char *func, const char *file, int line,
		int flags, const wchar_t *fmtp, ...);

#endif	/* WINNT && __ENABLE_L10N_INPUT__ */
#endif	/* WINNT */

#ifndef VCSAGDLL
	VCSAGEXPORT void VCSAGCCON VCSAgStartup();
#endif	/* !VCSAGDLL */

#ifdef  __cplusplus
}
#endif	/* __cplusplus */

#endif	/* !_VCSAgApi_h_ */
