# $Id: ArrayRep.pm,v 1.2.12.5 2008/07/19 15:57:44 sirisha Exp $
# $Copyrights: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

package ArrayRep;

use strict;
use Exporter;
our @ISA = ('Exporter');

my @EXPORT = ('vxvm_prepare_disks', 'vxvm_test_diskgroup_import',
	'is_cvm_master', 'cvm_slave_wait_for_shared_dg_import');
my $VERSION = '1.00';

require 5.000;
use ag_i18n_inc;
use POSIX qw(uname);

my ($OS, $DEVNULL, $VXCLUSTADM, $HACONF);

($OS) = uname;

my $VXDG = "/usr/sbin/vxdg";
my $VXDCTL = "/usr/sbin/vxdctl";
my $VXDISK = "/usr/sbin/vxdisk";

if ($OS eq "Windows NT") {
	$DEVNULL = "nul";
} else { # unix
	$DEVNULL = "/dev/null";
	$VXCLUSTADM = "/etc/vx/bin/vxclustadm";
	$HACONF = "$VCSHOME/bin/haconf";
}

#
# AIX only
#
my ($LQUERYVG, $GETLVODM, $RECREATEVG, $CHDEV, $ODMGET, $EXPORTVG );

if ($OS eq "AIX") {
	$LQUERYVG        = "/usr/sbin/lqueryvg";
	$GETLVODM        = "/usr/sbin/getlvodm";
	$RECREATEVG      = "/usr/sbin/recreatevg";
	$CHDEV           = "/usr/sbin/chdev";
	$ODMGET          = "/usr/bin/odmget";
	$EXPORTVG        = "/usr/sbin/exportvg";

}


sub create_lockfile()
{
	my ($self) = shift;
	my ($typ) = $self->{'resource_type'};

	if (0 == length $typ) {
		#
		# Resource type was not set in the $self object
		#
		($typ) = split('\n',
			`$ag_i18n_inc::HARES -value $self->{'resname'} Type`);
	}

	my ($LockFile);

	if ($OS eq "Windows NT") {
		$LockFile = $ag_i18n_inc::VCSHOME . "\\bin\\" . $typ . "\\.VCS_" . $typ . "_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
	} else {
		$LockFile = $ag_i18n_inc::VCSHOME . "/bin/" . $typ . "/.VCS_" . $typ . "_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
	}

	#TBD: if lockfile creation failed
	open(lfh, ">$LockFile");
	close(lfh);
}

sub remove_lockfile()
{
	my($self) = shift;
	my ($typ) = $self->{'resource_type'};

	if (0 == length $typ) {
		#
		# Resource type was not set in the $self object
		#
		($typ) = split('\n',
			`$ag_i18n_inc::HARES -value $self->{'resname'} Type`);
	}
	my ($LockFile);

	if ($OS eq "Windows NT") {
		$LockFile = $ag_i18n_inc::VCSHOME . "\\bin\\" . $typ . "\\.VCS_" . $typ . "_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
	} else {
		$LockFile = $ag_i18n_inc::VCSHOME . "/bin/" . $typ . "/.VCS_" . $typ . "_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
	}

	#TBD: if lockfile removal failed
	if (-f "$LockFile") {
		unlink("$LockFile");
	}
}

#
# The subroutine returns 110 if the online lock file for the resource
# exists on the system; 100 otherwise. The monitor entry point that
# invokes this subroutine must exit with the *return code* from this
# entry point.
#
sub check_lockfile()
{
	my ($self) = shift;
	my ($typ) = $self->{'resource_type'};

	if (0 == length $typ) {
		#
		# Resource type was not set in the $self object
		#
		($typ) = split('\n',
			`$ag_i18n_inc::HARES -value $self->{'resname'} Type`);
	}
	my ($LockFile);
	my $res_state = 100;

	if ($OS eq "Windows NT") {
		$LockFile = $ag_i18n_inc::VCSHOME . "\\bin\\" . $typ . "\\.VCS_" . $typ . "_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
	} else {
		$LockFile = $ag_i18n_inc::VCSHOME . "/bin/" . $typ . "/.VCS_" . $typ . "_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
	}

	if (-f "$LockFile") {
		$res_state = 110;
	}
	return $res_state;
}

sub serialize {
	my($self) = shift;
	my($grp) = split('\n',
		`$ag_i18n_inc::HARES -value $self->{'resname'} Group`);
	my($para) = split('\n', `$ag_i18n_inc::HAGRP -value $grp Parallel`);

	if ($para != 1) {
		return 0;
	}

	my $ret = "";
	open(INPUT,"$ag_i18n_inc::HARES -lock $self->{'resname'} 2>&1 |");
	while (my $line = <INPUT>) {
		chomp($line);
		$ret = ($ret) ? "$ret\n$line" : $line;
	}
	close INPUT;
	my $retval = $?;
	if ($retval == 0) {
		return 0;
	}

	my (@lockholder) = split('\s+', $ret);
	VCSAG_LOG_MSG("N",
	"Resource $self->{'resname'} will wait for onlining instance on $lockholder[9] to complete",
		60002, "$self->{'resname'}", "$lockholder[9]");
	while ($retval) {
		sleep(5);
		`$ag_i18n_inc::HARES -lock $self->{'resname'} > $DEVNULL 2>&1`;
		$retval = $?;
	}
	return 1; # had to wait
}

sub cvm_import {
	my($self) = shift;
	my $nodename = "";
	my ($grp) = split('\n',
		`$ag_i18n_inc::HARES -value $self->{'resname'} Group`);
	my ($par) = split('\n', `$ag_i18n_inc::HAGRP -value $grp Parallel`);
	if ($par != 1) {
		return 0;
	}
	my ($sup) = split('\n',
		`$ag_i18n_inc::HATYPE -value CVMVolDg SupportedActions`);
	if (! (($sup =~ /deport/) && ($sup =~ /import/))) {
		return 0;
	}

	`$ag_i18n_inc::HARES -lock $self->{'resname'} > $DEVNULL 2>&1`;
	my $retval = $?;
	while ($retval) {
		sleep(5);
		`$ag_i18n_inc::HARES -lock $self->{'resname'} > $DEVNULL 2>&1`;
		$retval = $?;
	}

	my (@states) = split('\n',
		`$ag_i18n_inc::HARES -state $self->{'resname'} -localclus`);
	foreach my $sts (@states) {
		if ($sts =~ /ONLINE/) {
			return 0;
		}
	}
	my $found = 0;
	my (@vxclus) = split('\n', `$VXCLUSTADM nidmap 2>$DEVNULL`);
	foreach my $v (@vxclus) {
		if ($v =~ /Master/) {
			my (@v1) = split('\s+', $v);
			$nodename = $v1[0];
			$found = 1;
			last;
		}
	}
	if (!$found) {
		return 0;
	}

	my ($typ) = $self->{'resource_type'};

	if (0 == length $typ) {
		#
		# Resource type was not set in the $self object
		#
		($typ) = split('\n',
			`$ag_i18n_inc::HARES -value $self->{'resname'} Type`);
	}

	VCSAG_LOG_MSG("N", 
	"Resource $self->{'resname'} is its group's first $typ resource in this cluster to online; all CVM Disk Groups used by this group will be imported on all nodes if they have not already been",
		60000, "$self->{'resname'}", "$typ");

	my $import_failed = 0;
	my (@deps) = split('\n', `$ag_i18n_inc::HARES -dep $self->{'resname'}`);
	foreach my $d (@deps) {
		if ($d =~ /#Group/) {
			next;
		}
		my (@fields) = split('\s+', $d);
		if ($fields[2] ne $self->{'resname'}) {
			next;
		}
		my ($typ) = split('\n',
			`$ag_i18n_inc::HARES -value $fields[1] Type`);
		if ($typ eq "CVMVolDg") {
			`$ag_i18n_inc::HARES -action $fields[1] import -sys $nodename`;
			my $ret = $?;
			if ($ret) {
				#TBD: get output from failure
				$import_failed = 1;
			}
		}
	}
	return $import_failed;
}

# Return 2 if the function is a nop since the resource is online on
# other systems in the local cluster
sub cvm_deport {
	my($self) = shift;
	my $DONT_DO_ANYTHING = 2;
	my $nodename = "";
	my $sleeptime = 0;
	my $ctr = 0;
	my ($grp) = split('\n',
		`$ag_i18n_inc::HARES -value $self->{'resname'} Group`);
	my ($par) = split('\n', `$ag_i18n_inc::HAGRP -value $grp Parallel`);
	if ($par != 1) {
		return 0;
	}
	my ($sup) = split('\n',
		`$ag_i18n_inc::HATYPE -value CVMVolDg SupportedActions`);
	if (! (($sup =~ /deport/) && ($sup =~ /import/))) {
		return 0;
	}
	my(@fields) = split('\s+', `$ag_i18n_inc::HASYS -nodeid`);
	my($sys) = $fields[5];

	`$ag_i18n_inc::HARES -lock $self->{'resname'} > $DEVNULL 2>&1`;
	my $retval = $?;
	while ($retval) {
		sleep(5);
		`$ag_i18n_inc::HARES -lock $self->{'resname'} > $DEVNULL 2>&1`;
		$retval = $?;
	}

	my (@states) = split('\n',
		`$ag_i18n_inc::HARES -state $self->{'resname'} -localclus`);
	foreach my $st (@states) {
		my (@fields) = split('\s+', $st);
		if (($fields[3] eq "ONLINE") && ($fields[2] ne $sys)) {
			return $DONT_DO_ANYTHING;
		}
	}
	my $found = 0;
	my (@vxclus) = split('\n', `$VXCLUSTADM nidmap 2>$DEVNULL`);
	foreach my $v (@vxclus) {
		if ($v =~ /Master/) {
			my (@v1) = split('\s+', $v);
			$nodename = $v1[0];
			$found = 1;
			last;
		}
	}
	if (!$found) {
		return 0;
	}
	my ($typ) = $self->{'resource_type'};

	if (0 == length $typ) {
		#
		# Resource type was not set in the $self object
		#
		($typ) = split('\n',
			`$ag_i18n_inc::HARES -value $self->{'resname'} Type`);
	}
	VCSAG_LOG_MSG("N",
	"Resource $self->{'resname'} is its group's last $typ resource in this cluster to offline; all CVM Disk Groups used by this group will be deported on all nodes if they have not already been",
		60001, "$self->{'resname'}", "$typ");
	my $deport_failed = 0;
	my (@deps) = split('\n', `$ag_i18n_inc::HARES -dep $self->{'resname'}`);
	foreach my $d (@deps) {
		if ($d =~ /#Group/) {
			next;
		}
		my (@fields) = split('\s+', $d);
		if ($fields[2] ne $self->{'resname'}) {
			next;
		}
		my ($typ) = split('\n',
			`$ag_i18n_inc::HARES -value $fields[1] Type`);
		if ($typ eq "CVMVolDg") {
			`$ag_i18n_inc::HARES -action $fields[1] deport -sys $nodename`;
			my $ret = $?;
			if ($ret) {
				#TBD: get output from failure
				$deport_failed = 1;
			}
		}
	}
	return $deport_failed;
}

#
# Arguments to the subroutine:
# ----------------------------
# dgname: The string comprising of the original diskgroups names 
# (without the _fd appended to the names) separated by a " "
# case: Type of Firedrill: gold/silver/bronze
# tagname: The value of the tag set on the disks on which the diskgroup 
#	is to be imported
# is_rac_cluster: 1 if this is a RAC environment in which the agent is
#	running; 0 otherwise
#
# Return Values:
# --------------
# 0 on success;
# 1 otherwise and in this case, an appropriate error message is logged
#
# Functionality:
# --------------
# This subroutine checks that after the disks have been setup right,
# the diskgroups can be successfully imported on the system. If the
# subroutine encounters error in attempting to import/deport the
# diskgroups, it returns a failure code (1) along with logging the
# appropriate error message.
#   
sub vxvm_test_diskgroup_import {

	my ($dgnamelist, $case, $tagname, $is_rac_cluster) = @_;
	my @dglist = split(' ',$dgnamelist);
	my $newdg;
	my $options = "";
	my $rac_options = "";

	#
	# $options contains the options to be used during the diskgroup
	# import in a non-RAC env whereas $rac_options contains the 
	# options to be used during the diskgroup import in a RAC env
	#

	my (@cmds) = ();
	my $ret = 0;

	if ($case ne "bronze") {
		#
		# GOLD or SILVER firedrill
		#
		$options = "-C -o useclonedev=on -o updateid -o tag=$tagname";
		$rac_options = "-s -C -o updateid -o tag=$tagname";
	} else {
		#
		# BRONZE Firedrill
		#
		$options = "-C -o updateid";
		$rac_options = "-s -C -o updateid";
	}

	foreach my $dgname (@dglist) {
		$newdg = "$dgname" . "_fd";

		#
		# Import the dg as a non-shared dg first. This is a
		# workaround for a bug in CVM. Refer to etrack incident 
		# 1053529 for more info on this bug.
		#

		push(@cmds, "$VXDG $options -n $newdg import $dgname");
		$ret = ar_run_cmds(\@cmds);
		if ($ret) {
			VCSAG_LOG_MSG("E", "Cannot import disk group $dgname in pitc $case; exiting",
				60003, "$dgname", "$case");
			return 1;
		}

		push(@cmds, "$VXDG deport $newdg");
		$ret = ar_run_cmds(\@cmds);
		if ($ret) {
			VCSAG_LOG_MSG("E", "Cannot deport disk group $newdg in pitc $case; exiting",
				60004, "$newdg", "$case");
			return 1;
		}
	}

	return 0;
}

#
# This subroutine prepares disks assuming VxVM to be the volume manager
# technology. This subrtoutine is called from the array-based
# technology specific prepare_disk subroutine.
# Arguments to the subroutine:
# ----------------------------
# disks: List of disks that the device group has been created upon. To
#	get this list of disks, run the technology specific command to view
#	the device group and the disks underlying it.
# tagname: The tagname to be used to tag the specific volume manager disks.
#	This is the tag that will be used to import the disk group on these
#	disks.
# is_rac_cluster: 1 if this is a RAC environment in which the agent is
#	running; 0 otherwise
# Return Value:
# -------------
# 0 on success;
# 1 otherwise. An appropriate error message is logged by this
#	subroutine when an error is encountered
# Functionality:
# --------------
# This subroutine does the following tasks:
#	1. Gets the volume manager disks that map to the disks the device
#	group is created on
#	2. For each volume manager disk that comprises the device group, the
#	subroutine does:
#		vxdisk clearimport to clear import information on the volume
#		  manager disks
#		vxdisk settag <tagname> to set a tag to uniquely identify the
#		  disks during an import operation
#	If any of these operations fail, the subroutine logs an
#	appropriate error message and returns 1 to indicate a failure.
#
sub vxvm_prepare_disks {
	my ($tagname, $is_rac_cluster, @disks) = (@_);
	my (%vxdisk) = ();
	my (@cmds) = ();
	my $ret = 0;

	VCSAG_LOGDBG_MSG(1,
		"is_rac_cluster is $is_rac_cluster; tag is $tagname");

	# This takes care of enclosure based naming
	my (@lines) = split('\n', `$VXDISK -eq list`);
	foreach my $l (@lines) {
		my (@fields) = split('\s+', $l);
		# column 5 is the name of the device and column 0 is the
		# name of the device as under VxVM - if enclosure based
		# naming has been used, column 0 will be different from
		# column 5.
		$vxdisk{$fields[5]} = $fields[0];
	}

	foreach my $disk (@disks) {
		if ($vxdisk{$disk}) {
			$disk = $vxdisk{$disk};
		}
		push(@cmds, "$VXDISK clearimport $disk");
		push(@cmds, "$VXDISK settag $disk $tagname");
		$ret = ar_run_cmds(\@cmds);
		if ($ret) {
			# Try once again, but this time run vxdctl
			# enable before trying out the cmds
			push(@cmds, "$VXDCTL enable");
			push(@cmds, "$VXDISK clearimport $disk");
			push(@cmds, "$VXDISK settag $disk $tagname");
			$ret = ar_run_cmds(\@cmds);
			if ($ret) {
				VCSAG_LOG_MSG("E", 
					"Could not tag disk $disk; exiting;",
					60012, "$disk");
				return 1;
			}
		}
	}
	# All has succeeded
	return 0;
}

#
# The subroutine is called ONLY in a RAC environment (i.e. if the group
# for which a FireDrill is being run is a parallel group).
# This routine checks if the node is the CVM MASTER
# It returns:
# 	-1 on error
# 	1 if the current node is the MASTER
# 	0 if the current node is not the MASTER
#
sub is_cvm_master {
	my (@lines) = split('\n', `$VXDCTL -c mode`);
	my $ret = $?;
	if ($ret != 0 || $#lines == -1) {
		VCSAG_LOG_MSG("E",
"Command [$VXDCTL -c mode] failed; Could not determine vxconfigd mode; exiting;",
			60005, "$VXDCTL");
		return -1;
	}

	#
	# The first line contains info abt the cluster state and
	# the role of the node in the cluster: MASTER/SLAVE
	#
	my $l = "";
	my $line = "";
	foreach $l (@lines) {
		if ($l =~ /^mode:/) {
			$line = $l;
			last;
		}
		next;
	}

	my (@fields) = split('\s+', $line);
	if ($fields[0] ne "mode:") {
		VCSAG_LOG_MSG("E",
"Could not determine vxconfigd mode from the output of the command [$VXDCTL -c mode]; exiting;",
			60006, "$VXDCTL");
		return -1;
	}

	#
	# From the manpage for vxdctl, clustered_state is one of:
	# 	cluster inactive,
	# 	cluster active  -  role not set,
	# 	cluster active - MASTER,
	# 	cluster active - SLAVE
	#
	if ($line =~ /cluster inactive/ || $line =~ /role not set/) {
		#
		# cvm is not active
		#
		VCSAG_LOG_MSG("E", "vxconfigd is not in the correct clustered state: $line",
			60007, "$line");
		return -1;
	}
	if ($fields[5] eq "MASTER") {
		return 1;
	}
	elsif ($fields[5] ne "MASTER") {
		return 0;
	}
}

#
# The subroutine is called ONLY in a RAC environment
# If the current node is a slave, it will wait for the MASTER to import
# the shared dg using this subroutine. 
# Arguments to the subroutine:
# ----------------------------
# dgnames: List of names of diskgroups that depend on the underlying
#	replication resource. This list is created during the
#	constructor for the replication resource object by checking the
#	resource dependencies on the replication resource and querying the
#	name of the diskgroup for those DiskGroup/CVMVolDg resources.
# Return Value:
# -------------
# The routine returns:
#	1 if there was some error
#	0 if it successfully waited and the MASTER successfully imported the
#	shared dg
# Functionality:
# --------------
# This routine is only called by a slave node while it is
# waiting for the shared dgs to be imported by the MASTER. If for some
# reason, there are errors in running vm commands on the slave node, the
# routine will return with an error. In the event that there is a list
# of diskgroups that has been passed into this routine, it is sufficient 
# to wait for any one of the diskgroups to be imported in a shared mode
# before the slave can return from this subroutine.
#
sub cvm_slave_wait_for_shared_dg_import {
	my ($dgnames) = @_;
	my $dgnames_fd = "";
	my (@cmds) = ();
	my $ret = 0;
	my (@dgnamelist) = split(' ', $dgnames);

	#
	# Contruct a string that contains each of the dg names with the
	# "_fd" appended to each dg name. The diskgroups will be
	# imported using these new names.
	#
	foreach my $dg (@dgnamelist) {
		$dgnames_fd = "$dg"."_fd"." "."$dgnames_fd";
	}

	while (1) {
		(@cmds) = ();
		my (@lines) = split('\n', `$VXDG list`);
		$ret = $?;
		my $l = "";
		if (($ret != 0) || ($#lines == 0)) {
			push(@cmds, "$VXDCTL enable");
			$ret = ar_run_cmds(\@cmds);
			if ($ret) {
				VCSAG_LOG_MSG("E", 
	"Could not get list of diskgroups: vxconfigd is not enabled; exiting;",
					60008);
				return 1;
			}

			@lines = split('\n', `$VXDG list`);
			$ret = $?;
			if (($ret != 0) || ($#lines == 0)) {
				VCSAG_LOG_MSG("E", 
	"Could not get list of diskgroups: vxdg list failed; exiting;",
					60009);
				return 1;
			}
		}
		foreach $l (@lines) {
			if ($l =~ /^NAME/) {
				next;
			}
			my (@fields) = split('\s+', $l);
			if (($fields[0] ne " ") && ($dgnames_fd =~ /\b$fields[0]\b/)) {
				#
				# Atleast one of the dgs in the dglist
				# has been imported
				#
				if ($fields[1] =~ /shared/) {
					return 0;
				} else {
					# dg is not imported in shared
					# mode. The master would import all 
					# the dgs as private diskgroups and 
					# then once that succeeds for all 
					# dgs, the master will import the 
					# dgs in shared mode. So even if 
					# one dg exists that is imported as a
					# pvt dg, the master is not done
					# yet preparing the dgs for
					# shared import.
					# So, quit the loop and come back
					# after 5 seconds. 
					last;
				}
			}
			next;
		}
		sleep 5;
	} #while (1)
}

sub ar_do {
	my ($cmd, $line, $ret);
	my $exit_code = 0;
	($cmd) = (@_);
	VCSAG_LOG_MSG("N", "Issuing command: $cmd", 60010, "$cmd");
	my $pid = open(INPUT,"$cmd 2>&1 |");
	if (!defined($pid)) {
		#
		# open failed
		#
		my $err = "$!";
		VCSAG_LOG_MSG("E", "Command failed with error $err",
			60014, "$err");
		return undef;
	}

	#
	# open succeeded
	#
	while ($line = <INPUT>) {
		chomp($line);
		$ret = ($ret) ? "$ret\n$line" : $line;
	}
	close INPUT;
	$exit_code = $?;
	if ($exit_code) {
		my $status = $exit_code / 256;
		VCSAG_LOG_MSG("E",
			"Command failed with error: $ret and exit code: $status",
			60013, "$ret", "$status");

	} else {
		VCSAG_LOG_MSG("N", "Command succeeded", 60011);
	}
	return $exit_code;
}

sub ar_run_cmds {
	my ($cmdsref) = @_;
	my $ret = 0;
	foreach my $cmd (@{$cmdsref}) {
		$ret = ar_do($cmd);
		if (!defined($ret) || $ret) { last; }
	}
	@{$cmdsref} = ();
	return $ret;
}



#
# Exports AIX LVM volume groups
# Input: reference to array of volume group names
# Return: 0 on success; non-zero on failure
#

sub aix_lvm_exportvgs {
	my ($vgsref) = @_;
	
	my @cmds = ();
	my $ret = 0;
	my $vgname;

	foreach $vgname ( @{$vgsref} ) {
		push( @cmds, "$EXPORTVG $vgname" );
		$ret = ar_run_cmds(\@cmds);

		if ($ret) {
			VCSAG_LOG_MSG( "W", "Could not export VG $vgname ", 
			60015, "$vgname" );
		}
	}
	return $ret;
}


#
# Construct a hash to map VGname to VG serial id
# Input: reference to array of volume group names
# Output: hash mapping VG name to VG id
# No return value
#

sub aix_lvm_get_vgs_id {
	my ( $vgnamesref, $hashref ) = @_;
	my @lines;
	my ( $line, $id );

	foreach my $vgname (@$vgnamesref) {
		@lines = `$ODMGET -q "name='$vgname' AND attribute='vgserial_id'" CuAt`;

		# Get this line: value = "00ca435d00004c00000001186871772e"
		($line) = grep( /^[\s]*value/, @lines );
		( undef, $id, undef ) = split( /"/, $line );

		${$hashref}{$vgname} = $id;
	}

}

#
# Construct a hash to map VGids to bcv disks
# Input:  reference to array of volume group names,
#         reference to array of disk names
# Output: hash mappig VGid to the disks in the VG
# No return value.
#
sub aix_lvm_get_vgdisks {
	my ( $vgidsref, $hashref, $disksref ) = @_;
	my ( $vgid, $disk );

	foreach $vgid (@$vgidsref) {
		${$hashref}{$vgid} = [];
	}

	foreach $disk (@$disksref) {
		$vgid = `$LQUERYVG -p $disk -v 2`;
		chomp $vgid;
		push( @{ ${$hashref}{$vgid} }, $disk );
	}
}



#
# Given a disk belonging to a VG and an LVMVG resource name,
# get the VG major number from the disk and use that to modify
# the MajorNumber attribute of the LVMVG resource
#

sub aix_lvm_update_majornum_attrib {
	my ( $lvmresname, $disksref ) = @_;

	my @disks = @{$disksref};

	#
	# We need to modify the Major Number of the parent LVMVG resource since
	# recreatevg does not let us specify a Major Number.
	#
	{

		my $set_rw = 0;
		my $ro	  = 0;
		
		#
		# Open config for writing if required
		#
		$ro = `$HACLUS -value ReadOnly`;
		if ( $ro == 1 ) {
			#
			# status is RO .. converting to RW
			#
			$set_rw = 1;
			`$HACONF -makerw`;
		}
		
		#
		#Get the major number that recreatevg assigned us.
		#
		my $vgid  = `$LQUERYVG -p $disks[0] -v 2`;
		my $majornum = `$GETLVODM -d  $vgid`;

		#
		#Modify the parent LVMVG resource's MajorNumber attribute
		#
		`$HARES -modify $lvmresname MajorNumber $majornum`;
		
		#
		# Close config if we opened it for writing
		#
		if ( $set_rw == 1 ) {
			#
			# We set it to RW .. so we change it to RO
			#
			$set_rw = 0;
			`$HACONF -dump -makero`;
		}
	}
}

1;
