# $Id: ag_i18n_inc.pm,v 2.23.2.14 2008/09/16 00:46:44 vikasg Exp $
# $Copyrights: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# exported functions can be used after doing the following:
#       use ag_i18n_inc;

package ag_i18n_inc;
use Exporter;
@ISA = ('Exporter');
@EXPORT = ('$VCSAG_RES_UNKNOWN', '$VCSAG_RES_OFFLINE', '$VCSAG_RES_ONLINE',
'$VCSAG_RES_INTENTOFFLINE', '$VCSAG_RES_INTENTIONALOFFLINE', 'VCSAG_SET_ENVS',
'VCSAG_RESET_LANG', 'VCSAG_SET_LANG', 'VCSAG_LOG_MSG', 'VCSAG_LOGDBG_MSG',
'VCSAG_SQUEEZE_SPACES', 'VCSAG_SYSTEM', 'VCSAG_GET_ATTR_VALUE', 'VCSAG_SU',
'VCSAG_SET_RES_EP_TIMEOUT', '$VCSCONF', '$VCSHOME', '$VCSLOG', '$VCSAG_SUCCESS',
'$HAATTR', '$HACLUS', '$HADECRYPT', '$HAENCRYPT', '$HAGRP', '$HAIFCONFIG',
'$HALOG', '$HALOGIN', '$HARES', '$HASTATUS', '$HASYS', '$HATYPE', '$HAUSER',
'$NOTIFIER', '$CLUSTER_FQPN');
use File::Basename;
use POSIX qw(uname);
use strict;

my ($ACTION_ENTRY_PT) = "actions";
my ($VCSAG_LOGDBG) = "NOT_DEFINED";
my ($VCS_LOG_CATEGORY, $VCS_LOG_SCRIPT_NAME);
my ($SYSTEM, $FILE_NAME, $AGENT_DIR, $VCSAG_BASE_AGENT_DIR, $VCSAG_OS, $CLUS_PROD);
my ($VCS_SUPPLANG, $VCS_USERLANG, $VCS_IS_CLANG_SET) = ("C", $ENV{"LANG"}, 1);

# important VCS env variables
our ($VCSCONF, $VCSHOME, $VCSLOG, $HADECRYPT, $HAENCRYPT, $CLUSTER_FQPN);

# states of a resource
our($VCSAG_RES_UNKNOWN, $VCSAG_RES_OFFLINE, $VCSAG_RES_ONLINE, $VCSAG_RES_INTENTOFFLINE) = (99, 100, 110, 200);
our($VCSAG_RES_INTENTIONALOFFLINE) = ($VCSAG_RES_INTENTOFFLINE);

# return values of VCSAG_GET_ATTR_INDEX/VALUE
our($VCSAG_SUCCESS) = (0);
my ($VCSAG_NULL_ATTR, $VCSAG_NULL_ARGLIST) = (1, 2);
my ($VCSAG_ATTR_NOTFOUND,$VCSAG_INVAL_ARGLIST) = (3, 4);
my ($VCSAG_INVAL_INDEX, $VCSAG_INVAL_NUMVALS) = (5, 6);

$VCS_USERLANG = "" if (! defined($VCS_USERLANG));

$CLUS_PROD = $ENV{"CLUSTER_PRODUCT"};
$CLUS_PROD = "VCS" if (! defined($CLUS_PROD));

if ($CLUS_PROD eq "VAD")
{
    $CLUSTER_FQPN = "Veritas Application Director";
    $VCSCONF = $ENV{"VAD_CONF"} || "/etc/VRTSvad";
    $VCSHOME = $ENV{"VAD_HOME"} || "/opt/VRTSvad";
    $VCSLOG = $ENV{"VAD_LOG"} || "/var/VRTSvad";
    $HADECRYPT = "$VCSHOME/sbin/vaddecrypt";
    $HAENCRYPT = "$VCSHOME/bin/vadencrypt";
}
else
{
    $CLUSTER_FQPN = "Veritas Cluster Server";
    $VCSCONF = $ENV{"VCS_CONF"} || "/etc/VRTSvcs";
    $VCSHOME = $ENV{"VCS_HOME"} || "/opt/VRTSvcs";
    $VCSLOG = $ENV{"VCS_LOG"} || "/var/VRTSvcs";
    $HADECRYPT = "$VCSHOME/sbin/vcsdecrypt";
    $HAENCRYPT = "$VCSHOME/bin/vcsencrypt";
}

# commands
our $HAATTR      = "$VCSHOME/bin/haattr";
our $HACLUS      = "$VCSHOME/bin/haclus";
our $HAGRP       = "$VCSHOME/bin/hagrp";
our $HAIFCONFIG  = "$VCSHOME/bin/haifconfig";
our $HALOG       = "$VCSHOME/bin/halog";
our $HALOGIN     = "$VCSHOME/bin/halogin";
our $HARES       = "$VCSHOME/bin/hares";
our $HASTATUS    = "$VCSHOME/bin/hastatus";
our $HASYS       = "$VCSHOME/bin/hasys";
our $HATYPE      = "$VCSHOME/bin/hatype";
our $HAUSER      = "$VCSHOME/bin/hauser";
our $NOTIFIER    = "$VCSHOME/bin/notifier";

$VCSAG_OS = "UNIX";

$FILE_NAME = $0;

# set entry point
$VCS_LOG_SCRIPT_NAME = basename ($FILE_NAME);

# set agent name
$AGENT_DIR = dirname ($FILE_NAME);
$VCSAG_BASE_AGENT_DIR = basename ($AGENT_DIR);

# special treatment to action entry point
if ($VCSAG_BASE_AGENT_DIR eq $ACTION_ENTRY_PT)
{
    $VCS_LOG_SCRIPT_NAME = $VCSAG_BASE_AGENT_DIR;
    $AGENT_DIR = dirname (dirname ($FILE_NAME));
    $VCSAG_BASE_AGENT_DIR = basename ($AGENT_DIR);
}

# set category-id
($SYSTEM) = uname ();
if ($SYSTEM eq "AIX")
{
    $VCS_LOG_CATEGORY = 10011;
}
elsif ($SYSTEM eq "HP-UX")
{
    $VCS_LOG_CATEGORY = 10021;
}
elsif ($SYSTEM eq "Linux")
{
    $VCS_LOG_CATEGORY = 10031;
}
elsif ($SYSTEM eq "SunOS")
{
    $VCS_LOG_CATEGORY = 10001;
}
else # if ($SYSTEM eq "Windows NT")
{
    $VCS_LOG_CATEGORY = 10041;

    # Since we don't want Perl compiler to include Windows specific module on
    # Unix, we have to include the Windows module using 'require' followed by
    # 'import' instead of 'use'.
    require Win32;
    Win32->import;
    $VCSHOME = Win32::GetShortPathName($VCSHOME);

    $HAATTR = "$VCSHOME\\bin\\haattr";
    $HACLUS = "$VCSHOME\\bin\\haclus";
    $HAGRP = "$VCSHOME\\bin\\hagrp";
    $HAIFCONFIG = "$VCSHOME\\bin\\haifconfig";
    $HALOG = "$VCSHOME\\bin\\halog";
    $HALOGIN = "$VCSHOME\\bin\\halogin";
    $HARES = "$VCSHOME\\bin\\hares";
    $HASTATUS = "$VCSHOME\\bin\\hastatus";
    $HASYS = "$VCSHOME\\bin\\hasys";
    $HATYPE = "$VCSHOME\\bin\\hatype";
    $NOTIFIER = "$VCSHOME\\bin\\notifier";

    if ($CLUS_PROD eq "VAD")
    {
        $HADECRYPT = "$VCSHOME\\sbin\\vaddecrypt";
        $HAENCRYPT = "$VCSHOME\\bin\\vadencrypt";
    }
    else
    {
        $HADECRYPT = "$VCSHOME\\sbin\\vcsdecrypt";
        $HAENCRYPT = "$VCSHOME\\bin\\vcsencrypt";
    }

    $VCSAG_OS = "WINDOWS";

    $VCS_LOG_SCRIPT_NAME =~ s/\.pl$//;
}

if ($VCSAG_OS eq "UNIX")
{
    my ($PATH) = $ENV{PATH};
    $PATH = "/usr/bin:/bin:/usr/sbin:/sbin:$PATH";
    $ENV{PATH} = $PATH;
}

$ENV{"LANG"} = $VCS_SUPPLANG;
$ENV{VCS_LOG_SCRIPT_NAME} = $VCS_LOG_SCRIPT_NAME;
$ENV{VCS_LOG_CATEGORY} = $VCS_LOG_CATEGORY;
if (!defined($ENV{VCS_LOG_AGENT_NAME}) || $ENV{VCS_LOG_AGENT_NAME} eq "")
{
    $ENV{VCS_LOG_AGENT_NAME} = $VCSAG_BASE_AGENT_DIR;
}

$ENV{VAD_LOG_SCRIPT_NAME} = $VCS_LOG_SCRIPT_NAME;
$ENV{VAD_LOG_CATEGORY} = $VCS_LOG_CATEGORY;
if (!defined($ENV{VAD_LOG_AGENT_NAME}) || $ENV{VAD_LOG_AGENT_NAME} eq "")
{
    $ENV{VAD_LOG_AGENT_NAME} = $VCSAG_BASE_AGENT_DIR;
}

# set the environment variables required for logging.
# this function must be called once before calling the
# logging function VCSAG_LOG_MSG/VCSAG_LOGDBG_MSG
#
# usage:
# 0. VCSAG_SET_ENVS () # only for triggers
# 1. VCSAG_SET_ENVS ($resource_name)
# 2. VCSAG_SET_ENVS ($resource_name, $category)
# 3. VCSAG_SET_ENVS ($resource_name, $entry_point)
# 4. VCSAG_SET_ENVS ($resource_name, $category, $entry_point)
# 5. VCSAG_SET_ENVS ($resource_name, $entry_point, $category)
sub VCSAG_SET_ENVS
{
    # resource name and/or category and/or entry point.
    # passed by caller [optional]
    my ($VCS_LOG_RESOURCE_NAME, $OPT_ARG1, $OPT_ARG2) = @_;

    if ($#_ == -1) # for triggers
    {
        $ENV{VCS_LOG_CATEGORY} = 6;
        $ENV{VCS_LOG_AGENT_NAME} = "";

        $ENV{VAD_LOG_CATEGORY} = 6;
        $ENV{VAD_LOG_AGENT_NAME} = "";
        return;
    }

    $VCSAG_LOGDBG = $ENV{CLUSTER_LOGDBG};
    $VCSAG_LOGDBG = "" if (! defined($VCSAG_LOGDBG));

    $ENV{VCS_LOG_RESOURCE_NAME} = $VCS_LOG_RESOURCE_NAME;
    $ENV{VAD_LOG_RESOURCE_NAME} = $VCS_LOG_RESOURCE_NAME;

    if ($OPT_ARG1)
    {
        if ($OPT_ARG1 !~ /\D/)
        {
            $ENV{VCS_LOG_CATEGORY} = $OPT_ARG1;
            $ENV{VAD_LOG_CATEGORY} = $OPT_ARG1;
        }
        elsif ($OPT_ARG1 !~ /[^a-z]/)
        {
            $ENV{VCS_LOG_SCRIPT_NAME} = $OPT_ARG1;
            $ENV{VAD_LOG_SCRIPT_NAME} = $OPT_ARG1;
        }
    }

    if ($OPT_ARG2)
    {
        if ($OPT_ARG2 !~ /\D/)
        {
            $ENV{VCS_LOG_CATEGORY} = $OPT_ARG2;
            $ENV{VAD_LOG_CATEGORY} = $OPT_ARG2;
        }
        elsif ($OPT_ARG2 !~ /[^a-z]/)
        {
            $ENV{VCS_LOG_SCRIPT_NAME} = $OPT_ARG2;
            $ENV{VAD_LOG_SCRIPT_NAME} = $OPT_ARG2;
        }
    }
}


# reset LANG/locale to C
#
# usage:
# VCSAG_SET_LANG ()
sub VCSAG_SET_LANG
{
    unless ($VCS_IS_CLANG_SET)
    {
        $ENV{"LANG"} = $VCS_SUPPLANG;
        $VCS_IS_CLANG_SET = 1;
    }
}


# reset LANG/locale to user defined LANG/locale
#
# usage:
# VCSAG_RESET_LANG ()
sub VCSAG_RESET_LANG
{
    if ($VCS_IS_CLANG_SET)
    {
        $ENV{"LANG"} = $VCS_USERLANG;
        $VCS_IS_CLANG_SET = 0;
    }
}


# for internal use only
#
# usage:
# VCSAG_IS_DBG_ENABLED (<dbg>)
#
# example:
# VCSAG_IS_DBG_ENABLED (1)
sub VCSAG_IS_DBG_ENABLED
{
    my ($dbg) = @_;
    my ($dbg_str) = "DBG_".$dbg;

    # VCSAG_LOGDBG is defined for agents only. take care of triggers.
    return 1 if ($VCSAG_LOGDBG eq "NOT_DEFINED");

    return 1 if ($VCSAG_LOGDBG =~ /(^|\s|,|;)$dbg_str(;|,|\s|$)/);
    return 0;
}


# all perl entry points must use this function if it is not a debug msg.
#
# usage:
# 1. VCSAG_LOG_MSG ("<sev>", "<msg>", <msgid>)
# 2. VCSAG_LOG_MSG ("<sev>", "<msg>", <msgid>, "$param1")
# 3. VCSAG_LOG_MSG ("<sev>", "<msg>", <msgid>, "-encoding <encoding>")
# 4. VCSAG_LOG_MSG ("<sev>", "<msg>", <msgid>, "-encoding <encoding>", "$param1")
#
# example:
# 1. VCSAG_LOG_MSG ("N", "a test message", 1);
# 2. VCSAG_LOG_MSG ("N", "a test message of $test", 1, "$test")
# 3. VCSAG_LOG_MSG ("N", "a test message", 1, "-encoding utf8")
# 4. VCSAG_LOG_MSG ("N", "a test message of $test", 1, "-encoding utf8", "$test")
sub VCSAG_LOG_MSG
{
    # @params includes "-encoding <encoding>" and all the parameters.
    # caller needs to make sure that it has passed the parameters in
    # the correct order.
    # if <encoding> is passed as an argument then the format of the
    # params would be :
    # $params[0] = "-encoding <encoding>"
    # $params[1] = <param1>
    # $params[2] = <param2> and so on.
    # otherwise :
    # $params[0] = <param1>
    # $params[1] = <param2> and so on.
    my ($severity, $msg, $msgid, @params) = @_;
    my ($i, $log_msg) = (0, "");
    my ($encoding, $param_idx, $parameters) = ("", 0, "", "");

    if ($#params >= 0)
    {
        $encoding = $params[0] if ($params[0] =~ /^\s*-encoding\s/);
    }
    $param_idx = 1 if ($encoding);

    for ($i = $param_idx; $i <= $#params; $i++)
    {
        $params[$i] = "%$params[$i]" if ($params[$i] =~ /^-/);
        $parameters .= "\"$params[$i]\" ";
    }

    $parameters = "-parameters " . "$parameters" if ("$parameters");
    $log_msg = "$HALOG -add \"$msg\" -sev $severity -msgid $msgid";
    $log_msg .= " $encoding $parameters";

    $ENV{"LANG"} = $VCS_USERLANG if ($VCS_IS_CLANG_SET);
    if ($VCSAG_OS eq "UNIX")
    {
        system ($log_msg);
    }
    else # if ($VCSAG_OS eq "WINDOWS")
    {
        system ("$log_msg");
    }
    $ENV{"LANG"} = $VCS_SUPPLANG if ($VCS_IS_CLANG_SET);
}


# perl entry points must use this function if they need to fork a command
# using system call
#
# input params :
#   1. a string of command with arguments
#
# output params :
#   1. return value of system($command)
#
# usage:
# 1. $retval = VCSAG_SYSTEM($command);
#
sub VCSAG_SYSTEM
{
    my $retval = 0;
    my $command = $_[0];

    return 0 unless (defined($command) && $command);

    $ENV{"LANG"} = $VCS_USERLANG if ($VCS_IS_CLANG_SET);
    if ($VCSAG_OS eq "UNIX")
    {
        $retval = system ($command);
    }
    else # if ($VCSAG_OS eq "WINDOWS")
    {
        $retval = system ("$command");
    }
    $ENV{"LANG"} = $VCS_SUPPLANG if ($VCS_IS_CLANG_SET);

    return $retval;
}


# perl entry points must use this function if they need to fork a command
# using su call
#
# input params :
#   1. user name
#   2. a string of env variables
#   3. a string of command with arguments
#
# output params :
#   1. return value of system($command)
#
# usage:
# 1. $retval = VCSAG_SU($user, $envvars, @command);
#
sub VCSAG_SU
{
    my ($user, $envvars, @command) = @_;

    if ($envvars ne "")
    {
        return VCSAG_SYSTEM("su $user -c \"$envvars; @command\"");
    }
    else
    {
        return VCSAG_SYSTEM("su $user -c \"@command\"");
    }
}


# sets timeout of the resource's entry point.
# avoid use of this function.
#
# input params :
#   1. timeout value
#
# output params : none
#
# usage:
# 1. VCSAG_SET_RES_EP_TIMEOUT(<timeout>);
#
# example:
# 1. VCSAG_SET_RES_EP_TIMEOUT(130);
#
sub VCSAG_SET_RES_EP_TIMEOUT
{
    my $timeout = @_;
    my $timeout_file = "$VCSLOG/log/tmp/.$ENV{VCS_LOG_RESOURCE_NAME}.tmo";

    open(TMOFILE, ">$timeout_file");
    print TMOFILE "$timeout";
    close(TMOFILE);
}


# all perl entry points must use this function if it is not a debug msg.
#
# usage:
# 1. VCSAG_LOGDBG_MSG (<dbg>, "<msg>")
# 2. VCSAG_LOGDBG_MSG (<dbg>, "<msg>", <msgid>)
# 3. VCSAG_LOGDBG_MSG (<dbg>, "<msg>", <msgid>, "$param1")
# 4. VCSAG_LOGDBG_MSG (<dbg>, "<msg>", <msgid>, "-encoding <encoding>")
# 5. VCSAG_LOGDBG_MSG (<dbg>, "<msg>", <msgid>, "-encoding <encoding>", "$param1")
#
# example:
# 1. VCSAG_LOGDBG_MSG (1, "a test message")
# 2. VCSAG_LOGDBG_MSG (1, "a test message", 1)
# 3. VCSAG_LOGDBG_MSG (1, "a test message of $test", 1, "$test")
# 4. VCSAG_LOGDBG_MSG (1, "a test message", 1, "-encoding utf8");
# 5. VCSAG_LOGDBG_MSG (1, "a test message of $test", 1, "-encoding utf8", "$test")
sub VCSAG_LOGDBG_MSG
{
    # @params includes "-encoding <encoding>" and all the parameters.
    # caller needs to make sure that it has passed the parameters in
    # the correct order.
    # if <encoding> is passed as an argument then the format of the
    # params would be :
    # $params[0] = "-encoding <encoding>"
    # $params[1] = <param1>
    # $params[2] = <param2> and so on.
    # otherwise :
    # $params[0] = <param1>
    # $params[1] = <param2> and so on.
    my ($dbg, $msg, $msgid, @params) = @_;
    my ($i, $log_msg) = (0, "");
    my ($encoding, $param_idx, $parameters) = ("", 0, "", "");

    return if (VCSAG_IS_DBG_ENABLED($dbg) == 0);

    $log_msg = "$HALOG -add \"$msg\" -dbg $dbg";

    if ($msgid)
    {
        $log_msg .= " -msgid $msgid";

        if ($#params >= 0)
        {
            $encoding = $params[0] if ($params[0] =~ /^\s*-encoding\s/);
        }
        $param_idx = 1 if ($encoding);

        for ($i = $param_idx; $i <= $#params; $i++)
        {
            $params[$i] = "%$params[$i]" if ($params[$i] =~ /^-/);
            $parameters .= "\"$params[$i]\" ";
        }

        $parameters = "-parameters " . "$parameters" if ("$parameters");
        $log_msg .= " $encoding $parameters";
    }

    $ENV{"LANG"} = $VCS_USERLANG if ($VCS_IS_CLANG_SET);
    if ($VCSAG_OS eq "UNIX")
    {
        system ($log_msg);
    }
    else # if ($VCSAG_OS eq "WINDOWS")
    {
        system ("$log_msg");
    }
    $ENV{"LANG"} = $VCS_SUPPLANG if ($VCS_IS_CLANG_SET);
}


# remove leading and trailing spaces & squeeze the spaces
#
# input params :
#   1. strings
#
# output params :
#   1. space squeezed strings
#
# usage:
# 1. ($a, ...) = VCSAG_SQUEEZE_SPACES ($a, ...);
#
# example:
# 1. $str1 = VCSAG_SQUEEZE_SPACES ($str1);
# 2. ($str1, $str2) = VCSAG_SQUEEZE_SPACES ($str1, $str2);
# 3. @str = VCSAG_SQUEEZE_SPACES (@str);
# 4. @str = VCSAG_SQUEEZE_SPACES ($str1, $str2);
sub VCSAG_SQUEEZE_SPACES
{
    my (@string) = @_;
    my ($i);

    return if ($#string == -1);

    for ($i = 0; $i <= $#string; $i++)
    {
        $string[$i] =~ s/\s+/ /g;
        $string[$i] =~ s/^\s+//g;
        $string[$i] =~ s/\s+$//g;
    }

    return ($#string == 0) ? $string[0] : @string;
}


# returns value of a given attribute
#
# input params :
#   1. attribute name
#   2. index of attribute (optional). value starts from 0.
#   3. arglist [should start from attibutes. ResourceName and
#               CleanReason should not be passed]
#
# output params :
#   1. returns VCSAG_SUCCESS when successful
#   2. index of attribute if success
#
sub VCSAG_GET_ATTR_INDEX
{
    my ($attr, @arglist) = @_;
    my ($i, $index_of_attr, $num_of_args, $num_of_vals) = (0, -1, 0, 0);

    VCSAG_SQUEEZE_SPACES ($attr);

    return VCSAG_PRINT_ERROR ($VCSAG_NULL_ATTR) if ($attr eq "");
    return VCSAG_PRINT_ERROR ($VCSAG_NULL_ARGLIST, $attr) if ($#arglist == -1);

    if ($arglist[0] eq "-1" || $arglist[0] !~ /\D/)
    {
        $index_of_attr = $arglist[0];
        shift (@arglist);
        return VCSAG_PRINT_ERROR ($VCSAG_NULL_ARGLIST, $attr) if ($#arglist == -1);
    }

    $num_of_args = $#arglist + 1;

    if ($index_of_attr >= 0 && $index_of_attr < $num_of_args &&
        $attr eq $arglist[$index_of_attr])
    {
        return ($VCSAG_SUCCESS, $index_of_attr);
    }

    for ($i = 0; $i < $num_of_args; $i += 2)
    {
        return ($VCSAG_SUCCESS, $i) if ($arglist[$i] eq $attr);

        if (($i+1) >= $num_of_args)
        {
            return VCSAG_PRINT_ERROR ($VCSAG_INVAL_ARGLIST, $attr);
        }

        $num_of_vals = $arglist[$i+1];
        if ($num_of_vals =~ /\D/)
        {
            return VCSAG_PRINT_ERROR ($VCSAG_INVAL_NUMVALS, $attr, $num_of_vals);
        }

        $i += $num_of_vals;

        if (($i+1) >= $num_of_args)
        {
            return VCSAG_PRINT_ERROR ($VCSAG_INVAL_ARGLIST, $attr);
        }
    }

    return VCSAG_PRINT_ERROR ($VCSAG_ATTR_NOTFOUND, $attr);
}


# returns value of a given attribute
#
# input params :
#   1. attribute name
#   2. index of attribute (optional). value starts from 0.
#   3. index of value required (optional. must be passed alongwith index
#                               of attribute) [valid only for vector and
#                               association] value starts from 1.
#   4. arglist [should start from attibutes. ResourceName and
#               CleanReason should not be passed]
#
# output params :
#   1. returns VCSAG_SUCCESS when successful
#   2. value of attribute if success
#   2. index of attribute if success
#
sub VCSAG_GET_ATTR_VALUE
{
    my ($attr, @arglist) = @_;
    my ($index_of_attr, $index_of_val) = (-1, 0);
    my ($i, $num_of_args, $num_of_vals) = (0, 0, 0);
    my (@retval, $arg_index_of_val);

    return VCSAG_PRINT_ERROR ($VCSAG_NULL_ATTR) if ($attr eq "");
    return VCSAG_PRINT_ERROR ($VCSAG_NULL_ARGLIST, $attr) if ($#arglist == -1);

    if ($arglist[0] eq "-1" || $arglist[0] !~ /\D/)
    {
        $index_of_attr = $arglist[0];
        shift (@arglist);
        return VCSAG_PRINT_ERROR ($VCSAG_NULL_ARGLIST, $attr) if ($#arglist == -1);
    }

    if ($arglist[0] eq "-1" || $arglist[0] !~ /\D/)
    {
        $index_of_val = $arglist[0];
        shift (@arglist);
        return VCSAG_PRINT_ERROR ($VCSAG_NULL_ARGLIST, $attr) if ($#arglist == -1);
    }

    @retval = VCSAG_GET_ATTR_INDEX ($attr, $index_of_attr, @arglist);
    return $retval[0] if ($retval[0] ne $VCSAG_SUCCESS);

    $index_of_attr = $retval[1];
    $num_of_args = $#arglist + 1;
    if (($index_of_attr+1) >= $num_of_args)
    {
        return VCSAG_PRINT_ERROR ($VCSAG_INVAL_ARGLIST, $attr);
    }

    $num_of_vals = $arglist[$index_of_attr+1];
    if ($num_of_vals =~ /\D/)
    {
        return VCSAG_PRINT_ERROR ($VCSAG_INVAL_NUMVALS, $attr, $num_of_vals);
    }

    if ($index_of_val < 1)
    {
        return ($VCSAG_SUCCESS, $num_of_vals, $index_of_attr);
    }

    if ($index_of_val > $num_of_vals)
    {
        return VCSAG_PRINT_ERROR ($VCSAG_INVAL_INDEX, $attr,
                                  $index_of_val, $num_of_vals);
    }

    $arg_index_of_val = $index_of_attr+1+$index_of_val;
    return VCSAG_PRINT_ERROR ($VCSAG_INVAL_ARGLIST, $attr) if (($arg_index_of_val) >= $num_of_args);

    return ($VCSAG_SUCCESS, $arglist[$arg_index_of_val], $index_of_attr);
}


# print debug message for a given error id
#
sub VCSAG_PRINT_ERROR
{
    my ($error_id, @args) = @_;
    my ($error_string) = ("");

    if ($error_id == $VCSAG_NULL_ATTR)
    {
        $error_string = "Attribute argument is pointing to NULL.";
    }
    elsif ($error_id == $VCSAG_NULL_ARGLIST)
    {
        $error_string = "Attribute:$args[0]:Argument list is pointing to NULL.";
    }
    elsif ($error_id == $VCSAG_ATTR_NOTFOUND)
    {
        $error_string = "Attribute $args[0] not found in the argument list.";
    }
    elsif ($error_id == $VCSAG_INVAL_ARGLIST)
    {
        $error_string = "Attribute:$args[0]:Argument list is not valid.";
    }
    elsif ($error_id == $VCSAG_INVAL_INDEX)
    {
        $error_string = "Attribute:$args[0]:Index $args[1] is not valid. Total number of elements is $args[2].";
    }
    elsif ($error_id == $VCSAG_INVAL_NUMVALS)
    {
        $error_string = "Attribute:$args[0]:Number of elements $args[1] is not a number.";
    }
    elsif ($error_id != $VCSAG_SUCCESS)
    {
        $error_string = "Unknown error id:$error_id.";
    }

    VCSAG_LOGDBG_MSG (1, $error_string) if ($error_string ne "");
    return ($error_id, @args);
}

1;
