/*
 * $Copyright: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#ifndef _VX_IMC_H
#define _VX_IMC_H

static char sccsid_gab_imc_h[] = "@(#)comms:$RCSfile: gab_imc.h,v $     $Revision: 1.3.2.3 $";

#include <linux/version.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18)

#include <linux/types.h>
#include <linux/fs.h>
#include <linux/file.h>
#include <linux/list.h>
#include <linux/module.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
#define GAB_VXIMC_FILEPATH_ABS	"/proc/vximc/"
#define GAB_VXIMC_FILEPATH_REL	"vximc"
#else
#define GAB_VXIMC_FILEPATH_ABS	"/proc/sys/vximc/"
#define GAB_VXIMC_FILEPATH_REL	"sys/vximc"
#endif

struct vx_imc_ioctl {
	char	symname[32];	/* name of symbol */
	void	*addr;		/* address of symbol */
};

struct vx_imc_symbol {
	struct list_head	slist;		/* list of symbols */
	struct vx_imc_ioctl	ioctl;
};

struct vx_imc_module {
	struct list_head	mlist;		/* list of modules */
	struct list_head	slist;		/* list of symbols */
	struct file		*file;		/* file */
	atomic_t		cnt;		/* reference count */
	char			modname[32];	/* name of module */
};

extern int gab_imc_export(char *, void *);
extern int gab_imc_unexport(char *);
extern void* gab_imc_import(char *, char *);
extern void gab_imc_deport(char *);
extern void gab_imc_init(struct module *);
extern void gab_imc_exit(struct module *);

#define VX_IMC_INIT			gab_imc_init(THIS_MODULE)
#define VX_IMC_EXIT			gab_imc_exit(THIS_MODULE)

#define VX_REGISTER_SYMBOL(sym, mod, addr) 	gab_imc_export((sym), (addr))
#define	VX_UNREGISTER_SYMBOL(sym)		gab_imc_unexport((sym))
#define VX_GET_SYMBOL(sym, mod)			gab_imc_import(sym, mod)
#define VX_PUT_SYMBOL(sym)			gab_imc_deport(sym)

#else

#define VX_REGISTER_SYMBOL(sym, mod, addr) inter_module_register(sym, mod, addr)
#define VX_UNREGISTER_SYMBOL(sym)	inter_module_unregister(sym)
#define VX_GET_SYMBOL(sym, mod)		inter_module_get_request(sym, mod)
#define VX_PUT_SYMBOL(sym)		inter_module_put(sym)

#define VX_IMC_INIT
#define VX_IMC_EXIT

#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16) */

#endif /* _VX_IMC_H */
