/*
 * $Copyright: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#ifndef GABIOC_H
#define GABIOC_H

static char sccsid_gabioc_h[] = "@(#)comms:$RCSfile: gabioc.h,v $	$Revision: 4.14.2.5 $";

#ifdef HPUX
#include <sys/ioctl.h>
#include <sys/sysmacros.h>
#endif

#ifdef SOLARIS
#include <sys/ioccom.h>
#include <sys/mkdev.h>
#endif

#if defined(LINUX) && !defined(_KERNEL)
#include <sys/types.h>
#include <sys/ioctl.h>
#endif

#ifdef _AIX
#include <sys/stream.h>
#include <stropts.h>
#endif

#include <sys/gabbid.h>
#include <gabhdr.h>
#include <sys/gabvers.h>

#if defined(WINNT) || defined(LINUX)
#define GAB_MAJOR(dev) 		0
#define GAB_MINOR(dev) 		0
#define GAB_MAKEDEV(x, y, z)	/* empty */
#endif

#if (defined(SOLARIS) || defined(HPUX) || defined(_AIX)) && !defined(_KERNEL)
#define GAB_MAJOR(dev)       	major(dev)
#define GAB_MINOR(dev)       	minor(dev)
#define GAB_MAKEDEV(major, minor, dev) ((dev) = makedev((major), (minor)))
#endif 

/*  The alignment issue must be solved for all i386 platforms, not just linux.
 * #if defined(LINUX)
 * #define GAB_ALIGNTO(x)		__attribute__ ((aligned(x)))
 * #else
 * #define GAB_ALIGNTO(x)		/ * empty * /
 * #endif
 */

#ifdef NTGAB 

typedef	INT_PTR 	dev_t;

/*
// IOCTL codes
// The NTDDK gives only 10 bits for encoding ioctl codes. This is
// not sufficient for storing the lengths of the input and output
// parameters within the ioctl code.
*/
#define NTGAB_DEVICE_TYPE            0xA500

#define GAB_KRL_DEV_NAME    L"\\\\.\\gab_0"

#ifndef PLAT_IOWR
#define PLAT_IOWR(a, code, type)    \
    CTL_CODE(NTGAB_DEVICE_TYPE, code, METHOD_BUFFERED, FILE_ANY_ACCESS)
#endif

#ifndef PLAT_IOW
#define PLAT_IOW    PLAT_IOWR
#endif

#ifndef PLAT_IOR
#define PLAT_IOR    PLAT_IOWR
#endif

#ifndef PLAT_BZERO
#define PLAT_BZERO  RtlZeroMemory
#endif

/* for Singal emulation */
#ifndef	SIGKILL
#define	SIGKILL	9	/* kill (cannot be caught or ignored) */
#endif

#ifndef	SIGQUIT
#define	SIGQUIT	9	/* NT use SIGKILL as SIGQUIT for now  */
#endif

#ifndef	SIGUSR1
#define	SIGUSR1	16	/* user defined signal 1 */
#endif

#else	/* NTGAB */

#ifndef PLAT_IOWR
#define PLAT_IOWR   _IOWR
#endif

#ifndef PLAT_IOW
#define PLAT_IOW    _IOW
#endif

#ifndef PLAT_IOR
#define PLAT_IOR    _IOR
#endif


#endif /* NTGAB */

#define GAB_DEPEND_MAX 20

/* structure used by gabconfig to configure driver
 */
typedef struct gab_config_t {
	int	gf_buildid;
	int 	gf_iofence;
	int 	gf_stable;
	int	gf_dskhbint;
	int	gf_dskmiss;
	int	gf_nodecnt;
	int 	gf_flags;
} gab_config_t;

/* structure used by gabconfig to configure driver
 */
typedef struct gab_ioc_config {
	int		gi_cmd;
	gab_config_t	gi_config;
} gab_ioc_config_t;

/* gab configuration ioctl command
 */
#define GAB_IOCFG_CONFIG   1
#define GAB_IOCFG_UNCONFIG 2
#define GAB_IOCFG_LIST     3

/* ioc test send structure
 */

typedef struct {
	int		gt_mode;	/* what mode (state) */
	unsigned int	gt_point;	/* apply at which test point */
	unsigned int	gt_v1;
	unsigned int	gt_v2;
	/* long long	gt_stat_nall;*/	/* statistics: # of attempts */
	/* long long	gt_stat_nfail;*/ /*  and # of emulated failures */
} gab_ioc_simtest_t;

enum gab_simtest_state {	/* ioc args for each mode/state/op */
        GAB_TS_ENABLE,			/* mode point */
        GAB_TS_DISABLE,			/* mode point */
        GAB_TS_PANIC,			/* mode point */
        GAB_TS_RANDOM_SIZE,		/* mode point v1 v2 */
        GAB_TS_FIXED_SIZE,		/* mode point v1 v2 */
	GAB_TS_NOP,			/* mode point */
	GAB_TS_TIME_BIT = 8,
        GAB_TS_RANDOM_TIME,		/* mode point v1 v2 */
        GAB_TS_FIXED_TIME		/* mode point v1 v2 */
};

#if defined(HPUX) || defined(SOLARIS) || defined(LINUX) || defined(_AIX) || defined (_WIN64)

typedef struct gab_iptr_t {
        unsigned long long gi_val;
} gab_iptr_t;

#if defined(BITS) && BITS == 32
#define GAB_IPTR_SET(gi, val)   ((gi).gi_val = PLAT_PTR_2ULONG(val))
#else
#define GAB_IPTR_SET(gi, val)   ((gi).gi_val = (unsigned long long) (val))
#endif
#define GAB_IPTR_GET(gi)        (PLAT_NUM2PTR((gi).gi_val))
#define GAB_IPTR_VAL(gi)        ((gi).gi_val)
#define GAB_IPTR_ADD(dst, src, val)   ((dst).gi_val = (src).gi_val + val)
#define KPTR             "%llx"

#elif defined(WIN32)   /* NT */

typedef struct gab_iptr_t {
        LARGE_INTEGER gi_val;
} gab_iptr_t;

#define GAB_IPTR_SET(gi, val) \
        ((gi).gi_val.LowPart = (unsigned) (val), \
        (gi).gi_val.HighPart = (unsigned) (((unsigned long)(val))>>32))
#define GAB_IPTR_GET(gi) \
        ((void*)(gi).gi_val.LowPart)
#define GAB_IPTR_VAL(gi)        ((gi).gi_val.LowPart)
#define GAB_IPTR_ADD(dst, src, val)   \
        ((dst).gi_val.LowPart = (src).gi_val.LowPart + (unsigned) (val), \
        (dst).gi_val.HighPart = (src).gi_val.HighPart +       \
				((unsigned long)(val)) >> 32)
#define KPTR             "%x"
#endif

#define GAB_MON_STAT_ARRAY_N      5

typedef struct {
	int	hb_interval;	/* client heartbeat timeout in msec*/
	int	hb_warntime;
	int	hb_stat[GAB_MON_STAT_ARRAY_N];
} gab_port_hb_t;

/*
 * Structure used by gabport get port information
 */
typedef struct {
	char		gg_portid;      /* Port to display /set options for*/
	int		gg_buildid;
	int		gg_flags;
	int		gg_clntflags;
	gab_mbr_t	gg_membrlist;
	gab_port_hb_t	gg_porthb;
}gab_gabport_t;


#ifdef _AIX 
#define MAXDISKNAME 36
#else
#define MAXDISKNAME 100
#endif /* ifdef _AIX */

#define GAB_DISK_SIGNSIZE   8
#define GAB_DISK_MAXMSG   500 /* (1024-sizeof(gab_dhdr_t)-sizeof(uint)*2)/2 */
#define GAB_DISK_MSGCNT     2
#define GAB_BSIZE        1024
#if (defined(SOLARIS) || defined(HPUX)) && !defined(_KERNEL)
#include <sys/param.h>
#define PLAT_DEV_BSIZE DEV_BSIZE
#elif defined(_AIX) && !defined(_KERNEL)
#include <sys/dir.h> 
#define PLAT_DEV_BSIZE DEV_BSIZE
#elif defined(WINNT) || defined(LINUX)
#define PLAT_DEV_BSIZE   512
#endif
/*
 * Disk signature = 
 * system component(3 bytes) + nid(1 byte) + user component(4 bytes).
 */
#define GAB_DISK_SIGPREFIX "GAB"
#define GAB_DISK_SYSSIGSZ    3
#define GAB_DISK_USRSIGSZ    4

typedef struct gab_dhdr_t {
        char   dh_signature[GAB_DISK_SIGNSIZE];
        unsigned int    dh_acc_seq;
        unsigned int    dh_own_seq;
} gab_dhdr_t;

typedef struct gab_dmsg_t {
        unsigned int dm_msg_seq;
        unsigned char dm_msg[GAB_DISK_MAXMSG];
} gab_dmsg_t;

typedef struct gab_dblk_t {
        gab_dhdr_t db_hdr;
        gab_dmsg_t db_msg[GAB_DISK_MSGCNT];
} gab_dblk_t;

typedef struct gab_disk_info_t {
	int	gi_flags;		/* flags reuse config/unconfig  */
	int	gi_buildid;		/* utility/kernel match check	*/
	union {
		dev_t	dev;		/* disk device (maj/min)	*/
		char	fill[8];
	} gi_udev;
#define gi_dev 	gi_udev.dev
	unsigned int gi_major;
	unsigned int gi_minor;
	unsigned int gi_portid;		/* portid assigned to		*/
	int	gi_start;		/* 64K area starting at .	*/
	char 	gi_name[MAXDISKNAME];	/* dev filename			*/
	gab_mbr_t gi_active;		/* active peers			*/
	gab_mbr_t     gi_owner;		/* owner of disk region		*/
        int          gi_curr;                /* debug only */
        int          gi_prev;                /* debug only */
        int          gi_count;		/*have we initalized signature? */
        char    gi_signature[GAB_DISK_SIGNSIZE];
} gab_disk_info_t;

#ifdef NTGAB
typedef struct {
	int		sig_flag;	/* flag for singaling irp	*/
	int		sig_type;	/* type of signaling		*/
} gab_sig_info_t;

#define GAB_SIG_SUCCESS 0x01		// Use Irp to Process New Signal
#define GAB_SIG_PENDING 0x02		// No Irp, Signal Pending
#define GAB_SIG_MAXIRP	0x04		// max. irp reached
#define GAB_SIG_ERROR	0x80		// Error case

typedef struct {
	char		gi_name[MAXDISKNAME];
	int		gi_start;
} gab_xdisk_info_t;
#endif //NTGAB


/* config flags options
 */
#define GAB_CONFIGURE	0x01	/* configure; turned off by -l for list	*/
#define GAB_UNCONFIG	0x02	/* -u un-configure gab driver		*/
#define GAB_MONITOR	0x04	/* monitor resource			*/
#define GAB_RESET	0x08	/* used to clear faults			*/
#define GAB_SINGLENET	0x10	/* -s disable partition arbitration	*/
#define GAB_SEED	0x20	/* -x seed pre-existing partition	*/
#define GAB_ALIEN	0x40	/* used internally in CONNECTS	msg */
#define GAB_USRDTHHALT  0x80	/* halt system on user death		*/
#define GAB_ENABLE_HB_HALT 0x100 /* halt on client heartbeat failure */
#define GAB_ENABLE_REJOIN_HALT 0x200 /* don't halt on rejoin after split */
#define GAB_RESTARTENABLE   0x400 /* enable restart of port 'a'          */
#define GAB_KILLING_FOREVER 0x800 /* keep on killing process until success */
#define GAB_ENABLE_IOFENCE_QUORUM  0x1000 /* enable iofence cluster when no quorum */
#define GAB_DISABLE_IOFENCE_QUORUM 0x2000 /* disable iofence cluster when no quorum */
#define GAB_PENDING_RFG	0x4000		/* there are undelivered RFG
					 * use internally in CONNECTS msg */
#define GAB_DISABLE_REJOIN_HALT 0x8000 /* halt on rejoin after split */
#define GAB_DISABLE_HB_HALT 0x10000 /* don't halt on client heartbeat failure */
#define GAB_DISABLE_USRDTHHALT  0x20000	/* halt system on user death		*/


/*
 * gabdisk flags.
 */
#define GAB_DISK_CONFIG         0x01 /* Add disk object */
#define GAB_DISK_UNCONFIG       0x02 /* Delete disk object */
#define GAB_DISK_CLEAR_FAULTED  0x04 /* Reset faulted disk */
#define GAB_DISK_RDONLY	        0x08 /* Read only disk */
#define GAB_DISK_FAULTED        0x10 /* used to mark disk as faulted	*/
#define GAB_DISK_MONITOR        0x20 /* Monitor disk */
#define GAB_XDISK_GETOWN        0x40 /* Get disk ownership */
#define GAB_XDISK_SETOWN        0x80 /* Set disk ownership */
#define GAB_XDISK_CHKDISK      0x100 /* Reset disk ownership */
#define GAB_XDISK_RESETOWN     0x200 /* Reset disk ownership */
#define GAB_DISK_GETDISK       0x400 /* Retrieve next disk */
#define GAB_DISK_EXCLUSIVE     0x800 /* exclusive disk */
#define GAB_DISK_HEARTBEAT    0x1000 /* heartbeat disk */
#define GAB_DISK_USRWAIT      0x2000 /* User proc waiting for I/O completion */
#define GAB_DISK_INITIALIZING 0x4000 /* GAB disk is being initialized */
#define GAB_DISK_RESERVE      0x8000 /* Keep disk reserved */
#define GAB_DISK_PROBE       0x10000 /* Probe region for initial values. */
#define GAB_DISK_CHKSIG      0x20000 /* Check for disk signature */
#define GAB_DISK_INFO        0x40000 /* Debug only */
#define GAB_DISK_OPENED      0x80000 /* Disk device opened successfully */

/* 
 * Mask for filtering out the readonly/heartbeat/exclusive bits.
 */
#define GAB_DISK_TYPE (GAB_DISK_EXCLUSIVE | GAB_DISK_HEARTBEAT)
#define GAB_DISK_FILTER ~(GAB_DISK_RDONLY | \
                          GAB_DISK_EXCLUSIVE | \
                          GAB_DISK_HEARTBEAT | \
                          GAB_DISK_CHKSIG)

/* gabport input flags 
*/
#define GABPORT_GETINFO		0x01
#define GABPORT_SETHBTIMEOUT	0x02

/* kernel tunables
 */
#define GAB_NUMNIDS 		128
#define GAB_NUMPORTS		32
#define GAB_FLOWCTRL		5
#define GAB_LOGBUFSIZE		48100
#define GAB_LOG_FLAG            0x3D
#define GAB_LOWMEM_TIMEOUT	1000	/* 1 sec */
#define GAB_ISOLATE_TIMEOUT     120000  /* 2 minutes */
#define GAB_KILL_NTRIES         5
#define GAB_TIMER_PRI           17

/* configuration defaults
 */
#define GAB_NUMDEVS 		16
#define GAB_FENCEWAIT           15000
#define GAB_STABLEWAIT          5000
#define GAB_DSKHBINT		1000
#define GAB_DSKMISS		4
#define GAB_NODECNT		0
#define GAB_SYNCDEFAULT		120000	/* 2 minutes */

/* portid range
 */
#define GAB_LOWID  'a'
#define GAB_HIGHID 'z'
#define GAB_NULLID 0x7f

/* struture used by gab library to perform gab_register()
 */
typedef struct {
        int 			flags;
        int 			timeout;
	int			pid;
        int 			signal;
	unsigned int 		portid;
	unsigned int 		buildid;
	char			depend[GAB_DEPEND_MAX];
} gab_reg;

/*
 * structure for getting gab tunables
 */
typedef struct {
        unsigned int gk_max_cpu;
        unsigned int gk_numnids;
        unsigned int gk_numports;
        unsigned int gk_flowctrl;
        unsigned int gk_logbufsize;
        unsigned int gk_msglogsize;
        unsigned int gk_logflag;
        unsigned int gk_isolate_time;
        unsigned int gk_kill_ntries;
        unsigned int gk_conn_wait;
#ifdef _AIX
        unsigned int gk_timer_pri;
#endif
} gab_ktunables_t;

typedef struct {
	unsigned int		gen;
	gab_mbr_t		cur;
	gab_mbr_t		jeop;
} gab_display;

typedef struct {
	int	gs_msec;		/* duration of sync state in ms */
	int	gs_buildid;		/* utility/kernel match check	*/
} gab_sync_t;


#define GAB_IOC_CONFIG		PLAT_IOWR('g', 1, gab_ioc_config_t)
#define GAB_IOC_REG		PLAT_IOWR('g', 2, gab_reg)
#define GAB_IOC_UNREG		PLAT_IOW( 'g', 3, int)
#define GAB_IOC_HB		PLAT_IOW( 'g', 4, int)
#define GAB_IOC_DISPLAY		PLAT_IOWR('g', 5, gab_display)
#define GAB_IOC_DISK		PLAT_IOWR('g', 6, gab_disk_info_t)
#define GAB_IOC_SYNC		PLAT_IOW( 'g', 7, gab_sync_t)
#define GAB_IOC_SIMTEST 	PLAT_IOWR('g', 0x12, gab_ioc_simtest_t)
#define GAB_IOC_PORT 		PLAT_IOWR('g', 0x13, gab_gabport_t)
#define GAB_IOC_DEBUG 		PLAT_IOWR('g', 0x14, gab_debug_t)
#define GAB_IOC_PANIC		PLAT_IOW( 'g', 0x15, gab_panic_t)
#define GAB_IOC_KTUNE		PLAT_IOR( 'g', 0x16, gab_ktunables_t)
#define GAB_IOC_VER		PLAT_IOR( 'g', 0x17, int)

#ifdef NTGAB
#define GAB_IOC_WAIT_SIG	PLAT_IOWR('g', 8, gab_sig_info_t)
#define GAB_IOC_USER_INT	PLAT_IOWR('g', 9, NULL)
#define GAB_IOC_XDISK_USER_INT  PLAT_IOWR('g', 0xa, gab_xdisk_info_t)
#define GAB_IOC_WAIT_KILL	PLAT_IOWR('g', 0xb, gab_sig_info_t)
#define GAB_IOC_GETAPI  	PLAT_IOWR('g', 0xd, (gab_api_t *))
#define GAB_IOC_RELEASEAPI 	PLAT_IOWR('g', 0xf, void)
#endif //NTGAB

/*
 * GAB_IOC_PANIC reason constant
 */
#define GAB_NOMEM	1

typedef struct {
	int		reason;
	unsigned int	bugcheck_code;	/* windows only */
	unsigned int	param1;		/* windows only */
	unsigned int	param2;		/* windows only */
} gab_panic_t;
/*
 * NB:  The following structure contains 64 bit entities that follow
 *      32 bit entities.  Some platforms will align 64 bit things to
 *      32 bit boundaries.  Be certain that you align things correctly
 *      by adding padding fields, if necessary.   Do *NOT* use alignment
 *      attributes that work on one platform and cause another to choke.
 */

typedef struct {
	int		   gd_arg;
	unsigned           gd_cnt;
        int                gd_size;
	int		   gd_buildid;
	int 		   gd_cmd;
	int		   gd_filler;   /* All i386 platforms require this,
					   all others implicitly insert it */
	gab_iptr_t 	   gd_buff;
        gab_iptr_t         gd_kaddr;
} gab_debug_t;

/* gabdebug input options 
*/
#define GAB_DEBUG_SET   	 1	/* Set the debug level 	*/
#define GAB_GET_DEBMSK           2      /* Get the debug level  */
#define GAB_PRINT_PORT           3      /* print kernel port struct */
#define GAB_PRINT_QUE            4      /* print kernel que struct */
#define GAB_PRINT_MSG            5      /* print kernel msg struct */
#define GAB_PRINT_WORDS          6      /* print kernel memory   */
#define GAB_PRINT_LOG            7      /* print kernel port log */

#define GAB_PRINT_DEBLOG         8      /* print deblog */
#define GAB_GET_DEBLOG_BUF_SIZE  9      /*                      */
#define GAB_GET_MSGLOG_SIZE     10      /* Get per port msg. log size */
#define GAB_GET_MSGLOG          11      /* Get per port msg. log */
#define GAB_SET_MSGMSK          12      /* Set the msg log level */
#define GAB_GET_MSGMSK          13      /* Get the msg log level */
#define GAB_PRINTRST_DEBLOG     14      /* print and reset deblog */
#define GAB_GET_DEBLOG		15	/* Get debug log */
#define GAB_SET_DEBMSK		16	/* Set debug log mask */
#define GAB_GET_FLWMLOGSZ       17      /* Get per port msg. log size */
#define GAB_GET_FLWMSGLOG       18      /* Get per port flowcontrol log */
#define GAB_GET_CLNTMLOG_SIZE	19      /* Get per port client msg. log size */
#define GAB_GET_CLNTMLOG	20      /* Get per port client msg. log */
#define GAB_GET_MLOG_TUNABLE	21      /* Get msg. log size tunable */

/* gab debug message destination options
 */
#define GAB_DBG_PRINT_CONSOLE   1       /* send debug messages to console */
#define GAB_DBG_PRINT_BUFFER    2       /* send debug messages to internal buffer */
#define GAB_DBG_PRINT_ALL       3       /* send debug messages to console and buffer */

#endif /* GABIOC_H */
