/*
 * $Copyright: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 *
 */

#ifndef _LLTAPI_H
#define _LLTAPI_H

static char sccsid_lltapi_h[] = "@(#)comms:$RCSfile: lltapi.h,v $	$Revision: 4.55.2.15 $";

#if defined(SOLARIS) || defined(HPUX) || defined(AIX)
#ifndef LLTUNIX
#define LLTUNIX	1	/* XXX should be UNIX but already used in os.h */
#endif
#endif

#if defined(HPUX) /* { */
#ifdef _KERNEL /* { */

#include <elf.h>

#else /* _KERNEL }{ */

#if !defined(_UINT_T) && !defined(HPUX1123)
#define _UINT_T
typedef unsigned char   uchar_t;
typedef unsigned short  ushort_t;
typedef unsigned int    uint_t;
#endif

#endif /* _KERNEL } */
#endif /* HPUX } */


#if defined(LINUX)

#ifdef _KERNEL
#include <linux/skbuff.h>
#include <linux/if.h>

#ifndef LLT_MBLK
#define LLT_MBLK
typedef struct sk_buff 	llt_mblk_t;
typedef struct sk_buff 	mblk_t;	/* clients of LLT use mblk_t. */
#endif

#else /* _KERNEL */

#include <net/if.h>
#include <inttypes.h>
#endif /* _KERNEL */

#ifdef _KMEMUSER
typedef char		llt_mblk_t;
typedef char		mblk_t;
#endif /* _KMEMUSER } */

#define LLT_MAX_DEVNAME_LEN     IFNAMSIZ
#define LLT_MAX_SYMNAME_LEN     128

#ifndef _UINT_T
#define _UINT_T
typedef unsigned int 	uint_t;
typedef unsigned short 	ushort_t;
typedef unsigned char 	uchar_t;
typedef unsigned long 	ulong_t;
#endif

#endif	/* defined(LINUX) */

#ifdef LLTUNIX
#include <sys/stream.h>
typedef mblk_t		llt_mblk_t;
#endif /* LLTUNIX */

#define LLT_DEVICE		"/dev/llt"
#define LLT_NAME		"llt"

/*
 * maximum supported protocol version
 */
#define LLT_VER_MAJ		4
#define LLT_VER_MIN		0

/*
 * maximum 8-bit cluster ID supported version
 */
#define LLT_SMALLCID_VER_MAJ	3
#define LLT_SMALLCID_VER_MIN	7

/*
 * LLT protocol version that support 16-bit cluster ID
 * we auto-update our protocol major/minor version to this,
 * if 16-bit cluster ID is configured.
 */
#define LLT_BIGCID_VER_MAJ	4
#define LLT_BIGCID_VER_MIN	0

#define LLT_MAX_LINKS		8
#define LLT_MAX_TAG_LEN		12
#define LLT_MAX_MAC_LEN		10
#define LLT_MAX_USAGE_LEN	12

/*
 * reasons for CLOSENODE
 */
#define LLT_PANIC	0x0
#define LLT_TIMEOUT	0x1
#define LLT_RESET	0x2
#define LLT_SHUTDOWN	0x3
#define LLT_CLOSE	0x4

/*
 * values for connect_func() event code
 */
#define LLT_CONNECT		0x01	/* one or more links online */
					/* reason contains number of links */
#define LLT_DISCONNECT		0x02	/* all links offline */
					/* reason contains llt_close_t */
#define LLT_DOORBELL		0x03	/* peer node wants to connect */

typedef struct llt_user_s {
	ushort_t	lu_cmd;	
	ushort_t	lu_nid;

#ifdef UNIX
	uchar_t		lu_event;
	uchar_t		lu_reason;
#endif

#ifdef LINUX
   	uint_t          lu_len;		/* actual len */
        uint_t          lu_buflen;	/* buffer len */
        uchar_t         *lu_buf;
#endif
} llt_user_t;

/*
 * on unix: values for CTL part of user-mode STREAMS requests
 * or linux: llt_user_t.lu_cmd is set to one of XXX_REQ
 */
#define LLT_CONNECT_IND		0x81
#define LLT_RECV_IND		0x82
#define LLT_OPEN_REQ		0x83
#define LLT_SEND_REQ		0x84
#define LLT_DGRAM_REQ		0x85
#define LLT_BCAST_REQ		0x86
#define LLT_RECV_REQ    	0x05	/* linux only */

#ifdef LINUX

/* parameter for IOCTL_LLT_SETPORT */
typedef struct llt_setport_s {
	uint_t	llt_port;
	uchar_t	llt_usage[LLT_MAX_USAGE_LEN];
} llt_setport_t;

#endif	/* LINUX */

#ifndef _KERNEL /* { */
/*
 * structure for lltgethostbyname() and lltgethostbyaddr()
 */
typedef struct llt_hostent_s {
	char	*h_name;	/* name of host */
	int	h_length;	/* length of address */
	char	*h_addr;	/* LLT node ID */
} llt_hostent_t;

extern llt_hostent_t *lltgethostbyname(char *);
extern llt_hostent_t *lltgethostbyaddr(uint_t);

#endif /* !_KERNEL } */

/*
 * record packet and byte count for last 5, 30, 60 seconds.
 */
typedef struct llt_trafstat_s {
	uint_t	ts_cnt;
	uint_t	ts_cnt1;
	uint_t	ts_cnt10;
	uint_t	ts_cnt30;	
	uint_t	ts_byt;
	uint_t	ts_byt1;
	uint_t	ts_byt10;
	uint_t	ts_byt30;	
} llt_trafstat_t;

#define LLT_MAXPKTS2TRACK	32
#define LLT_POWER2BUCKETS	4
#define LLT_MAXLATBUCKETS	((1 << LLT_POWER2BUCKETS) + 1)

/*
 * LLT ioctl
 */
#ifdef LLTUNIX

#define LLT_IOCTL(x)	(('L' << 8) | (x))

#define	IOCTL_LLT_GETSTATE	LLT_IOCTL(100)
#define	IOCTL_LLT_SETSTATE	LLT_IOCTL(101)
#define	IOCTL_LLT_SETPORT	LLT_IOCTL(102)
#define	IOCTL_LLT_GETPARAM	LLT_IOCTL(103)
#define	IOCTL_LLT_SETPARAM	LLT_IOCTL(104)
#define	IOCTL_LLT_SETNODERANGE	LLT_IOCTL(105)
#define	IOCTL_LLT_GETSTATS	LLT_IOCTL(106)
#define	IOCTL_LLT_CLRSTATS	LLT_IOCTL(107)
#define	IOCTL_LLT_GETNODEINFO	LLT_IOCTL(108)
#define	IOCTL_LLT_GETPORTINFO	LLT_IOCTL(109)
#define	IOCTL_LLT_GETLINKINFO	LLT_IOCTL(110)
#define	IOCTL_LLT_SETLINKINFO	LLT_IOCTL(111)
#define	IOCTL_LLT_GETNODEADDR	LLT_IOCTL(112)
#define	IOCTL_LLT_SETNODEADDR	LLT_IOCTL(113)
#define	IOCTL_LLT_DELNODEADDR	LLT_IOCTL(114)
#define	IOCTL_LLT_FLUSHNODEADDR	LLT_IOCTL(115)
#define	IOCTL_LLT_ATTACH	LLT_IOCTL(116)
#define	IOCTL_LLT_CLEAR_ATTACH	LLT_IOCTL(117)
#define	IOCTL_LLT_GETKTUNE	LLT_IOCTL(119)

#define	IOCTL_LLT_GETAPI	LLT_IOCTL(200)
#define	IOCTL_LLT_REINIT	LLT_IOCTL(201)
#define	IOCTL_LLT_DBGPRINT	LLT_IOCTL(222)
#define	IOCTL_LLT_QSTAT		LLT_IOCTL(223)
#define	IOCTL_LLT_DBGINFO	LLT_IOCTL(224)
#define	IOCTL_LLT_GETNODEALL	LLT_IOCTL(225)

#define	IOCTL_LLT_GET_PCB_QSTAT	LLT_IOCTL(226)
#define	IOCTL_LLT_GET_NCB_QSTAT	LLT_IOCTL(227)
#define IOCTL_LLT_SETMSG	LLT_IOCTL(228)
#define IOCTL_LLT_SETMSGGRP	LLT_IOCTL(229)

#else /* LLTUNIX */

#define IOCTL_LLT_GETAPI	_IOR('l', 1, llt_api_t)
#define IOCTL_LLT_ATTACH	_IOWR('l', 2, llt_attach_t)
#define IOCTL_LLT_DETACH	_IOW('l', 3, llt_detach_t)
#define IOCTL_LLT_REINIT	_IO('l', 4)
#define IOCTL_LLT_USER		_IOWR('l', 5, llt_user_t)
#define IOCTL_LLT_GETSTATE	_IOWR('l', 6, uint_t)
#define IOCTL_LLT_SETSTATE	_IOW('l', 7, uint_t)
#define IOCTL_LLT_SETPORT	_IOW('l', 8, llt_setport_t)
#define IOCTL_LLT_GETPARAM	_IOWR('l', 9, llt_param_t)
#define IOCTL_LLT_SETPARAM	_IOW('l', 10, llt_param_t)
#define IOCTL_LLT_SETNODERANGE	_IOW('l', 11, llt_noderange_t)
#define IOCTL_LLT_GETSTATS	_IOR('l', 12, llt_netstat_t)
#define IOCTL_LLT_CLRSTATS	_IO('l', 13)
#define IOCTL_LLT_GETNODEINFO	_IOWR('l', 14, llt_nodeinfo_t)
#define IOCTL_LLT_GETPORTINFO	_IOWR('l', 15, llt_portinfo_t)
#define IOCTL_LLT_GETLINKINFO	_IOWR('l', 16, llt_linkinfo_t)
#define IOCTL_LLT_SETLINKINFO	_IOW('l', 17, llt_linkinfo_t)
#define IOCTL_LLT_GETNODEADDR	_IOWR('l', 18, llt_naddr_buf_t)
#define IOCTL_LLT_SETNODEADDR	_IOW('l', 19, llt_naddr_buf_t)
#define IOCTL_LLT_DELNODEADDR	_IOR('l', 20, llt_nodeaddr_t)
#define IOCTL_LLT_FLUSHNODEADDR	_IO('l', 21)
#define IOCTL_LLT_GETMEMSTAT	_IOR('l', 22, llt_memstat_t)
#define IOCTL_LLT_DBGPRINT	_IOW('l', 23, llt_dbgprint_t)
#define IOCTL_LLT_DBGINFO	_IOWR('l', 26, llt_dbginfo_t)
#define IOCTL_LLT_GETDBGADDR	_IOW('l', 27, int)
#define IOCTL_LLT_GETDBGDATA	_IOW('l', 28, char *)
#define IOCTL_LLT_GET_PCB_QSTAT	_IOR('l', 29, llt_pcb_qstat_t)
#define IOCTL_LLT_GET_NCB_QSTAT	_IOR('l', 30, llt_ncb_qstat_t)
#define IOCTL_LLT_GETNODEALL	_IOR('l', 31, char *)
#define IOCTL_LLT_DETACHALL	_IO('l', 32)
#define IOCTL_LLT_SETMSG	_IOW('l', 33, llt_setmsg_t)
#define IOCTL_LLT_SETMSGGRP	_IOW('l', 34, int)
#define IOCTL_LLT_GETKTUNE	_IOR('l', 36, llt_ktune_t)
#define IOCTL_LLT_GETSYMADDR	_IOWR('l', 37, llt_symaddr_t)
#define IOCTL_LLT_GETKDATA	_IOW('l', 38, llt_kdata_t)

#endif /* LLTUNIX */

/*
 * parameter codes for GETPARAM/SETPARAM
 */
#define LP_MYNODE	1	/* rw local nid */
#define LP_MYCLUSTER	2	/* rw local cid */
#define LP_ISCONFIG	3	/* ro is local nid set? */
#define LP_VERSION_MAJ	4	/* ro major version number */
#define LP_VERSION_MIN	5	/* ro minor version number */
#define LP_HEADERSIZE	6	/* ro length of an LLT packet header */
#define LP_ATOMICMTU	7	/* ro max data length for single packet */

#define LP_LOWNODE	10	/* ro lowest valid nid */
#define LP_HIGHNODE	11	/* ro highest valid nid */
#define LP_MAXNODE	12	/* ro maximum possible nid */
#define LP_HIGHPORT	13	/* ro highest registered port */
#define LP_NUMPORT	14	/* ro count of current registered ports */
#define LP_MAXPORT	15	/* ro maximum possible port */
#define LP_MAXLINKS	16	/* ro maximum possible links [0..MAXLINKS-1] */
#define LP_NUMLINK	17	/* ro count of current links */
#define LP_MAXMSGSIZE	18	/* ro maximum message size allowed */
#define LP_BUFSZ	19	/* rw size of trace buffer (in KB) */

#define LP_HEARTBEAT	20	/* rw heartbeat timer (hipri links) */
#define LP_PEERINACT	21	/* rw peer inactivity timer */
#define LP_OOS		22	/* rw out-of-sequence timer */
#define LP_RETRANS	23	/* rw retransmit timer */
#define LP_SERVICE	24	/* rw service timer */
#define LP_HEARTBEATLO	25	/* rw heartbeat timer (lowpri links) */
#define LP_ARPEXP	26	/* rw ARP expiration timer */
#define LP_ARPSEC	27	/* rw ARPREQ expiration timer */
#define LP_NUMRESPHB	28	/* rw count of resp hbs */
#define LP_SENDHB	29	/* rw sendhb expiration timer */

#define LP_LOWAT	30	/* rw flow control low water mark */
#define LP_HIWAT	31	/* rw flow control high water mark */
#define LP_WIN		32	/* rw flow control window */
#ifdef AIX
#define LP_DLPILOWAT	33	/* rw dlpi flow control low water mark */
#define LP_DLPIHIWAT	34	/* rw dlpi flow control high water mark */
#endif
#define LP_LINKBURST	35	/* rw link burst size */
#ifdef AIX
#define LP_UDPSNDBUF 	36      /* udp send buffer size*/
#define LP_UDPRCVBUF 	37      /* udp recieve buffer size*/
#endif
#define LP_XPORTLOWAT	38	/* rw port xmit flow control low water mark */
#define LP_XPORTHIWAT	39	/* rw port xmit flow control high water mark */
#define LP_RPORTLOWAT	40	/* rw port recv flow control low water mark */
#define LP_RPORTHIWAT	41	/* rw port recv flow control high water mark */

#define LP_NOPULLUP	45	/* rw set state packet pullup */
#define LP_DOORBELL	46	/* rw set state of DOORBELL upcalls */
#define LP_USEARP	47	/* rw set state of bcast ARP */
#define LP_BCASTHB	48	/* rw set bcast vs individual heartbeats */

#define LP_DEBUG	50	/* rw debug level */
#define LP_WARN		51	/* rw control warnings */
#define LP_XDROP	53	/* rw debug drop outgoing pkts */
#define LP_DBG_MINLINKS	54	/* rw reserved */
#define LP_CHECKSUM	55	/* rw set checksums */
#define LP_DBG_CKSM1	56	/* rw checksum test 1 */
#define LP_DBG_CKSM2	57	/* rw checksum test 2 */

#define LP_ACKVAL	58	/* rw set ack threshold */
#define LP_SWS		59	/* rw set sws */

#define LP_PEERTROUBLE	60	/* rw link inactivity */

#define LP_TRACELEVEL	61	/* rw trace level */

#define LP_STRICTSRC	62	/* rw check src address? */

#define LP_DLVIRQ	63	/* rw allow pkt delivery in interrupt ctxt */

#define LP_MAXVERSION_MAJ	64	/* ro maximum capable major number */
#define LP_MAXVERSION_MIN	65	/* ro maximum capable minor number */

#define LP_TIMETOREQHB	66	/* rw req hb from peer after */
#define LP_REQHBTIME	67	/* rw req hb from peer every */

#define LP_LARGEPKTLEN	71	/* rw large pkt size */
#define LP_RETRANSX	72	/* rw retransmit factor */

#define LP_SENDHBCAP	73	/* rw cap to send hbs outside timer context */

#define LP_MAXTHREAD	74	/* rw max threads */

#define LP_QENABLE_FREQ	75


/* parameter for IOCTL_LLT_GETPARAM, IOCTL_LLT_SETPARAM */
typedef struct llt_param_s {
	uint_t		lp_param;
	uint_t		lp_value;
} llt_param_t;


/* parameter for IOCTL_LLT_GETPORTINFO */
typedef struct llt_portinfo_s {
	ushort_t	pi_port;
	uchar_t		pi_registered;
	uchar_t		pi_local_xoff;
	uchar_t		pi_qfull;
	uint_t		pi_qcount;
	uint_t		pi_cookie;
	uint_t		pi_numnodes;
	llt_trafstat_t	pi_txstat;
	uint_t		pi_xmitcnt;
	uint_t		pi_xmitbytes;
	uint_t		pi_txlatcur;
	uint_t		pi_txlatency[LLT_MAXPKTS2TRACK];
	uint_t		pi_txlatdist[LLT_MAXLATBUCKETS];
	llt_trafstat_t	pi_rxstat;
	uint_t		pi_recvcnt;
	uint_t		pi_recvbytes;
	uint_t		pi_rxlatcur;
	uint_t		pi_rxlatency[LLT_MAXPKTS2TRACK];
	uint_t		pi_rxlatdist[LLT_MAXLATBUCKETS];
	uchar_t		pi_usage[LLT_MAX_USAGE_LEN];
	uint_t		pi_buflen;
	uchar_t		pi_nodeinfo[1];	/* one for each peer node */
	/* pi_nodeinfo flags format: */
#define LLT_CONNECTED	0x01
#define LLT_CLOSING	0x02
#define LLT_PEER_XOFF	0x04
#define LLT_OPENING	0x08
} llt_portinfo_t;


/* parameter for IOCTL_LLT_GETNODEINFO ( & IOCTL_LLT_GETNODEALL ) */
typedef struct llt_nodeinfo_s {
	uint_t		ni_nid;
	uint_t		ni_state;
	uint_t		ni_active_links;
	uint_t		ni_num_links;	/* min(LLT_MAX_LINKS, ni_buflen) */
	uint_t		ni_buflen;
	uchar_t		ni_linkstatus[1]; /* one entry for each ni_num_links */
#define LLT_LINK_DOWN		0 /* link peerinact timer expired */
#define LLT_LINK_UP		1 /* link is receiving pkts from peer node */
#define LLT_LINK_TROUBLE	2 /* no pkts received since llt_peertrouble but 
				   * peerinact timer not expired yet.
				   * note: we report links in such state as
 				   *       LLT_LINK_UP to userspace 
				   */
} llt_nodeinfo_t;

typedef struct llt_linkstat_s {
	uchar_t		ls_linkstatus[LLT_MAX_LINKS];
} llt_linkstat_t;

typedef struct llt_nodeall_s {
	uchar_t		*na_buf;
} llt_nodeall_t;

/* parameter for IOCTL_LLT_GETLINKINFO, IOCTL_LLT_SETLINKINFO */
typedef struct llt_linkinfo_s {
	uint_t		ll_linkno;
	uint_t		ll_flags;	/* fields to set */
#define LLT_LINKSTATE	0x01
#define LLT_LINKPRI	0x02
#define LLT_LINKBCAST	0x04
	uint_t		ll_valid;	/* ro */
	uint_t		ll_state;	/* rw */
#define LLT_DISABLE	0x00
#define LLT_RECV_ENABLE	0x01
#define LLT_XMIT_ENABLE	0x02
	uint_t		ll_pri;		/* rw */
#define LLT_HIPRI	0
#define LLT_LOWPRI	1
	uint_t		ll_linktype;		/* ro */
	uint_t		ll_mtu;			/* ro */
	ushort_t	ll_sap;			/* ro */
	uint_t		ll_sapfirst;		/* ro */
	uint_t		ll_addrlen;		/* ro */
	uint_t		ll_saplen;		/* ro */
	uint_t		ll_xmitcnt;		/* ro */
	uint_t		ll_xmitbytes;		/* ro */
	uint_t		ll_recvcnt;		/* ro */
	uint_t		ll_recvcnt_ether;	/* ro */
	uint_t		ll_recvbytes;		/* ro */
	uint_t		ll_errcnt;		/* ro */
	uint_t		ll_puthbcnt;		/* ro */
	uint_t		ll_badcksum;		/* ro */
	llt_trafstat_t	ll_txstat;		/* ro */
	llt_trafstat_t	ll_rxstat;		/* ro */
	uchar_t		ll_bcast[LLT_MAX_MAC_LEN];	/* rw */
	uchar_t		ll_tag[LLT_MAX_TAG_LEN];	/* ro */

#if defined(LLTUNIX)
	uint_t		ll_muxid;			/* ro */
#elif defined(LINUX)
	uchar_t ll_devname[LLT_MAX_DEVNAME_LEN];	/* ro */
#endif

} llt_linkinfo_t;

/*
 * parameter for IOCTL_LLT_GETNODEADDR, IOCTL_LLT_SETNODEADDR,
 * IOCTL_LLT_DELNODEADDR
 */
typedef struct llt_nodeaddr_s {
	uint_t		ln_nid;		/* nid+linkno define unique addr */
	uint_t		ln_linkno;	/* get linkno first by matching tags */
	uint_t		ln_perm;	/* is address permanent? */
	uint_t		ln_addrlen;	/* (out) length of actual address */
	uint_t		ln_buflen;	/* (in) length of address buffer */
	uchar_t		ln_addr[1];	/* Stores hardware address */
} llt_nodeaddr_t;


/* parameter for IOCTL_LLT_SETNODERANGE */
typedef struct llt_noderange_s {
	uint_t	nr_mode;
#define	LLT_INCLUDE	0
#define	LLT_EXCLUDE	1
	uint_t		nr_min;
	uint_t		nr_max;
} llt_noderange_t;

/* parameter for IOCTL_LLT_SETMSG */
typedef struct llt_msg_s {
	uint_t		lm_min;
	uint_t		lm_max;
	uint_t		lm_val;	
} llt_setmsg_t;

/* parameter for IOCTL_LLT_GETKTUNE */
typedef struct llt_ktune_s {
	uint_t		lk_maxnodes;
	uint_t		lk_maxports;
	uint_t		lk_nominpad;
	uint_t		lk_basetimer;

#ifdef SOLARIS
	uint_t		lk_rthreadpri;
#endif

#ifdef LINUX
	uint_t		lk_nqthread;
#endif

} llt_ktune_t;

#ifdef LINUX

/* parameter for IOCTL_LLT_GETSYMADDR */
typedef struct llt_symaddr_s {
	char		ls_symstr[LLT_MAX_SYMNAME_LEN];	/* symbol string (in) */
	uchar_t		*ls_addr;			/* address (out) */
} llt_symaddr_t;

/* parameter for IOCTL_LLT_GETKDATA */
typedef struct llt_kdata_s {
	uchar_t		*lk_srcaddr;	/* kernel addr */
	uint_t		lk_len;		/* # bytes */
	uchar_t		*lk_dstaddr;	/* userspace destination */
} llt_kdata_t;

#endif	/* LINUX */

/*
 * Networking statistics.
 * r_* refer to receive stats, x_* to xmit stats.
 *
 * this is a per-cpu structure in the driver.
 * (well actually a cache-aligned union defined below is per-cpu)
 *
 * userspace sees cumulative counts (via IOCTL_LLT_GETSTATS).
 */
typedef struct llt_netstat_s {
	uint_t	r_destdrops;	/* destid doesn't match our nid */
	uint_t	r_srcdrops;	/* src node id is out of range */
	uint_t	r_unconfdrops;	/* src node is not configured */
	uint_t	r_badgendrops;	/* connection generation num mismatch */
	uint_t	r_noconndrops;	/* node is not in connected state */
	uint_t	x_noconndrops;	/* node is not in connected state */
	uint_t	r_oowdrops;	/* out-of-window */
	uint_t	r_oospkts;	/* out-of-sequence packets */
	uint_t	r_dupdrops;	/* recv'd duplicate packets */
	uint_t	x_nobufs;	/* no mem for bufs to xmit */
	uint_t	x_retrans;	/* number of re-transmissions */
	uint_t	x_userpkts;	/* number of user pkts sent */
	uint_t	x_connpkts;	/* number of CR/CC/DR/DC pkts sent */
	uint_t	x_piggyacks;	/* number of piggybacked ACKs sent */
	uint_t	x_piggynaks;	/* number of piggybacked NAKs sent */
	uint_t	x_ackpkts;	/* number of ACK packets sent */
	uint_t	x_nakpkts;	/* number of NAK packets sent */
	uint_t	r_sapdrops;	/* rcv'd SAP is not ours */
	uint_t	r_streambad;	/* rcv'd unexpected stream primitive */
	uint_t	r_dlpibad;	/* rcv'd unexpected DLPI primitive */
	uint_t	r_dlpierr;	/* rcv'd DLPI error_ack primitive */
	uint_t	r_malformed;	/* rcv'd malformed packet */
	uint_t	r_userpkts;	/* number of user pkts received */
	uint_t	r_dgramdrop;	/* received datagrams dropped */
	uint_t	x_drops;	/* xmit drops, cant putnext to driver */
	uint_t	x_looppkts;	/* xmit loopback packets */
	uint_t	x_nolink;	/* no link to xmit packets (timed out) */
	uint_t	r_misalign;	/* Unaligned packets received */
	uint_t	r_multiblock;	/* Data received in multiple mblks */
	uint_t	r_nobufs;	/* no mem for bufs to recv */
	uint_t	r_dlpinolink;	/* no link to xmit packets (from DLPI) */
	uint_t	r_checksum;	/* bad packet checksum, dropped */
	uint_t	x_chainhdr;	/* xmit seperate header chained to payload */
	uint_t	r_badsrcaddr;	/* bad src (ether/udp) address */
	uint_t	r_oopd;		/* rcv packet with OOP data (HP) */

#ifdef LINUX
	uint_t  x_udpdrops;	/* udp xmit queue full, dropped */
#endif

} llt_netstat_t;

/*
 * error codes
 */
#ifdef LLTUNIX

#define LLT_SUCCESS	0
#define LLTERR_INVAL	EINVAL
#define LLTERR_BUSY	EBUSY
#define LLTERR_NOSUCH	ENXIO
#define LLTERR_SIZE	EMSGSIZE
#define LLTERR_CONN	ENOTCONN
#define LLTERR_NOTREG	ENOENT
#define LLTERR_ALREADY	EALREADY
#define LLTERR_IO	EIO
#define LLTERR_MEM	ENOSR
#define LLTERR_RDONLY	EROFS
#define LLTERR_FAULT	EFAULT
#define LLTERR_PERM	EPERM
#define LLTERR_FILE	EBADF
#define LLTERR_NOFILE	EMFILE
#define LLTERR_EMLINK	EMLINK  	
#else	/* LLTUNIX */

#define LLT_SUCCESS	0
#define LLTERR_INVAL	-EINVAL
#define LLTERR_BUSY	-EBUSY
#define LLTERR_NOSUCH	-ENXIO
#define LLTERR_SIZE	-EMSGSIZE
#define LLTERR_CONN	-ENOTCONN
#define LLTERR_NOTREG	-ENOENT
#define LLTERR_ALREADY	-EALREADY
#define LLTERR_IO	-EIO
#define LLTERR_MEM	-ENOSR
#define LLTERR_RDONLY	-EROFS
#define LLTERR_FAULT	-EFAULT
#define LLTERR_PERM	-EPERM
#define LLTERR_FILE	-EBADF
#define LLTERR_NOFILE	-EMFILE
#define LLTERR_EMLINK	-EMLINK

#endif	/* LLTUNIX */

#if defined(_KERNEL) || defined(_KMEMUSER) /* { */

/*
 * parameter for llt_register() callbacks
 */
typedef struct llt_funcs_s {
	int	(*recv_func)(void *, uint_t, llt_mblk_t *);
	void	(*backenable_func)(void *, uint_t);
	int	(*canput_func)(void *);
	void	(*connect_func)(void *, uint_t, int, int);
} llt_funcs_t;


#endif /* _KERNEL || _KMEMUSER } */

#ifdef _KERNEL /* { */

typedef struct llt_api {
#ifndef LINUX
	void	*lltapi_devinfo;
#endif
	int 	(*lltapi_register)(uint_t, void *, llt_funcs_t *, uchar_t *);
	int	(*lltapi_unregister)(uint_t);
	int 	(*lltapi_opennode)(uint_t, uint_t);
	int 	(*lltapi_closenode)(uint_t, uint_t, int);
	llt_mblk_t *(*lltapi_msgalloc)(uint_t, void (*)(void));
	int	(*lltapi_canput)(uint_t, uint_t);
	int	(*lltapi_send_data)(uint_t, uint_t, llt_mblk_t *);
	void	(*lltapi_backenable)(uint_t);
	int	(*lltapi_getstate)(uint_t *);
	int	(*lltapi_setstate)(uint_t);
	int	(*lltapi_getparam)(llt_param_t *);
	int	(*lltapi_setparam)(llt_param_t *);
	int	(*lltapi_getstats)(llt_netstat_t *);
	int	(*lltapi_clrstats)(void);
	int	(*lltapi_getportinfo)(llt_portinfo_t *);
	int	(*lltapi_getnodeinfo)(llt_nodeinfo_t *);
	int	(*lltapi_getlinkinfo)(llt_linkinfo_t *);
	int	(*lltapi_setlinkinfo)(llt_linkinfo_t *);
	int	(*lltapi_getnodeaddr)(llt_nodeaddr_t *);
	int	(*lltapi_setnodeaddr)(llt_nodeaddr_t *);
	int	(*lltapi_delnodeaddr)(llt_nodeaddr_t *);
	int	(*lltapi_flushnodeaddr)(void);
	int	(*lltapi_setnoderange)(llt_noderange_t *);
	int	(*lltapi_send_dgram)(uint_t, uint_t, llt_mblk_t *);
	int	(*lltapi_send_bcast)(uint_t, llt_mblk_t *);
	void	(*lltapi_hold)(void);
	void	(*lltapi_release)(void);
	void	(*lltapi_dumphdr)(llt_mblk_t *);
#ifdef LINUX
	void	(*lltapi_msgfree)(struct sk_buff *);
#endif
} llt_api_t;


#if defined(SOLARIS) /* PLATFORM  { */

#define LLT_API_INIT(api) { 						\
	dev_info_t *dp; 						\
	(api) = NULL;		     					\
        if (ddi_pathname_to_dev_t("/pseudo/clone@0:llt") == (dev_t)-1) {\
		cmn_err(CE_CONT, "LLT:10069: llt not loaded\n");	\
        } else if ((dp = ddi_find_devinfo(LLT_NAME, -1, 0)) == NULL) {  \
                cmn_err(CE_CONT, "LLT:10070: llt not initialized\n");	\
        } else {     							\
		(api) = (struct llt_api *)ddi_get_driver_private(dp);	\
		if (api) {						\
			((api)->lltapi_hold)();				\
		}							\
	}     								\
}

#define LLT_API_DEINIT(api) 						\
	((api)->lltapi_release)();					\
	(api) = NULL;

#elif defined(HPUX)

#define LLT_API_INIT(api) {						\
	Elf64_Sym sym;							\
	(api) = NULL;							\
	if (mod_obj_getsymp((const char *)"llt_api_local", NULL, 0,	\
		&sym) == 0) {						\
		(api) = (llt_api_t *)sym.st_value;			\
	}								\
}

#define LLT_API_DEINIT(api) (api) = NULL

#elif defined(AIX)

#ifndef NO_VEKI

typedef struct llt_veki_api {
	ki_t		llt_ki;
	struct llt_api 	*llt_api;
} llt_veki_api_t;

#define LLTAPI_NAME	"veritas.llt"
#define LLTAPI_VERSION	"1"

#define LLT_API_INIT(api)
#define LLT_API_DEINIT(api)

#else 

extern struct llt_api llt_api_local;
#define LLT_API_INIT(api) (api) = &llt_api_local
#define LLT_API_DEINIT(api) (api) = NULL

#endif /* NO_VEKI */

#elif defined(LINUX)

#include <linux/version.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18))
extern void *llt_imc_import(char *, char *);
#define LLT_API_INIT(api) {						\
	(api) = NULL;							\
	if (!(api = (llt_api_t *)llt_imc_import("lltapi", "llt"))){\
		printk("LLT: LLT is not loaded\n");			\
	}								\
}
#else
#define LLT_API_INIT(api) {						\
        (api) = NULL;                                                   \
        if (!(api = (llt_api_t *)inter_module_get_request("lltapi", "llt"))){\
                printk("LLT: LLT is not loaded\n");                     \
        }                                                               \
}
#endif

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18))
extern void llt_imc_deport(char *);
#define LLT_API_DEINIT(api)	((api) = NULL, llt_imc_deport("lltapi"))
#else
#define LLT_API_DEINIT(api) 	((api) = NULL, inter_module_put("lltapi"))
#endif

#else

ERROR - unknown platform

#endif /* PLATFORM  } */


#define LLT_API_REGISTER(api, port, cookie, funcs, usage) \
	((api)->lltapi_register((port), (cookie), (funcs), (usage)))

#define LLT_API_UNREGISTER(api, port) \
	((api)->lltapi_unregister(port))

#define LLT_API_OPENNODE(api, port, node) \
	((api)->lltapi_opennode((port), (node)))

#define LLT_API_CLOSENODE(api, port, node, reason) \
	((api)->lltapi_closenode((port), (node), (reason)))

#define LLT_API_MSGALLOC(api, size, freefn) \
	((api)->lltapi_msgalloc((size), (freefn)))

#ifdef LINUX
#define LLT_API_MSGFREE(api, msg) \
	((api)->lltapi_msgfree(msg))
#else	/* LINUX */
#ifdef LLT_DEBUG
#define LLT_API_MSGFREE(api, msg)	{				\
						msg->b_prev = NULL;	\
						msg->b_next = NULL;	\
						freemsg(msg);		\
					}
#else	/* LLT_DEBUG */
#define LLT_API_MSGFREE(api, msg)	freemsg(msg)
#endif	/* LLT_DEBUG */
#endif	/* LINUX */

#define LLT_API_CANPUT(api, port, node) \
	((api)->lltapi_canput((port), (node)))

#define LLT_API_SEND_DATA(api, port, node, msg) \
	((api)->lltapi_send_data((port), (node), (msg)))

#define LLT_API_BACKENABLE(api, port) \
	((api)->lltapi_backenable(port))

#define LLT_API_GETSTATE(api, state) \
	((api)->lltapi_getstate(state))

#define LLT_API_SETSTATE(api, state) \
	((api)->lltapi_setstate(state))

#define LLT_API_GETPARAM(api, param) \
	((api)->lltapi_getparam(param))

#define LLT_API_SETPARAM(api, param) \
	((api)->lltapi_setparam(param))

#define LLT_API_GETSTATS(api, stats) \
	((api)->lltapi_getstats(stats))

#define LLT_API_CLRSTATS(api) \
	((api)->lltapi_clrstats())

#define LLT_API_GETPORTINFO(api, info) \
	((api)->lltapi_getportinfo(info))

#define LLT_API_GETNODEINFO(api, info) \
	((api)->lltapi_getnodeinfo(info))

#define LLT_API_GETLINKINFO(api, info) \
	((api)->lltapi_getlinkinfo(info))

#define LLT_API_SETLINKINFO(api, info) \
	((api)->lltapi_setlinkinfo(info))

#define LLT_API_GETNODEADDR(api, addr) \
	((api)->lltapi_getnodeaddr(addr))

#define LLT_API_SETNODEADDR(api, addr) \
	((api)->lltapi_setnodeaddr(addr))

#define LLT_API_DELNODEADDR(api, addr) \
	((api)->lltapi_delnodeaddr(addr))

#define LLT_API_FLUSHNODEADDR(api) \
	((api)->lltapi_flushnodeaddr())

#define LLT_API_SETNODERANGE(api, range) \
	((api)->lltapi_setnoderange(range))

#define LLT_API_SEND_DGRAM(api, port, node, msg) \
	((api)->lltapi_send_dgram((port), (node), (msg)))

#define LLT_API_SEND_BCAST(api, port, msg) \
	((api)->lltapi_send_bcast((port), (msg)))

#define LLT_API_DUMPHDR(api, msg) \
	((api)->lltapi_dumphdr(msg))

#endif /* _KERNEL } */

#endif	/* _LLTAPI_H */
